// BUS.CPP 
//
// Implementación del bus que comunica planta con controlador. Dos buses conectados han de tener la misma ID.
//
// El bus maneja los paquetes de datos con asignación dinámica de memoria. No tiene en cuenta los datos internos 
// de cada paquete sino su ID y longitud
//
//

#include "base.h" // declaración de la clase base

#include <iostream> // Para salida por pantalla
//#include <limits>

#include <sys/ipc.h>
#include <sys/msg.h> // Emplearemos la cola de mensajes estándar System V para comunicar planta con controlador

#define DESPLAZ_ID 10000 // Desplazamiento de id entre planta y controlador que garantice unicidad en la comunicación

Bus::Bus(int id, tipo_proceso tipo) : Dispositivo(0,id) { 
										 // Constructor para el bus
			
	Bus::tipo = tipo; // Guardamos el tipo de bus (controlador o planta). 
	
	Bus::clave_cola = ftok(".", id); // Asignamos la clave de la cola con la que conectar al otro bus
	
	if(Bus::clave_cola == -1) { // Error al obtener id		
		std::cerr << "ERROR: No se pudo obtener id de la cola de mensajes del bus";		
		return; 
	}
	
	Bus::id_cola = msgget(Bus::clave_cola, IPC_CREAT | 0660); // Creamos cola
	
	if(Bus::id_cola == -1) { // Error al crear la cola		
		std::cerr << "ERROR: No se pudo crear la cola de mensajes del bus";		
		return; 
	}

	
	Bus::conexion = true; //Hay posibilidad de conexion. 
																  
}

int Bus::registrar(Discretizador *discretizador, BYTE *al_bus_c, BYTE *al_dispositivo_c) { 
					//Se pasa puntero al objeto discretizador y zonas de datos
					
	unsigned int i = 0; // Indice
	int err = 0; // error
			
	plantilla_datos_discretizador_sv parametros; // Estructura de datos del discretizador
	plantilla_datos_discretizador_sv parametros_otro; // Estructura de datos del discretizador_otro (vacío)
	
	plantilla_operacion_bus operacion; // Rellenamos previo de operación en el bus
	
	parametros = discretizador->obtiene_parametros(); // Obtenemos los parametros
	
	for(i=0 ; i < Bus::parametros.size() ; i++) // Comprobamos si el discretizador hizo un registro previo
		if(Bus::parametros[i].id == parametros.id) {
			err = 1; // Hubo registro previo. No se permite duplicación.
					 // Aunque se podrá sobreescribir
			break;   // Queda disponible el indice
	}
	parametros_otro = {0,0,0,0,0,0,0,0,false}; // En blanco (ceros)
	
	Bus::exclusion_bus.lock(); // no se permite concurrencia en esta sección, pues modificamos las listas	
	
	if (err == 0) { // Primera vez
		Bus::parametros.push_back(parametros); // Se copia toda la estructura de parámetros al vector
		Bus::parametros_otro.push_back(parametros_otro); // Se crea el vector de parámetros del discretizador remoto
	} else { // Sucesivas veces 		
		Bus::parametros[i]=parametros; // Se sobreescribe
		Bus::parametros_otro[i]=parametros_otro; 
		
	}
	
	operacion.enviar_cabecera = true; operacion.recibir_cabecera = true; // Habilitamos para enviar y recibir cabecera
	
	operacion.recibida_cabecera = false; // No se recibió ninguna cabecera
	operacion.recibida_cabecera_otro = false; 
	
	operacion.incompatible = false; // No hubo incompatibilidad aún
	operacion.difieren_colas = false ; operacion.difieren_fmuestreo = false; // No difiere nada aún	
	operacion.conteo = 0; operacion.conteo_otro = 0; 
	
	operacion.al_bus_c = al_bus_c;							// Ubicamos punteros
	operacion.al_dispositivo_c = al_dispositivo_c;
	
	if (err == 0)  // Primera vez
		Bus::operacion.push_back(operacion); // Se registra estructura de operación
	else // Sucesivas veces
		Bus::operacion[i]=operacion; // Se sobreescribe
	
	Bus::exclusion_bus.unlock(); // Finaliza zona de concurrencia. Registro completo. 
	
	return err;

}

unsigned int Bus::busca(int id) { // Buscar posición en el vector del dispositivo id. (en primer elemento devuelve 1)
								// devuelve 0 si no encontró nada
								// ATENCION: esta rutina ha de estar en zona de exclusión mutua cuando se llama
								// desde el método principal lecturaescritura
	unsigned int i;
	
	
	
	for (i=0;i < Bus::parametros.size() ; i ++) {
			if(Bus::parametros[i].id == id) return i+1; // se encuentra coincidencia, salimos con i+1
		
	}

	return 0; // no hubo coincidencia, salimos con 0

}

int Bus::lecturaescritura(Discretizador *discretizador, bool solo_lectura, int *err_es) {
												// Lectura/escritura en el bus. Protocolo y operación 
												// mediante colas y cabecera previa. Se devuelve error si hubiera
												// NOTA DE SEGURIDAD: se envían punteros genéricos a las memorias
												// de los discretizadores. El motivo de esta implementación es
												// la diversidad de estructuras de datos que se pueden manejar, por 
												// lo que se necesita posicionamiento genérico sin tamaño fijo
												
												// Con el flag solo_lectura, no enviará al otro bus. Como en el 
												// caso de los conectores, con ello se garantiza que se enviarán
												// datos válidos. 
				// HANDSHAKING
				// Nota: La cabecera se envía dos veces. Y el proceso se pone a transmitir datos cuando sabe
				// que el otro proceso recibió la cabecera y se preparó para la recepción. 
												
	unsigned int indice; // posición en los vectores de los datos de este discretizador	
	int id = 0; 
	long id_local = 0, id_remoto = 0;
	int err = 0, err2 = 0; //ids, discretizador y codigos de retorno
	int err3 = 0; // envío / recepción con éxito, código. 
	
	BYTE *a_bus, *a_dispositivo; // Punteros genéricos (son estructurados) 
	
	// Gestionamos las cabeceras para las colas de mensajes
	cabecera_sv_conteo *cabecera_a_bus; 
	cabecera_sv_conteo *cabecera_a_dispositivo;
	
	struct primero { // Primer mensaje (de tamaño fijo) que enviamos/recibimos, y que consiste en los parametros del 
		long mtype;   															// discretizador
		plantilla_datos_discretizador_sv parametros;		
	};
	
	primero primer_dato;
	
	if(Bus::conexion == false) return -1; // No hay conexion. Error -1. GRAVE (Esto activará alarma en el discretizador)	

	Bus::exclusion_bus.lock(); // no se permite concurrencia en esta sección
			
	id = discretizador->obtiene_id(); // obtenemos el id del discretizador
	
	switch(Bus::tipo) { // Según el tipo de proceso, modificamos id de envío y recepción (no pueden coincidir)
		case PROCESO_CONTROLADOR: // Los hacemos únicos según el id y el proceso
			id_local = (long)id;
			id_remoto = ((long)id) + DESPLAZ_ID; // Nota: los id no pueden ser mayores o iguales de DESPLAZ_ID
			break;
		case PROCESO_PLANTA:
			id_local = ((long)id) + DESPLAZ_ID;
			id_remoto = (long)id;
			break;
	};
	
	indice = Bus::busca(id); // ¿está el discretizador en la lista del bus?
	
	if (indice == 0) {
		Bus::exclusion_bus.unlock(); // Fin de sección crítica
		return -2; // Error -2. No está registrado el discretizador.GRAVE (Esto activará alarma en el discretizador)
	}
	indice--; // El primer elemento es cero. Trabajamos con este indice en los vectores de datos.	
	
	a_bus = Bus::operacion[indice].al_bus_c; // Obtenemos los punteros a las zonas de memoria del discretizador 
	a_dispositivo = Bus::operacion[indice].al_dispositivo_c;
	
	cabecera_a_bus = (cabecera_sv_conteo *)a_bus; // Punteros estructurados según cabecera 
	cabecera_a_dispositivo = (cabecera_sv_conteo *)a_dispositivo;	
	
	if (Bus::operacion[indice].enviar_cabecera) { // Hemos de enviar la cabecera -----------------------------
		primer_dato.parametros = Bus::parametros[indice]; // obtenemos los parámetros 
		primer_dato.mtype = id_local; // como tipo de dato, el id_local de nuestro discretizador
		err=msgsnd(Bus::id_cola, &primer_dato, sizeof(plantilla_datos_discretizador_sv), IPC_NOWAIT); 
																		// Envío sin bloqueo			
		if (err==-1 ) { // hubo error 
			Bus::exclusion_bus.unlock(); // Fin de sección crítica			
		    return -4; // Error -4. Error de envío de cabecera (visible en propiedades)
		}
		else Bus::operacion[indice].enviar_cabecera = false; 
											// Se envió. A partir de aquí no se envían más cabeceras, a no ser
											// que el otro proceso refleje en sus parámetros que no recibió la cabecera
											// entonces, se reintenta otra vez
	}
	
	if (Bus::operacion[indice].recibir_cabecera) { // Recepción y verificación de cabecera --------------------		 

		err=msgrcv(Bus::id_cola, &primer_dato, sizeof(plantilla_datos_discretizador_sv), id_remoto, IPC_NOWAIT); 
											// recepción sin bloqueo		
		
		// Si no recibió, recibió datos inadecuados, o huellas incorrectas, error -5
		if (err==-1 || err < (int)sizeof(plantilla_datos_discretizador_sv) || 
				primer_dato.parametros.huella1 != HUELLA1 || primer_dato.parametros.huella2 != HUELLA2) { 
			Bus::exclusion_bus.unlock(); // Fin de sección crítica
		    return -5; // Error -5. Error de recepcion de cabecera (visible en propiedades)		
		}
		
		Bus::parametros_otro[indice] = primer_dato.parametros; // en caso de exito guardamos el parametro valido
		
		Bus::operacion[indice].recibida_cabecera = true;
		Bus::parametros[indice].recibido = true; 		
		
		if (!Bus::parametros_otro[indice].recibido) // El otro bus no recibió la cabecera. Hay que repetir envío
			Bus::operacion[indice].enviar_cabecera = true; 
		else { 
				Bus::operacion[indice].enviar_cabecera = false; // En caso contrario, ya no hará falta enviar la cabecera
				Bus::operacion[indice].recibir_cabecera = false;// Ya se estableció el handshaking
				Bus::operacion[indice].recibida_cabecera_otro = true;// Y solo se recibirán datos
																	//	(en caso de compatibilidad)
		}					
		
		// IMPORTANTE: Compatibilidad. Los esid han de ser opuestos, entradas y salidas complementarias. Comprobacion	
		
		if(Bus::parametros[indice].esid + Bus::parametros_otro[indice].esid != 0 || // En caso de incompatibilidad
			Bus::parametros[indice].lal_dispositivo != Bus::parametros_otro[indice].lal_bus || 
					Bus::parametros[indice].lal_bus != Bus::parametros_otro[indice].lal_dispositivo) 			
						Bus::operacion[indice].incompatible = true; // Se activa bandera de incompatibilidad	}	

		// Guardamos id (mtype) en la estructura a bus del discretizador (para envío) 
			cabecera_a_bus -> mtype = id_local; // Queda permanente en todo el uso del bus
		
		// Comparaciones. El discretizador con menor cola tiene preferencia y se cambia la información
		// del que tiene más cola. Se igualan, se marca flag de diferencia (prevalece el discretizador que
		// eligió menor cola y por tanto mayor aproximación al tiempo real. 
		
		if(Bus::parametros[indice].tamano_cola > Bus::parametros_otro[indice].tamano_cola) {
			Bus::parametros[indice].tamano_cola = Bus::parametros_otro[indice].tamano_cola;
			Bus::operacion[indice].difieren_colas = true;			
		}
		
		// Guardamos el tamaño de cola acordado en Bus::conteo, y nos determina el tamaño máximo que usaremos
		// en transmisión/recepción		
		Bus::operacion[indice].conteo = Bus::parametros[indice].tamano_cola;
		
		
		if(Bus::parametros[indice].fmuestreo != Bus::parametros_otro[indice].fmuestreo)
			Bus::operacion[indice].difieren_fmuestreo = true; 
											// Si difieren muestreos, habrá asimetría evidente en las colas
											// Se activa esta bandera	

	}
	
	if (Bus::operacion[indice].incompatible) {
		Bus::exclusion_bus.unlock(); // Fin de sección crítica	
		return -3; // Error -3. Se ha constatado una incompatibilidad con este discretizador y el del otro lado. GRAVE
	}   // (Esto activará alarma en el discretizador)
	
	// Estamos en condiciones de utilizar las colas de datos (Paso 1: enviar; paso 2: recibir)
	
	
	
	// Si las cabeceras se recibieron por ambas partes, se realiza el intercambio de datos E/S
	if(Bus::operacion[indice].recibida_cabecera && Bus::operacion[indice].recibida_cabecera_otro) {
		
		if (Bus::operacion[indice].conteo > 0 && !solo_lectura) { 
									// ¿Podemos enviar datos? entonces los enviamos a la cola de mensajería
			cabecera_a_bus -> conteo_cola = Bus::operacion[indice].conteo--; 
			cabecera_a_bus -> estado = BUS_NORMAL; // Enviamos un estado normal. 
															// Asignamos conteo al buffer y decrementamos			
			err=msgsnd(Bus::id_cola, a_bus, sizeof(cabecera_sv_conteo)+Bus::parametros[indice].lal_bus - sizeof(long),
			IPC_NOWAIT); // Envío de todo el buffer de datos (cabecera + datos) sin bloqueo
			// Nótese que se envía también el estado de la cola; si es paquete normal o de parada
			err3++; // Para el discretizador que ha llamado.
		}
		
		// Recibimos datos con la id del discretizador, los guardamos en el buffer, actualizamos conteo_otro
		// Aquí no hay restricciones. Es decir, el bus siempre está en disposicion de recibir datos si hay proceso
		// de colas habilitado (situación habitual operativa en el bus). 
		
		err2 = msgrcv(Bus::id_cola, a_dispositivo, sizeof(cabecera_sv_conteo)+Bus::parametros[indice].lal_dispositivo
		- sizeof(long), id_remoto, IPC_NOWAIT); // recepción de todo el buffer de datos (cabecera + datos) sin bloqueo
		
				// OJO: Al ENVIAR y al RECIBIR hay que descartar el envío de long mtype, restando sizeof(long)
				// No contar con este factor puede provocar un desbordamiento de memoria y errores al liberarla
				
				
		if(err2 != -1) { // Fue una recepción válida ///// NO SE COMPRUEBA LONGITUD EFECTIVA				
				
				Bus::operacion[indice].conteo_otro = cabecera_a_dispositivo -> conteo_cola; 
													// Obtenemos conteo cola remota
				if(cabecera_a_dispositivo -> estado == BUS_NORMAL) {  // Se ha recibido un estado normal de transmisión 
															 // Recepción normal. 
					err3 += 2; // Para el discretizador que ha llamado. Exito en lectura
			
					if(Bus::operacion[indice].conteo < Bus::parametros[indice].tamano_cola) 
																// ¿Cola local alcanzó valor máximo? 	
						Bus::operacion[indice].conteo++;	// Si no es así, ampliamos nuestra cola en uno	
					
				} else { // Si el otro bus decidió terminar, se resetean datos y finaliza la transmisión
						
				// Se ha de resetear los datos del discretizador [indice] y esperar a que llegue otra cabecera
				// Estado inicial de las estructuras de operacion y parámetros.
					Bus::parametros[indice].recibido = false;
					Bus::operacion[indice].enviar_cabecera = true; Bus::operacion[indice].recibir_cabecera = true;
					Bus::operacion[indice].recibida_cabecera = false ; 
					Bus::operacion[indice].recibida_cabecera_otro = false; Bus::operacion[indice].difieren_colas = false;
					Bus::operacion[indice].difieren_fmuestreo = false; Bus::operacion[indice].conteo = 0;
					Bus::operacion[indice].conteo_otro = 0;
					Bus::parametros_otro[indice].id = 0; Bus::parametros_otro[indice].esid = 0;
					Bus::parametros_otro[indice].tamano_cola = 0; Bus::parametros_otro[indice].fmuestreo = 0; 
					Bus::parametros_otro[indice].lal_dispositivo = 0; Bus::parametros_otro[indice].lal_bus = 0; 
					Bus::parametros_otro[indice].recibido = false; 
					
				}
				
		}
					
		 
	
	} // Acabó todo el procesamiento de colas.
	

		
	Bus::exclusion_bus.unlock(); // Fin de sección crítica	
	
	if(err == -1 || err2 == -1) *err_es = -6; // Si hubo algún fallo de emisión/recepción, error -6 por argumento  
	
	return err3; // Se devuelve código de retorno positivo (o nulo)
												
}

// El código de retorno en entrada/salida de bus es el mismo que el considerado en el conector.

//	err3 = 1 si se envió correctamente
//	err3 = 2 si se recibió correctamente
//	err3 = 3 si se envió y recibió correctamente
//	
//	Este dato es valioso para el discretizador. Por ejemplo, si se recibió correctamente quiere decir que ya está
//	en disposición de enviar al dispositivo ese dato, o realizar otras acciones que se consideren oportunas. 


Bus::~Bus() { // Destructor del bus. Se cierra cola de mensajes
	
	
	unsigned int i = 0; // Indice
	unsigned int lal_bus = 0; // Longitud de mensaje 
	int id = 0;
	long id_local = 0; // id de dispositivos remotos
	BYTE *buffer;
	
//	int k;
	
	cabecera_sv_conteo *cabecera; // Enviamos una última cabecera, de cierre, a todos los discretizadores del otro lado
								 // De esta forma el otro bus vuelve a su estado inicial de espera
	
	bool destruir = true; // Destruir cola por defecto						 

	
	for (i = 0; i < Bus::operacion.size(); i++) {
		id = Bus::parametros[i].id; // Obtenemos las sucesivas id
		if (Bus::operacion[i].recibida_cabecera && Bus::operacion[i].recibida_cabecera_otro) {
			lal_bus = parametros[i].lal_bus;
			buffer = new BYTE[sizeof(cabecera_sv_conteo)+lal_bus];
			cabecera = (cabecera_sv_conteo *)buffer;
			switch(Bus::tipo) { // Según el tipo de proceso, modificamos id de envío y recepción (no pueden coincidir)
			
				case PROCESO_CONTROLADOR: // Los hacemos únicos según el id y el proceso
					id_local = (long)id; // Nota: los id no pueden ser mayores o iguales que DESPLAZ_ID 
					break;
				case PROCESO_PLANTA:				
					id_local = ((long)id) + DESPLAZ_ID;
					break;		
			};
			cabecera -> mtype = id_local;
			cabecera -> estado = BUS_PARADA; // Enviamos PARADA al otro bus
			destruir = false; // La cola será destruida por el otro proceso cuando se detenga.
			
				msgsnd(Bus::id_cola, buffer, sizeof(cabecera_sv_conteo)+lal_bus- sizeof(long), IPC_NOWAIT); 
																	// Envio
			delete buffer;
		}
//		std::cout << id_remoto <<"\t"<< sizeof(cabecera_sv_conteo) << "\t" << lal_bus << "   ";
//		std::cin >> k;
	}	
	 
 if(destruir) msgctl(Bus::id_cola, IPC_RMID, 0); // Cerrar la cola de mensajes de este lado	si no se envió mensaje final
	
}

void Bus::propiedades() { //Propiedades en pantalla
	
	// Nota sobre propiedades: 
	// En propiedades es preferente usar accesos atómicos std::atomic. Con vectores, o con datos masivos, 
	// un volcado previo de memoria entorpece lo menos posible los hilos que están funcionando, 
	// pues necesitan una gran cantidad de recursos de CPU. 
	
	unsigned int i; // indice
	
	
	enum tipo_proceso tipo; // Tipo de bus que emplearemos (lado del controlador o lado de la planta).
	int id_cola; // Identificación de cola
	bool conexion = false; // ¿hay posibilidad de conexión con el otro bus? 
	
	// Utilizamos una réplica de los datos para así interferir lo menos posible con los procesos en curso.
	// Bloqueamos con mutex, copiamos los datos, liberamos mutex. Y entonces podemos mostrar en pantalla
	// ya que la salida en pantalla suele ser mucho más lenta. 
	
	std::vector<plantilla_datos_discretizador_sv> parametros; // Parametros del discretizador local
	std::vector<plantilla_datos_discretizador_sv> parametros_otro; // Parametros que emite el otro discretizador
	std::vector<plantilla_operacion_bus> operacion; // Operación del discretizador
	
	std::string cadena1, cadena2, cadena3, cadena4, cadena5;	
	
				// strings genéricos
	Bus::exclusion_bus.lock(); // Bloqueamos
		
							   // Copiamos todos los vectores y parámetros
	tipo = Bus::tipo;
	id_cola = Bus::id_cola;
	conexion = Bus::conexion;
	
	parametros = Bus::parametros;
	parametros_otro = Bus::parametros_otro;
	operacion = Bus::operacion;
	
		
	Bus::exclusion_bus.unlock(); // Desbloqueamos	
		
	
	switch (tipo) {
		case PROCESO_CONTROLADOR:
			cadena1 = "Controlador";
			break;
		case PROCESO_PLANTA:
			cadena1 = "Planta";
			break;		
	};
	
	if(conexion) cadena2 = "SI"; else cadena2 = "NO";

	std::cout << "\n\n-------- PROPIEDADES DEL BUS ----------------------------------------\n\n";
	
	std::cout << "Propiedades generales del bus: ";
	std::cout << "\n\nTipo de bus: " << cadena1;
	std::cout << "\tIdentificación de cola:" << id_cola;
	std::cout << "\tEl bus está preparado: " << cadena2;
	
	for(i = 0 ; i < Bus::parametros.size(); i++) {
	
		std::cout << "\n\n<<DISCRETIZADOR " << i + 1 << " >>";
		std::cout << "\n\n\t\tDiscretizador Local 1\t\tDiscretizador Remoto\n";
		std::cout << "\nID:\t\t\t\t" << parametros[i].id << "\t\t\t" << parametros_otro[i].id;
		std::cout << "\nESID:\t\t\t\t" << parametros[i].esid << "\t\t\t" << parametros_otro[i].esid;	
		std::cout << "\nCola:\t\t\t\t" << parametros[i].tamano_cola << "\t\t\t" << parametros_otro[i].tamano_cola;
		std::cout << "\nfMuestreo:\t\t\t" << parametros[i].fmuestreo << "\t\t\t" << parametros_otro[i].fmuestreo;	
		
		std::cout << "\nLdispositivo:\t\t\t"<< parametros[i].lal_dispositivo << "\t\t\t"
									<< parametros_otro[i].lal_dispositivo;
		std::cout << "\nLbus:\t\t\t\t" << parametros[i].lal_bus << "\t\t\t" << parametros_otro[i].lal_bus;
		std::cout << "\nConteo:\t\t\t\t" << operacion[i].conteo << "\t\t\t" << operacion[i].conteo_otro;
		
		if(operacion[i].recibida_cabecera) cadena1 = "SI"; else cadena1 = "NO";
		if(operacion[i].recibida_cabecera_otro) cadena2 = "SI"; else cadena2 = "NO";
		
		std::cout << "\nCabec.Recibida:\t\t\t" << cadena1 << "\t\t\t" << cadena2;		
		
		if(operacion[i].incompatible) cadena3 = "SI"; else cadena3 = "NO";
		if(operacion[i].difieren_colas) cadena4 = "SI"; else cadena4 = "NO";
		if(operacion[i].difieren_fmuestreo) cadena5 = "SI"; else cadena5 = "NO";		
		
		std::cout << "\n\tIncompatible: " << cadena3;
		std::cout << "\nDifieren colas: " << cadena4;
		std::cout << "\tDifieren fmuestreos: " << cadena5;		 
		
		if (Bus::parametros.size() - i > 1) { // Si quedan más dispositivos en la lista, se espera orden de teclado
			std::cout << "\n\n<<Introduzca cualquier texto para continuar>>";
			std::cin >> cadena1;
		}
	
	}
	std::cout << "\n\n---------------------------------------------------------------------\n\n\n"; 
	
	return;
	
}

void Bus::propiedades_base() { //Propiedades de clase base. (Se usa la de dispositivo)
	
	Dispositivo::propiedades_base();
	
	return;
}
