
// CONECTABLE.CPP 

// Clase de conectables. Dispositivos que se pueden conectar unos con otros. 

#include "base.h"
#include <iostream> 

Conectable::Conectable(unsigned int t, int id) : Dispositivo(t,id) { // Constructor del conectable
	/*plantilla_caracteristicas caracteristicas; 
	
	caracteristicas.clase = CONECTABLE; // estamos construyendo un conectable
	caracteristicas.tiene_propiedades = true; // El dispositivo tiene propiedades (accesibles por menú)	
	Dispositivo::caracteristicas.push_back(caracteristicas); // Lo constatamos en el vector características*/
	
}

int Conectable::actualiza() { // Actualizador del conectable, llama a los métodos de entrada/salida de los conectores

	register unsigned int i;
	
	
	if(Conectable::comunicable == true) { // Si se ha habilitado la posibilidad de comunicación, se realiza la E/S
		Conectable::mutex_conectable.lock(); // De no haber mutex 
														// Data race con "propiedades" del conectable
		for(i=0; i<Conectable::num_elementos ; i++) {
			if(!Conectable::parametros[i].puerto_asignado) continue; // Conector no asignado, se va al siguiente elemento
			Conectable::err_reciente=Conectable::parametros[i].conector->entradasalida((Conectable *)this,				
									Conectable::parametros[i].solo_lectura);
										// Se realiza la llamada para actualizar entrada/salida			
			Conectable::parametros[i].err = Conectable::err_reciente; 
										// Actualizar elemento del vector de ultimos accesos
		}
	// Fin de ronda de conexiones y exclusión. 
	Conectable::mutex_conectable.unlock();

	}
	
	return Dispositivo::actualiza(); // Actualizar clase base
	   // A posibles efectos de depuración y aviso devolvemos el error que pudo haber en la clase base  

}

int Conectable::conectar(Conector *conector, unsigned int puerto) { // Conectar el puerto del dispositivo
	
	unsigned int i; // indice
	int err = 0; // error
	
	if(Dispositivo::obtiene_ejecucion()) return 1; // Se está ejecutando el dispositivo. Error. No se puede. 
	
	if (puerto == 0 || Conectable::parametros.size()==0 || conector == nullptr) 
		return 2; // ¡No se puede conectar sin puertos asignados, con puerto nulo o sin conector asignado a objeto!
		
	for (i = 0 ; i < Conectable::parametros.size(); i++) { // búsqueda de puertos en el vector parametros
			if (Conectable::parametros[i].puerto == puerto) { // encontramos puerto asignado
				if(Conectable::parametros[i].puerto_asignado) return 3; // El puerto estaba asignado ya, error 3				
				Conectable::parametros[i].puerto_asignado = true; // Todo correcto, se asigna puerto					
				Conectable::parametros[i].conector = conector; // se asigna conector al vector					
				err = conector->registrar((Conectable *)this,Conectable::parametros[i].esid, 
					Conectable::parametros[i].entrada, Conectable::parametros[i].salida,
					Conectable::parametros[i].lentrada, Conectable::parametros[i].lsalida); 
															// Registrar el puerto en el conector	
				
				if(err > 0) std::cerr << "ERROR: Al registrar conector. Codigo de error = " << err << "\n"; 
								
				break; // Salir del bucle
				
			}
	}
	
	if(err > 0) return err+3; // Devuelve error si hubo error en el registro
	return 0; // No hubo error hasta este punto. 
	
}
													
int Conectable::nuevo_puerto(unsigned int puerto, int esid, BYTE *entrada, BYTE *salida
								, unsigned int lentrada, unsigned int lsalida) { // Nuevo puerto

	unsigned int i; // indice 
	parametros_conectable parametros; 
								
	if (puerto == 0 || esid == 0 || (entrada==nullptr && salida ==nullptr) || (lentrada == 0 && lsalida ==0)) return 1; 
														// Valores no permitidos, salir con error 1

										
	if (Conectable::parametros.size() != 0) {
		for (i = 0 ; i < Conectable::parametros.size(); i++)
			if (Conectable::parametros[i].puerto == puerto) return 2; // No se permite duplicar número de puertos, error 2
	
	}
	// Si el puerto es nuevo, registrarlo con sus características y salir sin error
	
	// Registrar conectores con nullptr. Luego pueden ser asignados
	parametros.conector = nullptr; parametros.puerto = puerto; parametros.esid = esid; parametros.entrada = entrada;
	parametros.salida = salida; parametros.lentrada = lentrada; parametros.lsalida = lsalida; 
	parametros.solo_lectura = false; parametros.err = 0; parametros.puerto_asignado = false; 
	// Por defecto, para lectura y escritura (solo_lectura = false).
	// El puerto, aunque registrado, no se ha asignado a un conector (puerto_asignado = false)	
	
	Conectable::parametros.push_back(parametros); 
	
	Conectable::num_elementos++; // Hemos añadido un elemento. Actualizamos variable	
	
	return 0;
}

int Conectable::modificar_solo_lectura(unsigned int puerto, bool solo_lectura) { 
										// Modificar el flag de solo lectura en un puerto
	
	register unsigned int i;
	register int resultado = -1; // Si no encuentra puerto, error -1
	
	for (i = 0; i < Conectable::parametros.size() ; i++) {
		if(Conectable::parametros[i].puerto == puerto && Conectable::parametros[i].puerto_asignado) {
			Conectable::parametros[i].solo_lectura = solo_lectura; 
			resultado = 0; 
			break;
		}
	}
	
	return resultado; 

}

int Conectable::obtiene_err_reciente() { // Obtener el error reciente (para la confirmación lectura/escritura)
	
	return Conectable::err_reciente;
	
}

void Conectable::propiedades() {
	
	unsigned int i; // indice
	
	std::vector<parametros_conectable> parametros;
	
	Conectable::mutex_conectable.lock(); // Exclusion mutua en la copia		
	parametros = Conectable::parametros; // Copia de parámetros		
	Conectable::mutex_conectable.unlock();
	
	std::cout << "\nPROPIEDADES DEL CONECTABLE:\n";	 
	
	std::cout << "Puerto\tesid\tAsignado\tlentrada\tlsalida\tsal\tSolo lectura\n";
	for(i = 0; i < parametros.size() ; i++) {
		std::string asignado = parametros[i].puerto_asignado ? "SI" : "NO"; // De Bool a string
		std::string solo_lectura = parametros[i].solo_lectura ? "SI" : "NO"; // De Bool a string
		
		std::cout << parametros[i].puerto << "\t" << parametros[i].esid << "\t\t" << asignado << "\t\t"
		<< parametros[i].lentrada << "\t" << parametros[i].lsalida << "\t" << parametros[i].err << "\t" 
		<< solo_lectura << "\n";			
	}		
}

void Conectable::propiedades_base() { //Propiedades de clase base. Siguiente: dispositivo
	
	Conectable::propiedades(); 
	
	Dispositivo::propiedades_base();
	
	return;
}