// Conector.cpp

// Incluye métodos de la clase Conector y Conector_rec

#include "base.h"


#include <iostream> 


// /////// Métodos de CONECTOR

Conector::Conector(int id) : Dispositivo (0,id){ //Constructor del conector
												 // Este conector no es ejecutable (frecuencia cero, sin recursividad)
												 // Sirve como intermediario para la comunicación entre dispositivos
												 // El mutex asociado al conector permite que no haya 'data races'
												 // que son indeseables entre dos threads que comparten memoria, pues 
												 // provocaría corrupción de los datos											 


	Conector::conteo_accesos = 0L;

	
	Conector::conector.escrito_leido[0] = false; // leido/escrito es un flag con 0=dato leido y 1=dato escrito que indica
	Conector::conector.escrito_leido[1] = false; // a quien lee o escribe si en el otro lado actualizaron 
										// información o leyeron
										// Con esto, evitamos escribir o leer innecesariamente y se ahorra tiempo de
										// proceso. 
	Conector::conector.accesos[0] = 0L;
	Conector::conector.accesos[1] = 0L;
										
}

int Conector::actualiza() { // Actualización del conector. Esta clase no realiza ninguna acción al actualizar
							// ya que el conector es un elemento pasivo, sin thread, sin iteración que sólo sirve
							// de puente de conexión entre dos o más dispositivos
							// Si se declarase una subclase con thread, este "actualiza" sólo hace de puente para cerrar
							// el lazo
int resultado = 0;
		
	resultado += Dispositivo::actualiza(); // Actualizar clase base
	return resultado;		
}



// Nota: "entrada" se refiere a la entrada desde el punto de vista del conector (lentrada la longitud de datos asociada)
// y "salida", de igual forma es la salida con respecto al conector. Hay que tenerlo en cuenta al implementarlo con los
// conectables que son el tipo de objetos más habituales para gestionar las entradas/salidas por clase conector
// *entrada se refiere, desde el punto de vista del dispositivo, a su salida
// *salida se refiere, desde el punto de vista del dispositivo, a su entrada

int Conector::registrar(Conectable *disp, int esid, BYTE *entrada, BYTE *salida, 
									unsigned int lentrada, unsigned int lsalida) { // registro dispositivo	
	
	if(disp == nullptr || esid == 0 || (lentrada == 0 && lsalida == 0)) return 1; 
								// Parámetros no permitidos con esos valores, devuelve error. Nota: lentrada y lsalida 
								// no pueden ser simultaneamente nulos
	if ((Dispositivo*)disp->obtiene_ejecucion()) return 2; // Solo se puede registrar "en frío" sin ejecución de hilo. 														
	
	if(Conector::conector.conectables[0] == nullptr) { // Si es la primera conexión, se introducen los datos
											  // Pero antes reservamos memoria
		if (lentrada != 0) {
			Conector::conector.buffers[0] = new BYTE[lentrada]; // El tamaño de entrada determina el tamaño de buffer y 
										// determinará la salida en el otro dispositivo al otro lado de la conexión. 

										
			std::memset(Conector::conector.buffers[0], 0, lentrada); // Se rellena el buffer de ceros
		}
		
		if (lsalida != 0) {
			Conector::conector.buffers[1] = new BYTE[lsalida]; // El tamaño de salida determina el tamaño de buffer y
										// determinará la entrada en el otro dispositivo al otro lado de la conexión.
										
			std::memset(Conector::conector.buffers[1], 0, lsalida); // Se rellena el buffer de ceros
		}
		
		
		Conector::conector.lentradas[0] = lentrada; // Se copian los datos del nuevo dispositivo conectado
		Conector::conector.lsalidas[0] = lsalida;
		Conector::conector.entrada[0] = entrada;
		Conector::conector.salida[0] = salida;
		Conector::conector.conectables[0] = disp;
		Conector::conector.esid[0] = esid;
		Conector::conector.id[0] = disp -> obtiene_id();
				
	} else { // Ya había una conexión. Se registra en la posición 1. Antes se comprueba la reciprocidad de los conectores
		if(lentrada != Conector::conector.lsalidas[0] || lsalida != Conector::conector.lentradas[0]) return 4; 
																			// Fallo en reciprocidad
																			// longitudes no son recíprocas
		if(esid != -Conector::conector.esid[0]) return 5; 
										// no hay reciprocidad en los esid (uno negativo respecto al otro
		
	
		
		Conector::conector.lentradas[1] = lentrada; // Se copian los datos del nuevo dispositivo conectado
		Conector::conector.lsalidas[1] = lsalida;
		Conector::conector.entrada[1] = entrada;
		Conector::conector.salida[1] = salida;
		Conector::conector.conectables[1] = disp;
		Conector::conector.esid[1] = esid;	
		Conector::conector.id[1] = disp -> obtiene_id();
		
	}	

	return 0; // Acaba el registro con éxito
}


int Conector::datos_complementario(int *esid, unsigned int *lentrada, unsigned int *lsalida) { 
											// datos del complementario que ya está conectado
											// Este método es utilizado para muestreadores, que ignoran la estructura
											// de datos que van a procesar. Después éstos harán registro normal 
											//
											
	if(Conector::conector.conectables[0] == nullptr) return 1; 
									// No hay dispositivo previo en el conector. Salir con error. 
	if(Conector::conector.conectables[1] != nullptr) return 2; 
									// Está el dispositivo totalmente ocupado. Salir con error.

	*esid = -Conector::conector.esid[0]; 
							// Copiar datos de la otra parte ya registrada (identificación esid opuesta) y salir.
	*lsalida = Conector::conector.lentradas[0]; // Recordatorio: La salida de una parte es la entrada de la otra
	*lentrada = Conector::conector.lsalidas[0]; // El esid es negativo con respecto al esid del otro dispositivo. 

	return 0; // éxito
}


// En la nomenclatura bus-discretizador se usa hacia_bus y hacia_dispositivo que es una nomenclatura similar pero
// que resulta más intuitiva porque visualizamos mejor a donde se dirigen los flujos de datos

int Conector::entradasalida(Conectable *conectable, bool solo_lectura) {
	
	register unsigned int i1; // índice natural (posición 0 o 1 del conector)
	register unsigned int i2; // índice recíproco (posición 1 o 0 del conector)
	register int sal = 0; // Salida de estado de escrito-leido
					  // sal = 1 solo se ha podido escribir (entrada) --> Del dispositivo al conector
					  // sal = 2 solo se ha podido leer (salida) -- > Del conector al dispositivo
					  // sal = 3 se ha podido escribir y leer -- > En ambos sentidos
	register unsigned int lentrada, lsalida;
	
	
	
	Conector::exclusion_entradasalida.lock(); // Se impide la concurrencia de otros hilos 
								   // en esta sección crítica de actualización de datos
	
	i1=0;
	Conector::conteo_accesos++; // Actualizamos contador de accesos 
	
	// Determinamos índice
	if(Conector::conector.conectables[i1] != conectable) {
		i1++;
		if(Conector::conector.conectables[i1] != conectable) {
			Conector::exclusion_entradasalida.unlock(); // Fin de exclusión. 
			return -1; // El conectable no se registró previamente. Error
		}	
	}
	
	lentrada = Conector::conector.lentradas[i1]; // Necesitamos los tamaños de los buffers de E/S
	lsalida = Conector::conector.lsalidas[i1];	
	
	// El dispositivo que accedió ESCRIBE. Introduce datos al buffer del conector. 
	if (!Conector::conector.escrito_leido[i1] && !solo_lectura && lentrada > 0) { 
						//¿Está el flag en estado leído cuando accedió el otro dispositivo? Escribimos.
						// Si el dispositivo sólo ha solicitado lectura tampoco se escribirá
			
			std::memcpy(Conector::conector.buffers[i1], Conector::conector.entrada[i1], lentrada); 
									// Se utiliza el algoritmo rápido implementado en std
			Conector::conector.escrito_leido[i1] = true; 
									// Hemos escrito, esto lo recibirá el otro dispositivo cuando acceda
			sal++; 
			Conector::conector.accesos[i1]++; // Se cuenta este acceso para la estadística y comprobaciones
	}
	
	i2 = i1 == 0 ? 1 : 0; // Si i1 = 0 pasa a ser i2= 1 y si es i1= 1 pasa a ser i2 = 0 (complementario). 
	
	// El dispositivo que accedió LEE. Extrae datos del buffer del conector. 
	 if (Conector::conector.escrito_leido[i2] && lsalida > 0) { 
						//¿Está el flag de estado escrito cuando accedió el otro dispositivo? Leemos.
					// No varía el índice de conector.salida, es el índice natural del dispositivo //			
			std::memcpy(Conector::conector.salida[i1], Conector::conector.buffers[i2], lsalida); 
						// Se utiliza el algoritmo rápido implementado en std		
			Conector::conector.escrito_leido[i2] = false;	
			sal += 2;
	}		
	
	Conector::exclusion_entradasalida.unlock(); // Fin de exclusión. 
	
	return sal;	
}

void Conector::propiedades() { //Propiedades en pantalla
	
	// Nota sobre propiedades: 
	// En propiedades es preferente usar accesos atómicos std::atomic. Con vectores, o con datos masivos, 
	// un volcado previo de memoria entorpece lo menos posible los hilos que están funcionando, 
	// pues necesitan una gran cantidad de recursos de CPU. 
	
	std::string escrito_leido0;
	std::string escrito_leido1;
		
	if(Conector::conector.escrito_leido[0]) escrito_leido0 = "ESCRITO"; else escrito_leido0 = "LEIDO";
	if(Conector::conector.escrito_leido[1]) escrito_leido1 = "ESCRITO"; else escrito_leido1 = "LEIDO";

	std::cout << "\n\n-------- PROPIEDADES DEL CONECTOR -----------------------------------\n\n";
	
	std::cout << "\t\tDispositivo en extremo 1\t\tDispositivo en extremo 2\n";
	std::cout << "\nID:\t\t\t\t" << Conector::conector.id[0] << "\t\t\t" << Conector::conector.id[1];
	std::cout << "\nESID:\t\t\t\t" << Conector::conector.esid[0] << "\t\t\t" << Conector::conector.esid[1];
	
	std::cout << "\nLentrada:\t\t\t" << Conector::conector.lentradas[0] << "\t\t\t" << Conector::conector.lentradas[1];
	std::cout << "\nLsalida:\t\t\t" << Conector::conector.lsalidas[0] << "\t\t\t" << Conector::conector.lsalidas[1];
	std::cout << "\nNº escrituras efectivas:\t" << Conector::conector.accesos[0] 
											<< "\t\t\t" << Conector::conector.accesos[1];
	std::cout << "\nBytes totales:\t\t\t" << Conector::conector.accesos[0] * Conector::conector.lentradas[0] << "\t\t\t" 
									<< Conector::conector.accesos[1] * Conector::conector.lentradas[1];
	std::cout << "\nFlag escrito/leído:\t\t" << escrito_leido0 << "\t\t\t" << escrito_leido1;
	std::cout << "\nConteo global de accesos:" << Conector::conteo_accesos << "\n\n"; 
	std::cout <<     "---------------------------------------------------------------------\n\n\n";
	
	return;
	
}

void Conector::propiedades_base() { //Propiedades de clase base. Se indica la misma que Dispositivo
	
	Dispositivo::propiedades();
	
	return;
}

Conector::~Conector() { // Destructor de la clase. Es necesario para liberar las zonas de memoria reservadas
							// en los buffers
							
	
	if(Conector::conector.buffers[0] != nullptr) delete Conector::conector.buffers[0];
	if(Conector::conector.buffers[1] != nullptr) delete Conector::conector.buffers[1];	
//		std::cout << "Destructor Conector\n"; // Si se descomenta se comprueba si funciona el destructor virtual

}

//
// Métodos de Conector_rec /////////////////////////////////////////////// Conector para la recolección de datos
//

Conector_rec::Conector_rec(int id, Grabador *grabador) : Conector (id){ //Constructor del conector
												 // Al igual que su clase base este conector de recolección no es 
												 // ejecutable. Se provee puntero a grabador
	
	if (grabador != nullptr) grabador -> registrar(this); // Registramos en el grabador
	else Dispositivo::activa_alarma(); // Algo fue muy mal. Alarma. 
	
												 
}

int Conector_rec::registrar(Conectable *disp, int esid, BYTE *entrada, BYTE *salida,
								unsigned int lentrada, unsigned int lsalida) { // Modo propio de registro
	int retorno; 										// Hereda el registro de conector y le añade algunas 
														// tareas como crear los buffers de recolección
														
	retorno = Conector::registrar(disp, esid, entrada, salida, lentrada, lsalida);	// Registro estándar de conector
	
	// Sin embargo, este método derivado de conector, ha de hacer más operaciones. Se recomienda polimorfismo
	// al registrar desde un conectable, y utilizando el modificador (Conector *) como se puede ver en los ejemplos.

	if(retorno == 0 && Conector_rec::buffer_sondeo_entrada == nullptr && 
				Conector_rec::buffer_sondeo_salida == nullptr) { // ¿Fue exitoso? Creamos buffers propios -solo una vez-
		if (Conector::conector.lentradas[0] != 0) {
			Conector_rec::buffer_sondeo_entrada = new BYTE[Conector::conector.lentradas[0]]; 
			// El tamaño de entrada determina el tamaño de buffer 								 

										
			std::memset(Conector_rec::buffer_sondeo_entrada, 0, Conector::conector.lentradas[0]); 
													// Se rellena el buffer de ceros
		}
		
		if (Conector::conector.lsalidas[0] != 0) {
			Conector_rec::buffer_sondeo_salida = new BYTE[Conector::conector.lsalidas[0]]; 
			// El tamaño de salida determina el tamaño de buffer 
										
										
			std::memset(Conector_rec::buffer_sondeo_salida, 0, Conector::conector.lsalidas[0]); 
													// Se rellena el buffer de ceros
		}
		
	}
	
	return retorno;
}

int Conector_rec::crea_sonda(bool io, unsigned int pos, tipo_datos_rec tipo, std::string etiqueta) {
	
	plantilla_datos_rec dato; // Dato o parámetros del conector de recolección. Se vectoriza (varias sondas para recoger)
	BYTE *dir; // Puntero a los buffers
	int salida = 0; // Salida indicando error si hubiese
	unsigned int tam_pos; // Tamaño máximo del buffer si considerásemos el objeto que queremos insertar

	
	// Comprobaciones. 1) Se comprueba existencia de buffer, 2) se comprueba si la posición es adecuada según 
	if((!io && buffer_sondeo_entrada != nullptr) || (io && buffer_sondeo_salida != nullptr)) {
		tam_pos = pos + obtiene_sizeof(tipo);		
		switch (io) {
			case false: // Buffer de lectura (entrada) RESPECTO DEL PRIMER DISPOSITIVO QUE SE CONECTO
				if(tam_pos > Conector::conector.lentradas[0]) salida = -1;
				dir = buffer_sondeo_entrada + pos; // Dirección del dato a recoger (buffer + pos)
				break;
			
			case true: // Buffer de escritura (salida)
				if(tam_pos > Conector::conector.lsalidas[0]) salida = -1;
				dir = buffer_sondeo_salida + pos; // Dirección del dato a recoger (buffer + pos)
				break;			
		};
		if(salida < 0) { // Error -2: Posición y tipo sobrepasa tamaño del buffer
			salida--;		
			return(salida); 			
		}		
	} else {
		salida = -1; // Error -1: no hay buffer en esa posición de lectura/escritura. Error. 		
		return salida;
	}
	// Una vez hechas las comprobaciones, se rellenan los datos y se añaden al vector	
	
	dato.id = Dispositivo::obtiene_id();
	dato.conector = this;
	dato.pos = pos;
	dato.tipo = tipo;
	dato.io = io;
	dato.dir = dir;
	dato.etiqueta = etiqueta;
		
	Conector_rec::exclusion_vector.lock(); // Es improbable que haya concurrencia, por interface del usuario pero	
											// se previene data race en el vector entre el hilo principal y el hilo
											// del grabador
		
	Conector_rec::datos.push_back(dato);
	
	Conector_rec::exclusion_vector.unlock(); // Fin de exclusión	
	return salida;
	
}

void Conector_rec::recoleccion() {
		// Recolecta datos. Básicamente, los copia a los buffer propios para que haga uso el grabador con los 
		// datos proporcionados en la estructura datos_rec. 
	Conector::exclusion_entradasalida.lock(); // Bloqueo mutex, copia rapida y desbloqueo mutex	
	
		if(Conector::conector.lentradas[0] > 0)
			std::memcpy(buffer_sondeo_entrada, Conector::conector.buffers[0], Conector::conector.lentradas[0]); 
		
		if(Conector::conector.lsalidas[0] > 0)
			std::memcpy(buffer_sondeo_salida, Conector::conector.buffers[1], Conector::conector.lsalidas[0]); 
	
	Conector::exclusion_entradasalida.unlock();
	
}	


const std::vector<plantilla_datos_rec>& Conector_rec::obtener_datos() { // Obtener los datos del conector

return Conector_rec::datos;

} 	

unsigned int Conector_rec::obtiene_sizeof(tipo_datos_rec tipo) { 
														// Obtiene sizeof de un tipo de datos (implementación para 
														// clases que trabajan con tipos de datos en los argumentos)
	unsigned int salida;
	
	switch(tipo) {
		
		case FLOAT:
			salida = sizeof(float);
			break;
		case DOUBLE:
			salida = sizeof(double);
			break;
		case INT:
			salida = sizeof(int);
			break;
		case CHAR:
			salida = sizeof(char);
			break;
		case BOOL:
			salida = sizeof(bool);
			break;
		default:
			salida = 0; // Si el tipo es inválido, salida nula
			break;			
	};

	return salida; 

}	
	
void Conector_rec::propiedades() { //Propiedades en pantalla y opciones de recolección/parametrización
	
	Conector::propiedades(); // Se llama a la clase padre Conector para imprimir sus propiedades

}

void Conector_rec::propiedades_base() { //Propiedades de clase base. Se indica la misma que Dispositivo
	
	Dispositivo::propiedades();
	
	return;
}

Conector_rec::~Conector_rec() { // Destructor. Libera buffers reservados 
	
	if(Conector_rec::buffer_sondeo_entrada != nullptr) delete Conector_rec::buffer_sondeo_entrada;
	if(Conector_rec::buffer_sondeo_salida != nullptr) delete Conector_rec::buffer_sondeo_salida;
	
//		std::cout << "Destructor Conector_rec\n"; // Comprobación de destructor virtual. Al descomentar se
// ve claramente que ha sido invocado este destructor y después el de la clase base Conector
	
	// ¿hay que llamar al destructor de la clase padre? /////////////

}
