// CONTROL_MOTAMO - Control de motorCC y amortiguador combinados. Ejemplo de escalado. Dispositivos de
// diferente naturaleza conviviendo en el mismo proceso

#include "base.h"
#include "sistemas/controlador_motorcc.h"
#include "sistemas/controlador_amortiguador.h"
#include "sistemas/constante.h"
#include "sistemas/funciones_periodicas.h"

#include <iostream>

int main()
{
	
// CREACION DE WATCHDOGS ---------------------------------------------------------------------

	Watchdog<float> *watchdog_control = new Watchdog<float>(10,5301);

// CREACION DE CONTROLADORES -----------------------------------------------------------------
	
	Control_motorCC *control_motorCC = new Control_motorCC(50,1301, watchdog_control,1,1.5,0, false);
	Control_amortiguador *control_amort = new Control_amortiguador(50,1302, watchdog_control,19190,20320,0);
	
// CREACION DE DISPOSITIVOS GENERADORES Y FUNCIONES MATEMATICAS ------------------------------

	Constante *consigna_mot = new Constante(1,2301,600,-1000,1000);
	Constante *consigna_amort = new Constante(1,2302,-0.2,-1000,1000);

// CREACION DE BUSES -------------------------------------------------------------------------
	
	Bus *bus = new Bus(3301, PROCESO_CONTROLADOR);	

// CREACION DE DISCRETIZADORES ---------------------------------------------------------------
	
	Discretizador *discret_mot = new Discretizador(50,4301,2, bus);	
	Discretizador *discret_amo = new Discretizador(50,4302,2, bus);	
	
	
// CREACION DE CONECTORES --------------------------------------------------------------------
	
	Conector *control_dis_mot = new Conector(6301); // Entre controlador y discretizador // MOTOR
	Conector *consig_control_mot = new Conector(6302); // Entre consigna y controlador
	
	Conector *control_dis_amort = new Conector(6303); // Entre controlador y discretizador // AMORTIGUADOR
	Conector *consig_control_amort = new Conector(6304); // Entre consigna y controlador	
	
										   
// CREACION DE GRABADORES --------------------------------------------------------------------

										   
// CREACION DE LOS CONECTORES DE RECOLECCION DE DATOS (para grabar)---------------------------

	
// CONECTAMOS TODOS LOS COMPONENTES	----------------------------------------------------------	
		
	control_motorCC -> conectar(control_dis_mot,1); 
	discret_mot -> asociar(control_dis_mot); // Obtiene datos en discretizador, para E/S, con este conector 
	discret_mot -> conectar(control_dis_mot, 1); // Conectado
	
	
	consigna_mot -> conectar(consig_control_mot, 1); // Conectamos controlador a la consigna
	control_motorCC -> conectar(consig_control_mot, 2);
	
	control_amort -> conectar(control_dis_amort,1); 
	discret_amo -> asociar(control_dis_amort); // Obtiene datos en discretizador, para E/S, con este conector 
	discret_amo -> conectar(control_dis_amort, 1); // Conectado
	
	
	consigna_amort -> conectar(consig_control_amort, 1); // Conectamos controlador a la consigna
	control_amort -> conectar(consig_control_amort, 2);
	
// CREAMOS SONDAS PARA GRABAR DATOS A ARCHIVO EXTERNO ----------------------------------------		


// REGISTRAMOS CONTROLADORES EN WATCHDOGS ----------------------------------------------------
	
	watchdog_control -> registrar(control_motorCC, 20, true); // Registro de los controladores en el watchdog
	watchdog_control -> registrar(control_amort, 20, true); // En caso de fallo, detención automática

// CREAMOS EL MENU INTERACTIVO ---------------------------------------------------------------
	
	Interactivo menu;
	
// REGISTRAMOS TODOS LOS DISPOSITIVOS EN EL MENU INTERACTIVO
	menu.registro(control_dis_mot, "conector discretizador-motor");
	menu.registro(consig_control_mot, "conector consigna-motor");
	menu.registro(discret_mot, "Discretizador motor");
	menu.registro(control_motorCC, "Controlador motor");	
	menu.registro(consigna_mot,"Consigna motor");

	
	menu.registro(control_dis_amort, "conector discretizador-amort");
	menu.registro(consig_control_amort, "conector consigna-amort");
	menu.registro(discret_amo, "Discretizador amortiguador");
	menu.registro(control_amort, "Controlador amortiguador");
	menu.registro(consigna_amort,"Consigna amortiguador");
	menu.registro(watchdog_control,"Watchdog");
	
	menu.registro(bus, "Bus");

	
// EJECUTAMOS MENU INTERACTIVO ---------------------------------------------------------------

	menu.ejecutar(true); // ejecutamos y liberamos todos los objetos llamando a sus destructores (true)	
	
	std::cout << "\nSalida del programa CONTROL\n";  // Mensaje final
		
	return 0;
	
}