// Clase para los discretizadores --- DISCRETIZADOR. CPP 
// Implementación de la clase base Discretizador 
// Implementación de los discretizadores HW1 y HW2 para conexión con dispositivos reales 

#include "base.h" // declaración de la clase base
#include <iostream> // flujo estándar entrada/salida (pantalla)
 
												  // Constructor. Necesitamos registrar el bus
												  // Los datos de E/S los conocerá del dispositivo mediante el conector
										          // La id ha de coincidir con el discretizador del otro lado del bus.
												  // lcola es el máximo retardo en cola de los datos del discretizador
												  // en el bus. 
Discretizador::Discretizador(unsigned int t, int id, unsigned int lcola, Bus *bus) : Conectable(t, id) {
	
		
	if(id==0) std::cerr << "AVISO: Discretizador con id = 0. La comunicación no será efectiva\n";		
				
	
	if(bus != nullptr) Discretizador::bus = bus; // ¿Es un puntero a bus? Lo registramos para las operaciones de E/S
	else Discretizador::hay_bus = false; // Si no hay bus, se entiende que no se hará uso de él.
	
	Discretizador::parametros.id = id;
	Discretizador::parametros.fmuestreo = t;
	Discretizador::parametros.huella1 = HUELLA1; // Huella que identifica la cabecera
	Discretizador::parametros.huella2 = HUELLA2;
	Discretizador::parametros.recibido = false; // no se recibió la otra cabecera
	Discretizador::parametros.esid = 0; // No hay conector asociado, esid = 0
	
	if(lcola==0) {std::cerr << "AVISO: Cola en discretizador nula. Se asigna valor 1";	
					Discretizador::parametros.tamano_cola = 1;
	}
	else Discretizador::parametros.tamano_cola = lcola;		
	
}

int Discretizador::asociar(Conector *conector) { // Asociar el discretizador al dispositivo y al bus
											   // al otro lado del conector *conector. Se asignará al puerto 1
											   // y estará listo para conectarse y hacer E/S con el dispositivo
											   // En caso de éxito en conector, se registra en bus
	int err = 0, ret = 0; 
	unsigned int lal_dispositivo = 0;
	unsigned int lal_bus = 0;
	int esid_discretizador = 0;
	BYTE *al_bus, *al_dispositivo; // Punteros de datos que se calculan sumando el tamaño de cabecera
//	cabecera_sv_conteo *cabecera; // Puntero a cabecera para calcular puntero a zona de datos
	
	if(conector != nullptr) { // Si el conector es el apropiado, podemos llamar a sus métodos
	
									  // de lo contrario, consultamos con el conector y extraemos datos útiles
									  // para realizar la conexión y habilitar los buffers. 
		err=conector->datos_complementario(&esid_discretizador, 
							&lal_dispositivo, &lal_bus); // obtenemos datos
		
		if(err == 0) {
			Discretizador::parametros.lal_dispositivo = lal_dispositivo; // Copiamos parametros de conexión a la estructura 
																		 // de datos
			Discretizador::parametros.lal_bus = lal_bus;
			Discretizador::parametros.esid = esid_discretizador;					
		}
		
		if(Discretizador::esid_conexion != 0) { // La clase derivada HW2 quiere una conexión determinada. Comparamos
			if(Discretizador::esid_conexion != esid_discretizador) err = 1; // Si difieren, error; no se puede continuar
			if(Discretizador::lal_dispositivo_conexion != lal_dispositivo) err = 1;
			if(Discretizador::lal_bus_conexion != lal_bus) err = 1;
		}
	} else { // No hay conector (HW1). Se copian los datos de conexión suministrados en las variables de conexión
		Discretizador::parametros.lal_dispositivo = Discretizador::lal_dispositivo_conexion;
		Discretizador::parametros.lal_bus = Discretizador::lal_bus_conexion;
		Discretizador::parametros.esid = Discretizador::esid_conexion;
		
		lal_dispositivo = Discretizador::parametros.lal_dispositivo;
		lal_bus = Discretizador::parametros.lal_bus;
		esid_discretizador = Discretizador::parametros.esid; 
	}
	
	if(Discretizador::parametros.esid == 0) err = 1; // Ha de haber esid habilitado en cualquiera de los casos
	
	
	if(err > 0) return err+1; // ¿Error? salir, mostrar código 2 a efectos de depuración

	
	//Creamos espacios de memoria dinámica según demanda. Incluimos espacio para la cabecera. 
	if(al_bus_c == nullptr && al_dispositivo_c == nullptr && ((lal_dispositivo != 0 && lal_bus != 0) ||
		(Discretizador::lal_dispositivo_conexion != 0 && Discretizador::lal_bus_conexion != 0))) {
			
		unsigned int lal_dispositivo_dim = lal_dispositivo != 0 ? 
			lal_dispositivo : Discretizador::lal_dispositivo_conexion;
		
		unsigned int lal_bus_dim = lal_bus != 0 ? 
			lal_bus : Discretizador::lal_bus_conexion;
			
//	Discretizador::al_bus_c = std::malloc(lal_bus_dim+sizeof(cabecera_sv_conteo));	
		Discretizador::al_bus_c = new BYTE[lal_bus_dim+sizeof(cabecera_sv_conteo)];
//	Discretizador::al_dispositivo_c = std::malloc(lal_dispositivo_dim+sizeof(cabecera_sv_conteo));
		Discretizador::al_dispositivo_c = new BYTE[lal_dispositivo_dim+sizeof(cabecera_sv_conteo)];
	
		std::memset(Discretizador::al_bus_c, 0, lal_bus_dim+sizeof(cabecera_sv_conteo)); // Se rellenan los buffers de ceros
		std::memset(Discretizador::al_dispositivo_c, 0, lal_dispositivo_dim+sizeof(cabecera_sv_conteo)); 
	
		al_bus = al_bus_c + sizeof(cabecera_sv_conteo);
		al_dispositivo = al_dispositivo_c + sizeof(cabecera_sv_conteo);
		
		Discretizador::al_bus = al_bus; // Se utilizará de forma permanente estos punteros para las comunicaciones
		Discretizador::al_dispositivo = al_dispositivo;
}
	
//	al_bus = (BYTE *)(((cabecera_sv_conteo *)(al_bus_c))+1); // Puntero tras la cabecera, es zona de datos
//	al_dispositivo = (BYTE *)(((cabecera_sv_conteo *)(al_dispositivo_c))+1); 
	
	err = Conectable::nuevo_puerto(1, esid_discretizador, Discretizador::al_dispositivo,
	    Discretizador::al_bus, lal_dispositivo, lal_bus);  // Registramos únicamente el puerto 1
															   // La conexión al bus no necesita puerto, se hará
															   // por medio de su método específico
	if(err > 0) return err+2; // ¿Error? salir, mostrar código de error acumulado 	
	
	// Una vez asociado el conector y obtenidos los parámetros de datos en 'datos', se procede a asociar el bus
	// y de esta forma el discretizador ya está listo para su funcionamiento
	
	if(Discretizador::hay_bus) 
		Discretizador::bus->registrar((Discretizador *)this, Discretizador::al_bus_c, 
				Discretizador::	al_dispositivo_c); 
				// se registra con el puntero de la instancia del discretizador
	
	
	if(conector == nullptr) ret = 1; // Si no se proporcionó conector, se notifica con el retorno
	return ret; // En caso contrario, terminó de forma completa (retorno 0). 
								
}

int Discretizador::actualiza() { // Se da por supuesto que se dio una dirección válida de bus en el constructor.
							   // Los mensajes enviados pueden ir a la cola del bus o directamente a msg
	
	int codigo;
	int error_es = 0; // Código y error que podría devolver la E/S del bus
	bool solo_lectura = false; // ¿Se utilizará el bus en modo solo lectura? comprobamos conector
	
	if(Discretizador::hay_bus) { // Si hay bus, se puede utilizar
	
		switch(obtiene_err_reciente()) { // ajustamos solo_lectura según el resultado del conector
				case 1:
					solo_lectura = true; // Sólo se pudo escribir en el conector, por tanto no hay dato para el bus
					break;	
				default:
					solo_lectura = false; // caso habitual
					break;		
		};
		
		codigo = Discretizador::bus->lecturaescritura(this, solo_lectura, &error_es); 
												//Llamada al bus para efectuar la E/S
		
		Discretizador::retorno_bus = codigo; // Para las propiedades
		Discretizador::err = error_es;
		
		switch(codigo) {
			case -1: // Casos en los que hay condición de alarma (sin conexion, sin bus o incompatible con el otro lado)
			case -2:
			case -3:
				Dispositivo::activa_alarma();
				break;
			case 2: // Se recibió o se recibió/escribió. Se habilita escritura en conectable y flag para propiedades 
			case 3:
				Discretizador::escribe_a_dispositivo = true; // flag de escritura para propiedades
				Discretizador::modificar_solo_lectura(1, false); // Deshabilitamos solo lectura en conectable
				break;
			default: // Otros casos no se contemplan para actuar. Por defecto, no se escribe al conector
				Discretizador::modificar_solo_lectura(1, true); // Habilitamos solo lectura en conectable
				break;
		};
	
	}
	
	return Conectable::actualiza(); // Llamamos al actualizador del conectable	
}

void Discretizador::rutina() { // Rutina de thread 
	
	int err = 0; // Error al intentar hacer solo_lectura
	
	err = Conectable::modificar_solo_lectura(1, true); 
												 // Por defecto, el dispositivo solo efectuará lectura. Escribirá cuando 
												 // tenga datos válidos. Es recomendable realizar esto en 
												 // dispositivos intermedios (discretizadores)
												 // Pero es más comprometido hacerlo en extremos (controlador y planta)
												 // pues de hacerlo en ambos ningún dispositivo escribiría 
												 // y no se establecería ningún flujo de información. 
	
	if (err == -1) std::cerr << "AVISO: Discretizador sin conectar a un dispositivo\n"; 
	
	while (Dispositivo::obtiene_ejecucion()) { // Es una rutina genérica que solo se interrumpe si deja de estar en
												// ejecucion
		
		Discretizador::actualiza(); // Tan solo actualizar entradas/salidas por cada periodo
	}	
}

plantilla_datos_discretizador_sv Discretizador::obtiene_parametros() { // Obtener datos del discretizador	
	
	return Discretizador::parametros; 
}


Discretizador::~Discretizador() { // Destructor. Necesario para liberar la memoria de los buffers 
	
	if(Discretizador::al_bus_c != nullptr) delete Discretizador::al_bus_c;
	if(Discretizador::al_dispositivo_c != nullptr) delete Discretizador::al_dispositivo_c;
	
	// Método "antiguo" con std::free
	
//	if(Discretizador::al_bus_c != nullptr) std::free(Discretizador::al_bus_c); // Liberación completa de los buffers
//	if(Discretizador::al_dispositivo_c != nullptr) std::free(Discretizador::al_dispositivo_c);	
	
}

void Discretizador::propiedades() { //Propiedades en pantalla
	
	// Nota sobre propiedades: 
	// En propiedades es preferente usar accesos atómicos std::atomic. Con vectores, o con datos masivos, 
	// un volcado previo de memoria entorpece lo menos posible los hilos que están funcionando, 
	// pues necesitan una gran cantidad de recursos de CPU. 
	
	std::string escribe;
	std::string alarma;
		
	if(Discretizador::escribe_a_dispositivo) escribe = "SI"; else escribe = "NO";
	if(Dispositivo::obtiene_alarma()) alarma = "SI"; else alarma = "NO";

	std::cout << "\n\n-------- PROPIEDADES DEL DISCRETIZADOR ------------------------------\n\n";
	
	std::cout << "\nParámetros del discretizador";
	std::cout << "\n\tid:\t\t\t\t\t" << Discretizador::parametros.id;
	std::cout << "\n\tesid:\t\t\t\t\t" << Discretizador::parametros.esid;
	std::cout << "\n\tTamaño de cola:\t\t\t\t" << Discretizador::parametros.tamano_cola;
	std::cout << "\n\tFrecuencia de muestreo:\t\t\t" << Discretizador::parametros.fmuestreo;
	std::cout << "\n\tesid:\t\t\t\t\t" << Discretizador::parametros.esid;
	std::cout << "\n\tTamaño de paquete hacia el dispositivo:\t" << Discretizador::parametros.lal_dispositivo;
	std::cout << "\n\tTamaño de paquete hacia el bus:\t\t" << Discretizador::parametros.lal_bus;
	
	std::cout << "\n\nError del Bus:\t\t\t\t" << Discretizador::err;
	std::cout << "\nCodigo de retorno del Bus:\t\t" << Discretizador::retorno_bus;	
	std::cout << "\nEscribe a dispositivo:\t\t" << escribe;
	std::cout << "\nCondición de alarma:\t\t" << alarma;
	
	std::cout << "\n\nInformación: la condición de alarma se activa por incompatibilidad con el otro";
	std::cout << "\n             discretizador al otro lado del bus o por no poder haberse";
	std::cout << "\n             habilitado una línea en el sistema de cola de mensajes";             
	
	std::cout << "\n---------------------------------------------------------------------";
	
	return;
	
}

void Discretizador::propiedades_base() { //Propiedades de clase base	
	
	Conectable::propiedades();
	
	Dispositivo::propiedades();

	
	return;
}

// DISCRETIZADOR HW1 //////////////////////////////////////////////////

// Constructor de la clase. Se configuran parámetros y el bus al asociar.
Discretizador_HW1::Discretizador_HW1(unsigned int t, int id, int esid, unsigned int laplanta, 
		unsigned int lacontrolador, unsigned int lcola, Bus *bus) : Discretizador(t, id, lcola, bus) {
	
	Discretizador::esid_conexion = -esid; // esid complementario (negativo)
	Discretizador::lal_bus_conexion = lacontrolador;
	Discretizador::lal_dispositivo_conexion = laplanta;
	
	return;
		
}

int Discretizador_HW1::actualiza() { // Actualización en el ciclo, se llama a hardware y a la actualización de
									 // la clase base (discretizador)
									 
	Discretizador_HW1::hardware(); // Llamada a la rutina de E/S
	
	return Discretizador::actualiza(); // Llamada a la superclase y retorno

	
}

int Discretizador_HW1::asociar() { // Asociar al bus. No hay conector para proporcionar (void) 
	
	int err; 
	err = Discretizador::asociar(nullptr); // Se efectúa la asociación, sin conector	

	// Actualizamos los punteros contextualizados a planta-controlador
	
//	Discretizador_HW1::a_planta = Discretizador::al_dispositivo;
//	Discretizador_HW1::a_controlador = Discretizador::al_bus;
	std::cout << "///////////////////// err = " << err << "\n\n";
	return err;
	
	
}

void Discretizador_HW1::rutina() { // Rutina de thread 
	
	while (Dispositivo::obtiene_ejecucion()) { // Es una rutina genérica que solo se interrumpe si deja de estar en
												// ejecucion
		
		Discretizador_HW1::actualiza(); // Tan solo actualizar entradas/salidas por cada periodo
	}	
}

void Discretizador_HW1::hardware() { // Rutina de E/S con el hardware. Virtual. Cada subclase implementa su rutina
	
	
}

Discretizador_HW1::~Discretizador_HW1() { // Destructor
	
	
}
// DISCRETIZADOR HW2 //////////////////////////////////////////////////

// Constructor de la clase. Se configura el conector al asociar.

Discretizador_HW2::Discretizador_HW2(unsigned int t, int id, int esid, 
		unsigned int laplanta, unsigned int lacontrolador) : Discretizador(t, id, 1, nullptr) {
	
	Discretizador::esid_conexion = esid;
	Discretizador::lal_bus_conexion = laplanta;
	Discretizador::lal_dispositivo_conexion = lacontrolador;
	
	return;
	
}

int Discretizador_HW2::actualiza() { // Actualización en el ciclo, se llama a hardware y a la actualización de
									 // la clase base (discretizador)
									 
	Discretizador_HW2::hardware(); // Llamada a la rutina de E/S
	
	return Discretizador::actualiza(); // Llamada a la superclase y retorno
	
}

int Discretizador_HW2::asociar(Conector *conector) { // Asociar al conector. No hay bus para asociarse 
									 
	int err; 
	err = Discretizador::asociar(conector); // Se efectúa la asociación, con conector (sin bus, en constructor)	

	// Actualizamos los punteros contextualizados a planta-controlador
	
//	Discretizador_HW2::a_planta = Discretizador::al_bus;
//	Discretizador_HW2::a_controlador = Discretizador::al_dispositivo;
	
	return err;								 
	
	
}

void Discretizador_HW2::rutina() { // Rutina de thread 
	
		
	while (Dispositivo::obtiene_ejecucion()) { // Es una rutina genérica que solo se interrumpe si deja de estar en
												// ejecucion
		
		Discretizador_HW2::actualiza(); // Tan solo actualizar entradas/salidas por cada periodo
	}	
}

void Discretizador_HW2::hardware() { // Rutina de E/S con el hardware. Virtual. Cada subclase implementa su rutina
	
	
}

