// GRABADOR.CPP 
//
// 
// Clase de grabación de datos y volcado a archivo de texto
// Se mantiene en comunicación con los conectores_rec (conectores de recolección de datos). 
// 
//
//

#include "base.h" // declaración de la clase base

#include <iostream> // Para salida por pantalla
#include <fstream> // Para salida por archivo (grabador de datos)


Grabador::Grabador(unsigned int t, int id, tipo_proceso tipo, bool diferentes, unsigned int kbs): Dispositivo(t,id) { 
																			// Constructor
	
	Grabador::tipo = tipo; // Registramos tipo de proceso (para el nombre del archivo)
	Grabador::diferentes = diferentes; // Si queremos diferentes nombres en una sesión se pone a true
	Grabador::kbs = kbs; // Se guarda el límite	
	
}

void Grabador::rutina() { // Rutina de thread 
	
	
	// Se hace una actualización de la lista según los conectores_rec que hay. Se habilitan vectores
	actualizar_datos();
	Grabador::cuenta = 0; // Cuenta a cero (en la primera vez)
	Grabador::limite = false; // Se resetea límite
	Grabador::uso_bytes = 0; // Uso en bytes se resetea
	Grabador::cuenta_global++; // Cuenta global se incrementa
	
	while (Dispositivo::obtiene_ejecucion()) { 
												
		
		Grabador::actualiza(); // Actualización continua en su modo de trabajo
	}

	// Se solició parar la ejecución. Guardamos a archivo los datos recolectados. Si no hay datos, no graba
	
	if (Grabador::datos.size() > 0) // Para volcar a archivo tiene que haber vector de datos 
			Grabador::archivo();
}

int Grabador::actualiza() { // Actualización del grabador que consiste en recopilar los datos que ofrece cada 
							 // conector_rec
	
	unsigned int i; // indice general
	unsigned int ifloat = 0, idouble = 0, iint = 0, ichar = 0, ibool = 0; // indices por tipo 
	int salida; // Retorno
	
	Conector_rec *conector = nullptr; // Recorrido por conectores para llamar a recoleccion	
	
	
	for(i=0 ; i < Grabador::datos.size() ; i++) { // Recorremos el vector de datos y recolectamos
									// Los punteros son únicos así que se hace recolección cada vez que 
									// cambia el puntero del dato del vector con respecto al anterior
		if(Grabador::datos[i].conector != conector) { // ¿conector diferente al anterior? Recolecta de ese conector
			conector = Grabador::datos[i].conector;
			conector -> recoleccion();
		}
	
		switch(Grabador::datos[i].tipo) { // Registramos en la cola, según el tipo
			case FLOAT:
				Grabador::datos_float[ifloat++].push(*(float *)Grabador::datos[i].dir); 
				if(Grabador::limite) Grabador::datos_float[ifloat-1].pop(); // Si se sobrepasó recortamos por delante
				break;			
			case DOUBLE:
				Grabador::datos_double[idouble++].push(*(double *)Grabador::datos[i].dir); 
				if(Grabador::limite) Grabador::datos_double[idouble-1].pop();// Si se sobrepasó recortamos por delante
				break;			
			case INT:
				Grabador::datos_int[iint++].push(*(int *)Grabador::datos[i].dir); 
				if(Grabador::limite) Grabador::datos_int[iint-1].pop();// Si se sobrepasó recortamos por delante
				break;			
			case CHAR:
				Grabador::datos_char[ichar++].push(*(char *)Grabador::datos[i].dir); 
				if(Grabador::limite) Grabador::datos_char[ichar-1].pop();// Si se sobrepasó recortamos por delante
				break;			
			case BOOL:
				Grabador::datos_bool[ibool++].push(*(bool *)Grabador::datos[i].dir); 
				if(Grabador::limite) Grabador::datos_bool[ibool-1].pop();// Si se sobrepasó recortamos por delante
				break;			
			default: 
				break;
		};		
	
	}
	
	if(!Grabador::limite) { 
		Grabador::cuenta++; // Tras rellenar los datos y no haber límite, cuenta sube a uno
		
		Grabador::uso_bytes = (Grabador::ifloat * sizeof(float) + Grabador::idouble * sizeof(double) + 
		Grabador::iint * sizeof(int) + Grabador::ichar * sizeof(char) + Grabador::ibool * 
		sizeof(bool)) * Grabador::cuenta; // Cálculo de uso
		if(Grabador::kbs > 0) {	// Si kbs es cero no se hará comprobación del límite	
			if (uso_bytes > (Grabador::kbs * 1024)) Grabador::limite = true; 
										//¿uso sobrepasó el límite? entonces limite=true
		}
	}
	
	salida = Dispositivo::actualiza(); // Llamamos al actualizador de la clase padre (dispositivo, en este caso)
	
	return salida;

}

int Grabador::registrar(Conector_rec *conector) { // Registra un conector_rec

	unsigned int i; // indice
	int salida = 0;
	
	if (conector == nullptr) return -1; // No se admiten punteros nulos	

	for (i = 0 ; i < Grabador::conectores.size() ; i++) { // Comprobación de registro.
		if(Grabador::conectores[i] == conector) { // ¿Ya estaba el conector registrado? Error
				salida = -2;
				break;
		}
	}		 
	
	if (salida == 0 ) Grabador::conectores.push_back(conector); // Se guarda conector y se sale sin error

	return salida;
}

void Grabador::actualizar_datos() { // actualizar los datos al iniciarse el grabador. También habilita vectores
	
	std::vector<plantilla_datos_rec> datos; // vector de datos de cada conector 
	unsigned int i,j; // indices
	unsigned int ifloat = 0, idouble = 0, iint = 0, ichar = 0, ibool = 0; // indices por tipo 
	std::queue<float> a; std::queue<double> b; std::queue<int> c; std::queue<char> d;
	std::queue<bool> e; // Colas genéricas sin iniciar. 
	
	Grabador::mutex_grabador.lock(); // Modificamos vector de datos. Mutex (en propiedades() se lee el vector)
	Grabador::datos.clear(); // Borrar vector previo de datos
	
	for(i = 0 ; i < Grabador::conectores.size(); i++) { // Recorrido por la lista de conectores
	
		datos = Grabador::conectores[i] -> obtener_datos(); // Obtención
		
		for(j = 0 ; j < datos.size() ; j ++) { // recorrido por cada conector
		
			Grabador::datos.push_back(datos[j]);
			switch(datos[j].tipo) { // Acumulamos indices de cada tipo para reservar memoria posteriormente
			
				case FLOAT:					
					Grabador::datos_float.push_back(a);
					ifloat++;
					break;			
				case DOUBLE:
					Grabador::datos_double.push_back(b);
					idouble++;
					break;			
				case INT:
					Grabador::datos_int.push_back(c);
					iint++;
					break;			
				case CHAR:
					Grabador::datos_char.push_back(d);
					ichar++;
					break;			
				case BOOL:
					Grabador::datos_bool.push_back(e);
					ibool++;
					break;			
				default: 
					break;
			};		
			
		}
		
	}
	
	Grabador::mutex_grabador.unlock(); // Fin de zona segura
	
	Grabador::ifloat = ifloat; Grabador::idouble = idouble; Grabador::iint = iint; Grabador::ichar = ichar;
	Grabador::ibool = ibool; // Guardamos a variables estadísticas visibles por propiedades();
	
	return;
}

void Grabador::archivo() {
	
	std::string nombre, proceso; // Nombre del archivo
	unsigned int i; // indice
	unsigned int ifloat, idouble, iint, ichar, ibool; // indices por tipo 
	float periodo = Dispositivo::obtiene_period(); // Obtiene periodo
	float cuenta_tiempo = 0; // Cuenta de tiempo por paso, se incrementa periodo a periodo
	std::ofstream archivo_salida; // Descriptor de archivo para E/S
	
	if(Grabador::cuenta == 0 || Grabador::datos.size() == 0) return; // Nada que guardar, salir
	
	switch(tipo) { // Tipo proceso para incorporarlo al nombre del archivo
		case PROCESO_PLANTA:
			proceso = "PLANTA";
			break;
		case PROCESO_CONTROLADOR:
			proceso = "CONTROL";
			break;
		default:
			proceso = "DESCONOCIDO";
			break;
		
	};
	if(Grabador::diferentes) 
		nombre = proceso + "_ID" + std::to_string(Dispositivo::obtiene_id()) + "_" +
			std::to_string(Grabador::cuenta_global) + ".dat"; // Creamos nombre con diferencias en cuenta_global
	else	
		nombre = proceso + "_ID" + std::to_string(Dispositivo::obtiene_id()) + ".dat"; // Creamos nombre sin diferencias
	
	
	archivo_salida.open(nombre, std::ios::out); // El archivo sólo se usará para operaciones de salida (escribir a disco)
	
	// Empieza la escritura
	
	archivo_salida << "# " << nombre << "\n" << "#Tiempo";
	
	for(i=0 ; i < Grabador::datos.size() ; i++) { // Primer recorrido para las etiquetas del archivo de salida
		if(Grabador::datos[i].etiqueta.size() != 0) // Ha de haber etiqueta
			archivo_salida << "\t\t" << Grabador::datos[i].etiqueta;
		else
			archivo_salida << "\t\t" << i; // Si no hay etiqueta (vacío) se sustituye por la cuenta del indice	
	}
	
	archivo_salida << "\n"; // Intro
	
	while (Grabador::cuenta-- > 0) { // Mientras la cuenta sea positiva, guardamos los datos recolectados 
		ifloat = 0; idouble = 0; iint = 0; ichar = 0; ibool = 0; // Se ponen a cero los indices en cada iteración
		archivo_salida << cuenta_tiempo;
		
		for(i=0 ; i < Grabador::datos.size() ; i++) { // Recorremos el vector de datos y enviamos a flujo de archivo
		
			switch(Grabador::datos[i].tipo) { // Registramos según el tipo en el archivo de salida
				case FLOAT:
					archivo_salida << "\t\t" << Grabador::datos_float[ifloat].front(); 
					Grabador::datos_float[ifloat++].pop();
					break;			
				case DOUBLE:
					archivo_salida << "\t\t" << Grabador::datos_double[idouble].front(); 
					datos_double[idouble++].pop(); 
					break;			
				case INT:
					archivo_salida << "\t\t" << Grabador::datos_int[iint].front(); 
					Grabador::datos_int[iint++].pop(); 
					break;			
				case CHAR:
					archivo_salida << "\t\t" << Grabador::datos_char[ichar].front(); 
					Grabador::datos_char[ichar++].pop();
					break;			
				case BOOL:
					archivo_salida << "\t\t" << Grabador::datos_bool[ibool].front();
					Grabador::datos_bool[ibool++].pop();
					break;			
				default: 
					break; 
			};		
		}
		cuenta_tiempo += periodo; // Se incrementa la cuenta de tiempo por el periodo
		archivo_salida << "\n"; // Salto de linea
	}
	
	archivo_salida.close(); // Cierre del archivo
	
	Grabador::datos_float.clear(); // Borrado de los vectores 
	Grabador::datos_double.clear();
	Grabador::datos_int.clear();
	Grabador::datos_char.clear();
	Grabador::datos_bool.clear();
	
	Grabador::cuenta = 0; // Cuenta a cero
	
	return;
}

void Grabador::propiedades() {
	
	unsigned int i; // indice
	
	std::cout << "\n\n-------- PROPIEDADES DEL GRABADOR -----------------------------------\n\n";
	
	std::cout << "\nTAMAÑOS DATOS float:" << sizeof(float) << " double:" << sizeof(double) 
			  << " int:" << sizeof(int) << " char:" << sizeof(char) << " bool:" << sizeof(bool);
	std::cout << "\nifloat: " << Grabador::ifloat << " idouble: " << Grabador::idouble 
	<< " iint: " << Grabador::iint << " ichar: " << Grabador::ichar << " ibool: " << Grabador::ibool
	<< " Nº conectores:" << Grabador::conectores.size();
	std::cout << "\n\nNº Inicio: " << Grabador::cuenta_global << " Filas creadas: " << 
	Grabador::cuenta << " Total datos: " << (float)Grabador::uso_bytes / 1024 << " Kbytes" <<
	 " Límite: " << Grabador::kbs << " Kbytes\n\n";
	 
	Grabador::mutex_grabador.lock(); // Haremos un recorrido por el vector de parámetros de los conectores
	
	for(i = 0; i < Grabador::datos.size() ; i++) {
		std::cout << "Sonda: " << i+1 << " ---> id:" << Grabador::datos[i].id << "\tpos:" << Grabador::datos[i].pos <<
		"\tTipo:" << Grabador::datos[i].tipo << "\tio:" << Grabador::datos[i].io << "\tEtiqueta:" << 
		Grabador::datos[i].etiqueta << "\n";	
		
	}	
	
	Grabador::mutex_grabador.unlock();
	
	
	std::cout << "\n\n---------------------------------------------------------------------\n\n\n";
	
	
}

void Grabador::propiedades_base() { //Propiedades de clase base. Dispositivo
	
	Dispositivo::propiedades_base();
	
	return;
}





