// INTERACTIVO.CPP

// Implementación del sistema interactivo por menús para la planta y el controlador

#include "base.h" // declaración de la clase base
#include <iostream> // E/S pantalla
#include <string> // Manejo de cadenas

// No es necesario un constructor

// Registro de los dispositivos

int Interactivo::registro(Dispositivo *dispositivo, std::string nombre) { 
						// Registramos un nuevo dispositivo en el bus

//	std::vector<plantilla_caracteristicas> caracteristicas;		
	
	if(dispositivo == nullptr) return -1; // No se admiten punteros nulos. 
		
	Interactivo::dispositivos.push_back(dispositivo); // Registramos dispositivo (para acceder a sus métodos)
	Interactivo::nombres.push_back(nombre); // Registramos nombre		
//	caracteristicas = dispositivo -> obtiene_caracteristicas(); // Obtenemos vector de caracteristicas
//	Interactivo::caracteristicas.push_back(caracteristicas); // Registramos caracteristicas (propiedades)
	
	Interactivo::ejecutable.push_back(dispositivo -> obtiene_ejecutable()); // Registramos si es ejecutable
	Interactivo::ejecucion.push_back(dispositivo -> obtiene_ejecucion()); // Registramos si se está ejecutando
	Interactivo::pausa.push_back(dispositivo -> obtiene_pausa()); // Registramos flag de pausa
	Interactivo::alarma.push_back(dispositivo -> obtiene_alarma()); // Registramos si hay alarma	
	
	return 0; // éxito
}

int Interactivo::ejecutar(bool borrar) { // Ejecución del menú (borrar = true -> libera objetos con 'delete'
	
	unsigned int opcion; //opción escogida en el menú del primer nivel
	bool todos_cerrados = true; // ¿Estaban todos los hilos detenidos al terminar?
	plantilla_primer_nivel accion; // Acción escogida en las opciones generales

	
	int err; // salida de error
	unsigned int i; // indice genérico para los bucles for
		
	if(Interactivo::dispositivos.size() == 0) { // ¿Ningún dispositivo? Error
		std::cerr << "\nERROR: No se cargó ningún elemento en el menú interactivo\n";
		return -1; // Código de error
	}
	
	do { // realizar este bucle indefinidamente hasta que se ordene salir con 0
		
		
		opcion = Interactivo::menu_general(); // Elegimos opción en el menú general
		
		if(opcion == 0 || opcion == Interactivo::dispositivos.size() + Interactivo::opciones_1.size() + 1)
			continue; // Hemos escogido salir o refrescar. Vamos a comprobación y repetimos menu general en refresco

		if(opcion <= Interactivo::dispositivos.size()) // Hemos elegido un dispositivo, al menú de dispositivo
			menu_dispositivo(opcion-1); // Se resta 1 porque la opción 1,2,3... se convierte en indice de vector 0,1,2...
			
		else { // Se eligió una de las opciones generales
		
			i = opcion - Interactivo::dispositivos.size() - 1 ; // El indice apunta al valor del vector de opciones 
			
			accion = Interactivo::opciones_1[i]; // Leemos la acción requerida y con base en ella actuamos
			
			switch(accion) {
				case INICIAR:
				std::cout << "\nArrancando todos los dispositivos...\n";
				Interactivo::arrancar_todos();
				break;
			case DETENER:
				std::cout << "\nDeteniendo todos los dispositivos...\n";
				Interactivo::detener_todos();
				break;
			case PAUSAR:
				std::cout << "\nPausando todos los dispositivos...\n";
				Interactivo::pausar_todos();
				break;
			case REANUDAR:
				std::cout << "\nRetomando todos los dispositivos pausados...\n";
				Interactivo::retomar_todos();
				break;				
				
			}
				
		}
		
	} while (opcion != 0); // Hasta que no queramos salir, no se rompe el bucle de menús
	
	std::cout << "\n\nDeteniendo hilos... "; // No pueden quedar hilos en ejecución si salimos del proceso principal 
										// han de unirse todos con std::thread.join(). Detenemos uno por uno. 
	
	for(i = 0 ; i < Interactivo::dispositivos.size() ; i++) { // Detención de hilos. Liberación de recursos 
		std::thread& hilo = Interactivo::dispositivos[i] -> acaba(err);
		if(err == 0) { // Se acaba, efectivamente. 
			hilo.join(); // Se espera al hilo porque se le dio orden de salida
			todos_cerrados = false;
		}
//		if(borrar) delete Interactivo::dispositivos[i]; // Se liberan los dispositivos de memoria llamando a sus 
													    // respectivos destructores (destructor virtual)
	}
	
	if (todos_cerrados) std::cout << "ya estaban detenidos"; else std::cout << "OK";	
	
	if(borrar) {
		for(i = 0 ; i < Interactivo::dispositivos.size() ; i++) { // borrado de objetos
			delete Interactivo::dispositivos[i];
		}
	}	
	
	std::cout << "\n\nSalida del menú\n";	// Mensaje final
	
	return 0;
}

unsigned int Interactivo::menu_general() { // Ejecución del menú en su nivel 1. (nivel más alto). Devuelve selección. 
					 // Este nivel nos ofrece un listado de todos los dispositivos cargados en el sistema	
					 // Podemos seleccionar el dispositivo que queramos o realizar acciones generales
	unsigned int opcion;
	unsigned int i,j; //indices y contadores
//	plantilla_primer_nivel tipo;
	
	std::string ejecutable; 
	std::string ejecucion; 
	std::string pausa; 
	std::string alarma; 
	
	// Cadenas SI / NO en ejecucion, ejecutable, pausa y alarma
	
	Interactivo::refresca_estados(); // Refrescamos estados
	
	Interactivo::conformar_menu_primer_nivel(); // Creamos/actualizamos el vector de opciones posibles	 
	
	std::cout << "\n\nMENU GENERAL ---- LISTA DE DISPOSITIVOS ----" << 
	"E1=Ejecutable E2=Ejecutandose PA=Pausa AL=ALARMA\n\n";
	std::cout << "Opción\tID Muestreo Hz\t\tE1 E2 PA AL\tTiempo seg.\tNombre\n\n";
	
	for(i=0 ; i < Interactivo::dispositivos.size() ; i++) { // Recorremos vector de dispositivos
	
			ejecutable = Interactivo::ejecutable[i] ? "SI" : "NO";
			ejecucion = Interactivo::ejecucion[i] ? "SI" : "NO";
			pausa = Interactivo::pausa[i] ? "SI" : "NO";
			alarma = Interactivo::alarma[i] ? "SI" : "NO";
	
			std::cout << "  " << i+1 << ")\t" << Interactivo::dispositivos[i] -> obtiene_id() << "\t" << 
			Interactivo::dispositivos[i] -> obtiene_muestreo() << "\t\t" << ejecutable << " " << 
			ejecucion << " " << pausa << " " << alarma << "\t" << Interactivo::dispositivos[i]->obtiene_tiempo_f() <<
			"\t" << Interactivo::nombres[i] << "\n";
	
	}
	std::cout << "\n\n";
	// Mostramos las opciones generales variables
	for (j=0 ; j < Interactivo::opciones_1.size() ; j++) {
		switch (Interactivo::opciones_1[j]) {
			case INICIAR:
				std::cout << "  " << ++i <<") Iniciar todos los dispositivos \n";
				break;
			case DETENER:
				std::cout << "  " << ++i <<") Detener todos los dispositivos \n";
				break;
			case PAUSAR:
				std::cout << "  " << ++i <<") Pausar todos los dispositivos \n";
				break;
			case REANUDAR:
				std::cout << "  " << ++i <<") Reanudar todos los dispositivos que están pausados \n";
				break;
			// Nunca se da el caso default:
		}		
	}	
		// Opciones generales permanentes	
		std::cout << "  " << ++i <<") Actualizar lista \n\n" ;	
	
		std::cout << "  " << "0) Salir del programa\n\n";	
	do { // Entrada de teclado
	
		std::cout << "Seleccione opción (puede ser un dispositivo o un comando general -> "; // Realizamos la pregunta
		std::cin >> opcion; //espera orden de teclado
		
		if (std::cin.fail()) {   // En caso de introducir caracteres ilegales (letras p.ej) necesitamos filtrarlos
							// Pues de lo contrario entramos en un bucle infinito
			std::cin.clear(); // Borrar flujo/cola de entrada    
			std::cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n'); // Descartar elementos indeseables
		}
		
	} while (opcion > i); // Se esperan valores permitidos y se repetirá la pregunta si no es así
	
    return opcion;	
		
}

void Interactivo::conformar_menu_primer_nivel() { //Hacemos la lista de opciones del menú nivel uno, según el contexto
	
		unsigned int i; // indice
		bool todos_ejecutandose = true, todos_parados = true, todos_pausados = true, alguno_pausado = false; 
		Interactivo::opciones_1.clear(); // Borramos vector de opciones 1
		
		for (i = 0 ; i < Interactivo::dispositivos.size() ; i++) { // recopilamos datos de interés
			if(Interactivo::ejecutable[i]) { // Sólo afecta a los dispositivos ejecutables
				if(Interactivo::ejecucion[i]) todos_parados=false; else todos_ejecutandose=false;				
				if(!Interactivo::pausa[i] && Interactivo::ejecucion[i]) 
					todos_pausados = false;
				if(Interactivo::ejecucion[i] && Interactivo::pausa[i]) alguno_pausado = true;	
					
			}
		}		
		// A partir de estos datos recopilados, conformamos el menú
		
	
		
		if(!todos_ejecutandose) Interactivo::opciones_1.push_back(INICIAR);	
		if(!todos_parados) Interactivo::opciones_1.push_back(DETENER);	
		if(!todos_pausados && !todos_parados) Interactivo::opciones_1.push_back(PAUSAR);	
		if(!todos_parados && alguno_pausado) Interactivo::opciones_1.push_back(REANUDAR);
	
	return;
}

void Interactivo::menu_dispositivo(unsigned int dispositivo) { // Menú del dispositivo seleccionado. Menú de Nivel 2
	// Este menú es más especializado y nos permite interactuar con un dispositivo en concreto.  
	
	unsigned int opcion; // Entrada de teclado del menú
	unsigned int i,j; // Indices/contadores
	int err; // Variable de errores 
	
	do {
		j = 0; // Inicializamos en cada ciclo
		std::cout << "\n\nMENU DE DISPOSITIVO ------------ ID=" << Interactivo::dispositivos[dispositivo]->obtiene_id() 
		<< "  Nombre: " << Interactivo::nombres[dispositivo] << "-----\n\n";	

		
		std::cout << "  " << ++j <<") Propiedades de este dispositivo \n";
		std::cout << "  " << ++j <<") Propiedades de las clases base de este dispositivo \n";
		

		// Poblamos el vector de opciones de primer nivel para un solo dispositivo
		// Es decir, creamos, según el contexto arranque/parada/pausa/reanudar
		
		Interactivo::opciones_1.clear(); // Borramos vector de opciones. Y lo recreamos según el contexto
										// Siempre y cuando estemos ante un dispositivo ejecutable 
		if (Interactivo::ejecutable[dispositivo]) {
			if(!Interactivo::ejecucion[dispositivo]) Interactivo::opciones_1.push_back(INICIAR);	
			if(Interactivo::ejecucion[dispositivo]) Interactivo::opciones_1.push_back(DETENER);
			if(!Interactivo::pausa[dispositivo] && Interactivo::ejecucion[dispositivo]) 
							Interactivo::opciones_1.push_back(PAUSAR);
			if(Interactivo::pausa[dispositivo] && Interactivo::ejecucion[dispositivo]) 
						    Interactivo::opciones_1.push_back(REANUDAR);
		
		}
		
		std::cout << "\n"; // Salto de linea, nueva sección del menú
		
		for (i=0 ;i<Interactivo::opciones_1.size() ; i++) {
			switch (Interactivo::opciones_1[i]) {
				case INICIAR:
					std::cout << "  " << ++j <<") Iniciar el dispositivo\n";
					break;
				case DETENER:
					std::cout << "  " << ++j <<") Detener el dispositivo\n";
					break;
				case PAUSAR:
					std::cout << "  " << ++j <<") Pausar el dispositivo\n";
					break;
				case REANUDAR:
					std::cout << "  " << ++j <<") Reanudar el dispositivo\n";
					break;
				// Nunca se da el caso default:
			}
		}		
		std::cout << "\n" << "  " << "0) Salir del menu de dispositivo\n\n";	
	
		do { // Entrada de teclado
	
			std::cout << "Seleccione opción de dispositivo -> "; // Realizamos la pregunta
			std::cin >> opcion; //espera orden de teclado
			
	/*		if (std::cin.fail()) {   // En caso de introducir caracteres ilegales (letras p.ej) necesitamos filtrarlos
								// Pues de lo contrario entramos en un bucle infinito
				std::cin.clear(); // Borrar flujo/cola de entrada    
				std::cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n'); // Descartar elementos indeseables
			} */
			
		} while (opcion > j); // Se esperan valores permitidos y se repetirá la pregunta si no es así
		
		// Ya tenemos una opción válida. Hemos escogido una propiedad o queremos ejecutar un comando
		
		
		if(opcion == 1 || opcion == 2) { // Hemos escogido ir a propiedades y propiedades de base 
		
			switch (opcion) {
				case 1:
					dispositivos[dispositivo] -> propiedades();
					break;
				case 2:
					dispositivos[dispositivo] -> propiedades_base();
					break;
			};			

								// Establecemos el indice adecuado para el vector de opciones 1
		} else { // Hemos escogido ejecutar comando de inicio/parada/pausa/reanudar
		
			if(opcion == 0) continue; // Se pidió salir, vamos a condicion de bucle y saldrá
		
			i = opcion -  3 ; // Establecemos el indice adecuado para el vector de opciones 1
								
			switch (Interactivo::opciones_1[i])	{ // Llamadas según la opción elegida.
			
				case INICIAR:
					if(Interactivo::ejecutable[dispositivo] && !Interactivo::pausa[dispositivo] && 
							!Interactivo::ejecucion[dispositivo]) { // ¿Cumple los requisitos?																													
						std::cout << "Iniciando > ID" << Interactivo::dispositivos[i] -> obtiene_id() << "\n";				
						Interactivo::dispositivos[dispositivo] -> inicia();
					}
					break;								
					
				case DETENER:
					if(Interactivo::ejecutable[dispositivo] && Interactivo::ejecucion[dispositivo]) { 
								// ¿Es ejecutable y se está ejecutando? Intentamos parar. Notificamos.
						std::cout << "Deteniendo > ID" << Interactivo::dispositivos[dispositivo]->obtiene_id() << "\n";
						std::thread &hilo = Interactivo::dispositivos[dispositivo] -> acaba(err);
							if(err == 0) hilo.join(); // Se espera al hilo y se liberan sus recursos. 
							if (err == -1) std::cout << "\n\tAVISO: No se pudo detener el dispositivo ID: " 
								<< Interactivo::dispositivos[dispositivo] -> obtiene_id() << "\n";				
		
					}	
					break;
				
				case PAUSAR:
					if(Interactivo::ejecutable[dispositivo] && Interactivo::ejecucion[dispositivo] 
							&& !Interactivo::pausa[dispositivo]) { 
								// ¿Es ejecutable, se está ejecutando y no está pausado? Intentamos pausar
						std::cout << "Pausando > ID" << Interactivo::dispositivos[dispositivo] -> obtiene_id() << "\n";
						err = Interactivo::dispositivos[dispositivo] -> pausa(true);
						if (err == -1) std::cout << "\n\tAVISO: No se pudo pausar el dispositivo ID: " 
							<< Interactivo::dispositivos[dispositivo] -> obtiene_id() << "\n";				
		
					}	
					break;
				case REANUDAR:
					if(Interactivo::ejecutable[dispositivo] && Interactivo::ejecucion[dispositivo] 
								&& Interactivo::pausa[dispositivo]) {  
								// ¿Es ejecutable, se está ejecutando y está pausado? Intentamos retomar
						std::cout << "Reanudando > ID" << Interactivo::dispositivos[dispositivo] -> obtiene_id() << "\n";
						err = Interactivo::dispositivos[dispositivo] -> pausa(false);
						if (err == -1) std::cout << "\n\tAVISO: No se pudo retomar el dispositivo ID: " 
							<< Interactivo::dispositivos[dispositivo] -> obtiene_id() << "\n";	
					}
					break;
				
				default: // No se debió llegar aquí
					break;
			};	
			
			refresca_estados(); // Aviso: Cada vez que se modifiquen los estados de ejecución, actualizamos lista
								// En los métodos arrancar_todos(), detener_todos(), etc. también se invoca esta
								// función. 
		}		
		
	} while (opcion != 0); // Si elegimos opción 0, salimos	
	
	return;
}

void Interactivo::arrancar_todos() { // Arrancar todos los dispositivos
	
	unsigned int i;
	
	
	for(i = 0 ; i < Interactivo::dispositivos.size() ; i++) {
		if(Interactivo::ejecutable[i] && !Interactivo::pausa[i] && !Interactivo::ejecucion[i]) { 
															// ¿Cumple los requisitos?															
			std::cout << "Iniciando > ID" << Interactivo::dispositivos[i] -> obtiene_id() << "  \n";
				
			Interactivo::dispositivos[i] -> inicia(); // No se comprueba el error porque garantizamos que no ocurra
					
		}	
	}
	refresca_estados(); // Se refrescan los estados de todos los dispositivos en los vectores (ejecucion, pausa, etc.)
//	std::cin >> i;
}	
		
void Interactivo::detener_todos() { // Detener todos los dispositivos
	
	unsigned int i; 
	int err; // número de error
	
	for(i = 0 ; i < Interactivo::dispositivos.size() ; i++) {
		if(Interactivo::ejecutable[i] && Interactivo::ejecucion[i]) { 
			// ¿Es ejecutable y se está ejecutando? Intentamos parar. Notificamos.
			std::cout << "Deteniendo > ID" << Interactivo::dispositivos[i] -> obtiene_id() << "  \n";
			std::thread &hilo = Interactivo::dispositivos[i] -> acaba(err);
			if(err == 0) hilo.join(); // Se espera al hilo y se liberan sus recursos. 
			if (err == -1) std::cout << "\n\tAVISO: No se pudo detener el dispositivo ID: " 
				<< Interactivo::dispositivos[i] -> obtiene_id() << "\n";				
		
		}	
	}
	refresca_estados(); // Se refrescan los estados de todos los dispositivos en los vectores (ejecucion, pausa, etc.)
}
void Interactivo::pausar_todos() { // Pausar todos los dispositivos

	unsigned int i; 
	int err; // número de error
	
	for(i = 0 ; i < Interactivo::dispositivos.size() ; i++) {
		if(Interactivo::ejecutable[i] && Interactivo::ejecucion[i] && !Interactivo::pausa[i]) { 
					// ¿Es ejecutable, se está ejecutando y no está pausado? Intentamos pausar
			std::cout << "Pausando > ID" << Interactivo::dispositivos[i] -> obtiene_id() << "  \n";
			err = Interactivo::dispositivos[i] -> pausa(true);
			if (err == -1) std::cout << "\n\tAVISO: No se pudo pausar el dispositivo ID: " 
				<< Interactivo::dispositivos[i] -> obtiene_id() << "\n";				
		
		}	
	}
	refresca_estados(); // Se refrescan los estados de todos los dispositivos en los vectores (ejecucion, pausa, etc.)	
}
void Interactivo::retomar_todos() { // Retomar todos los dispositivos

	unsigned int i; 
	int err; // número de error
	
	for(i = 0 ; i < Interactivo::dispositivos.size() ; i++) {
		if(Interactivo::ejecutable[i] && Interactivo::ejecucion[i] && Interactivo::pausa[i]) {  
			// ¿Es ejecutable, se está ejecutando y está pausado? Intentamos retomar
			std::cout << "Reanudando > ID" << Interactivo::dispositivos[i] -> obtiene_id() << "  \n";
			err = Interactivo::dispositivos[i] -> pausa(false);
			if (err == -1) std::cout << "\n\tAVISO: No se pudo retomar el dispositivo ID: " 
				<< Interactivo::dispositivos[i] -> obtiene_id() << "\n";				
		
		}	
	}
	refresca_estados(); // Se refrescan los estados de todos los dispositivos en los vectores (ejecucion, pausa, etc.)
}

void Interactivo::refresca_estados() {        	//Se refrescan los estados de todos los dispositivos 
												// en los vectores(ejecucion, pausa, etc.)
	unsigned int i; 
	
	for(i = 0 ; i < Interactivo::dispositivos.size() ; i++) {
		Interactivo::ejecutable[i] = Interactivo::dispositivos[i] -> obtiene_ejecutable();
		Interactivo::ejecucion[i] = Interactivo::dispositivos[i] -> obtiene_ejecucion();						
		Interactivo::pausa[i] = Interactivo::dispositivos[i] -> obtiene_pausa();							
		Interactivo::alarma[i] = Interactivo::dispositivos[i] -> obtiene_alarma();			
	}							
}								
								