// DISCRETIZADOR_MOTORCC_HW1.CPP 

// Discretizador hardware de clase HW1. Ejemplo para un motor CC

// Archivo de implementación de la clase

#include "discretizador_motorcc_hw1.h"
#include <iostream> 

// Constructor

	// Configuramos la_planta y la_controlador en el constructor, referencia en	
	// esid = 3 (ver estructuras de datos en 'estructuras_datos.h')

Discretizador_motorcc_HW1::Discretizador_motorcc_HW1(unsigned int t, int id, // Constructor
		unsigned int lcola, Bus *bus) : Discretizador_HW1(t, id, 3, sizeof(ENTRADA_ES_3_HW),
		sizeof(SALIDA_ES_3_HW), lcola, bus) { 
	
	// En este espacio del constructor se inicializa el hardware y las interfaces, si se necesitara ///////
		
	// wiringPiSetup(); // Se inicia configuración	
	
	// pcf8591Setup (PCF, 0x48); // Configurar PCF8591 on en el pin 120, y en la dirección 0x48	
	
	// pinMode (PIN_ALARMA, INPUT); // Configurar pin digital GPIO4 como entrada de alarma
			
	/////////////////////////////////////////////////////////////////// fin de inicialización hardware ///	
									
}	

int Discretizador_motorcc_HW1::asociar() { // Asociación. No hay que olvidar actualizar los punteros estructurales
	
	int err; 
	err = Discretizador_HW1::asociar(); // Se efectúa la asociación, sin conector	
	
	if(err == 1) { // Asociación exitosa, aunque sin conector -> err = 1. Se pueden asignar los punteros de estructuras
		Discretizador_motorcc_HW1::entrada_es_3 = (ENTRADA_ES_3_HW *)Discretizador_motorcc_HW1::al_dispositivo;
		Discretizador_motorcc_HW1::salida_es_3 = (SALIDA_ES_3_HW *)Discretizador_motorcc_HW1::al_bus;
	}
	
	// Estos punteros de estructuras son empleados por la rutina hardware
	
	return err;									
}						

// Entrada /Salida, según el hardware a emplear o puertos del microcontrolador

void Discretizador_motorcc_HW1::hardware() { // Rutina de E/S con el hardware mediante una interfaz específica	
		
		int tac_i, vol_i; // Entrada y salida analógica (tacómetro y voltaje de salida) en formato entero y coma flotante
		float tac_f, vol_f;
		bool alarma = false; // Alarma
		
		if(Discretizador_motorcc_HW1::entrada_es_3 == nullptr || 
			Discretizador_motorcc_HW1::salida_es_3 == nullptr) return; // Con punteros nulos no se puede trabajar
		
		// No se leen los demás valores del motor como posición angular e intensidad por el puerto esid=3
		
//		tac_i = analogRead(PCF + 0); // Se lee puerto analógico de entrada, placa PCF8591
		tac_i = 125; // Para depuración, y visualización, se lee el valor 125 (en 8 bits)
		
		// Conversión, de entero a float del valor del tacómetro. 
		// Sabemos sus límites en coma flotante con TAC_MAX y TAC_MIN, por tanto, convertimos a decimal su valor		
		tac_f = MIN_TAC + (MAX_TAC - MIN_TAC) * ((float)tac_i / (float)ESCALA_ADC);
		
		// Se actualizan valores en la estructura. Ésta se envía hacia el controlador vía bus
		Discretizador_motorcc_HW1::salida_es_3->s1 = tac_f;
		
		// Se actualizan valores en la estructura. Ésta se recibe desde el controlador vía bus
		vol_f = Discretizador_motorcc_HW1::entrada_es_3->e1;
		
//		std::cout << "\n" << vol_f; //////////////////////// DEPURACIÓN
		
		vol_f = vol_f > MAX_VOL ? MAX_VOL : vol_f; // Comprobamos límites que no se pueden sobrepasar
		vol_f = vol_f < MIN_VOL ? MIN_VOL : vol_f;
		
		// Conversión, de float a entero del voltaje. 
		// Sabemos sus límites en coma flotante con VOL_MAX y VOL_MIN, por tanto obtenemos su valor
		
		// convertimos en tanto a uno con respecto a los límites
		
		vol_f = (vol_f - MIN_VOL) / (MAX_VOL - MIN_VOL); 
		
		// Convertimos a entero
		
		vol_i = (int)((float)ESCALA_DAC * vol_f);
		
		// Escribimos en el puerto de salida
		
//		analogWrite (PCF + 0, vol_i);

		// Leemos la alarma, que es un bit digital, conectado a GPIO4 (por ejemplo)
		
		// alarma = digitalRead(PIN_ALARMA) == HIGH ? true : false; // Se lee alarma estado HIGH = true
		Discretizador_motorcc_HW1::salida_es_3->alarma = alarma; // Se actualiza valor en la estructura
		
		
		tac_i = vol_i; // Esta instrucción evita un aviso de 'variable no usada' cuando se compila habiendo
					   // comentado las instrucciones E/S Raspberry. No tiene efecto en el funcionamiento del programa.
}

int Discretizador_motorcc_HW1::actualiza() { // Actualización
	
	Discretizador_motorcc_HW1::hardware(); // Llamada a la rutina de E/S
		
	return Discretizador_HW1::actualiza(); // Llamada a la superclase (cadena de actualizaciones)
									
}

void Discretizador_motorcc_HW1::rutina() { // Rutina de thread 
	
	while (Dispositivo::obtiene_ejecucion()) { // Es una rutina genérica que solo se interrumpe si deja de estar en
												// ejecucion
		
		Discretizador_motorcc_HW1::actualiza(); // Tan solo actualizar entradas/salidas por cada periodo
	}	
}

Discretizador_motorcc_HW1::~Discretizador_motorcc_HW1() { // Destructor 
	
	// En este espacio del destructor se finaliza las conexiones hardware y las interfaces, si se necesitara ///////
	
	
	
	
	
	
	/////////////////////////////////////////////////////////////////// fin de cierre con el hardware /////////////
	
}
