// MOTORCC_ARDUINO.H 

// Declaración de clases para el microcontrolador Arduino. Hardware -> Dinamico -> PID en un motor CC
// 
//

#include "Arduino.h"

#define ESCALA_ADC 256 // 10 bits tienen los convertidores D/A de Arduino
#define ESCALA_DAC 256 // 8 bits tienen las salidas analógicas, por modulación de ancho de pulso (PWM)

/* DEFINIMOS VALOR MAXIMO Y MINIMO DE LA VELOCIDAD DEL TACÓMETRO DE ENTRADA (DE ENTERO A FLOAT) EN RADS/SEG */

#define MAX_TAC 10000.
#define MIN_TAC -10000.

/* DEFINIMOS VALOR MAXIMO Y MINIMO DEL VOLTAJE DE LA ENTRADA DE CONSIGNA DEL MOTOR EN RADS/SEG */

#define MAX_CON 2000.
#define MIN_CON -2000.

/* DEFINIMOS VALOR MAXIMO Y MINIMO DEL VOLTAJE DE SALIDA AL MOTOR (DE FLOAT A ENTERO) EN VOLTIOS */

#define MAX_VOL 50.
#define MIN_VOL -50.

#define MAX_COLA 4 // Longitud máxima por cola (para control PID ha de ser como mínimo de 2)

class Hardware { // Clase base. Gestiona aspectos del hardware, principalmente la entrada/salida
	int canalE = 9999, canalS; // Canales de comunicación con el hardware de control del motor (velocidad de giro)
	int canal_consigna = 9999;	
	int canal_alarma;
	bool alarma = false; // Alarma
	int comunicacion_hardware();
	
protected:
	float entrada = 0, salida = 0;
	float consigna = 0;	
	int actualiza();  // Actualización

public:	
	Hardware(unsigned int canalE, unsigned int canalS, unsigned int canal_consigna, unsigned int canal_alarma);
		// Canales de entrada y salida (analogicos) y canal de alarma (digital)
		
		// entrada: velocidad de giro del motor CC suministrado por un tacómetro digital
		// salida: tensión suministrada al motor, acondicionada posteriormente y propocionada en el devanado
		
		// alarma: señal digital suministrada por el motor. Si éste falla, se detecta y se envía con valor high
		// La clase derivada que se encarga del control podrá tomar medidas como detener el motor (tensión cero)
	bool estado_alarma();
};

class Dinamico : public Hardware { // Clase derivada dinamico. Memoriza estados previos. De uso por el controlador PID
		
protected:
	float cola1[MAX_COLA]; // Cuatro colas con capacidad máxima de MAX_COLA
	float cola2[MAX_COLA];
	float cola3[MAX_COLA];
	float cola4[MAX_COLA];
	unsigned int longitudes[4]; // Longitudes de cada cola	
	float estado_actual[MAX_COLA]; // Estado actual y con el que se actualizarán las colas en cada iteración/actualización
	
	int actualiza(); // Actualización

public:	
	Dinamico(unsigned int canalE, unsigned int canalS, unsigned int canal_consigna, unsigned int canal_alarma,
		unsigned int longitud1, unsigned int longitud2, 
			unsigned int longitud3, unsigned int longitud4); // Constructor. Argumentos: canales E/S
															 // canal de alarma y tamaños de colas
	
};

class Controlador_PID : public Dinamico { // Clase derivada Controlador_PID.

	float p; // Coeficientes PID
	float i;
	float d;
	
	float t; // Periodo (según el ciclo del controlador)
	
	float a; // Coeficientes para las ecuaciones en diferencias
	float b;
	float c;
	
protected:
	
public:	
	Controlador_PID(unsigned int canalE, unsigned int canalS, 
	unsigned int canal_consigna, unsigned int canal_alarma,	float p, float i, float d, float t); 
											// Constructor. Argumentos: canales E/S, canal de consigna, canal de alarma
											// y también parámetros p, i, d.
	int actualiza(); // Actualización (es pública porque se ejecuta en loop(); )

	
};