// Planta_CARRO.cpp: Implementación de los dispositivos de planta del Carro. Basado en MotorCC (carro eléctrico)

// Obsérvese que los puertos a conectar son 4,5,6 en el dispositivo Carro (1,2,3 son los de MotorCC, que se conservan)

// Esta implementación de planta está conectada con el proceso de control control_carro.cpp
// El carro usa su puerto 4 para realizar las comunicaciones con el controlador.


//#include "base.h"

#include "sistemas/aleatorio.h"
#include "sistemas/carro_motorizado.h"
#include "sistemas/constante.h"
#include "base.h"

#include <iostream>

int main()
{	

// CREACION DE WATCHDOGS ---------------------------------------------------------------------

// CREACION DE CONTROLADORES -----------------------------------------------------------------

	
// CREACION DE DISPOSITIVOS DE FISICOS DE PLANTA, GENERADORES Y FUNCIONES MATEMATICAS --------

	Aleatorio *rozamiento = new Aleatorio(20,2501,419,577,129,899,1002,0.9,2.5); // 0.9, 2.5		
	Carro *carro = new Carro(500,2503,0.001,50,0.15,0.20,-0.15,-0.20,0); 
 

// CREACION DE BUSES -------------------------------------------------------------------------
	
	Bus *bus = new Bus(3501, PROCESO_PLANTA);	

// CREACION DE DISCRETIZADORES ---------------------------------------------------------------
	
	Discretizador *discret_car = new Discretizador(50,4501,2, bus);		
	
// CREACION DE CONECTORES CONVENCIONALES -----------------------------------------------------
	
//	Conector *car_dis = new Conector(6501); //Entre carro y discretizador	
//	Conector *roz_car = new Conector(6502); // Entre rozamiento y carro	
//  Conector *vol_car = new Conector(6503); // Entre voltaje y carro	

										   
// CREACION DE GRABADORES --------------------------------------------------------------------
	
	Grabador *grabador = new Grabador(10,8501, PROCESO_PLANTA, false, 500); // 500 kbytes de grabacion max

	
// CREACION DE LOS CONECTORES DE RECOLECCION DE DATOS (para grabar)---------------------------	
											
	Conector_rec *car_dis = new Conector_rec(7501, grabador); //Entre Carro y discretizador	
	Conector_rec *roz_car = new Conector_rec(7502, grabador); // Entre rozamiento y carro		
	
// CONECTAMOS TODOS LOS COMPONENTES	----------------------------------------------------------		
	
	rozamiento -> conectar(roz_car,1); // rozamiento-carro
	carro -> conectar(roz_car,5);	
	
	carro -> conectar(car_dis, 4); // Motor-discretizador	
	discret_car -> asociar(car_dis); // El discretizador ha de capturar los datos de esid, del motor (asociar)
									// Obtiene datos de E/S con este conector (aplicar en todos los discretizadores)
	
	discret_car -> conectar(car_dis, 1); // Tras asociar, conectar en 1 
	
// CREAMOS SONDAS PARA GRABAR DATOS A ARCHIVO EXTERNO ----------------------------------------	
	
	car_dis -> crea_sonda(false, 0, FLOAT, "Posición");
	car_dis -> crea_sonda(false, 0+sizeof(float), FLOAT, "Velocidad");
	car_dis -> crea_sonda(false, 0+2*sizeof(float), FLOAT, "Intensidad");
	car_dis -> crea_sonda(false, 0+3*sizeof(float), BOOL, "detector_derecha");	
    car_dis -> crea_sonda(false, 0+3*sizeof(float)+sizeof(bool), BOOL, "detector_izquierda");
	car_dis -> crea_sonda(false, 0+3*sizeof(float)+2*sizeof(bool), BOOL, "Alarma");
	car_dis -> crea_sonda(true, 0, FLOAT, "Voltaje");
	roz_car -> crea_sonda(false, 0, FLOAT, "Rozamiento");	
	

// CREAMOS EL MENU INTERACTIVO ---------------------------------------------------------------
	
	Interactivo menu;
	
// REGISTRAMOS TODOS LOS DISPOSITIVOS EN EL MENU INTERACTIVO	
	
	menu.registro(rozamiento, "Rozamiento");
	menu.registro(carro, "Carro");	
	menu.registro(roz_car, "conector roz_car");
	menu.registro(car_dis, "conector car_dis");
	menu.registro(discret_car, "discretizador_car");
	menu.registro(bus, "Bus");
	menu.registro(grabador, "Grabador");
	
// EJECUTAMOS MENU INTERACTIVO ---------------------------------------------------------------

	menu.ejecutar(true); // ejecutamos y liberamos todos los objetos llamando a sus destructores (true)	
	
	std::cout << "\nSalida del programa PLANTA\n";  // Mensaje final
		
	return 0;
	
}