// Planta_CARRO_SOLO.cpp: Implementación de los dispositivos de planta del Carro. Basado en MotorCC (carro eléctrico)

// Obsérvese que los puertos a conectar son 4,5,6 en el dispositivo Carro (1,2,3 son los de MotorCC, que se conservan)

// Esta implementación no necesita controlador. El usuario aquí puede averiguar el funcionamiento del carro 
// por sí solo, como velocidad, voltaje, etc. gracias a un generador de voltaje. 

// La acción de voltaje sin controlador se realiza por el puerto 6 del dispositivo de carro. A semejanza del motorCC. 

//#include "base.h"

#include "sistemas/aleatorio.h"
#include "sistemas/carro_motorizado.h"
#include "sistemas/constante.h"
#include "base.h"

#include <iostream>

int main()
{	

// CREACION DE WATCHDOGS ---------------------------------------------------------------------

// CREACION DE CONTROLADORES -----------------------------------------------------------------

	
// CREACION DE DISPOSITIVOS DE FISICOS DE PLANTA, GENERADORES Y FUNCIONES MATEMATICAS --------

	Aleatorio *rozamiento = new Aleatorio(20,2401,419,577,129,899,1002,0,0.6);		
	Constante *voltaje = new Constante(1,2402,10,100,-100);	
	Carro *carro = new Carro(500,2403,0.001,20,0.1,0.15,-0.1,-0.15,0); 
 

// CREACION DE BUSES -------------------------------------------------------------------------
	
	Bus *bus = new Bus(3401, PROCESO_PLANTA);	

// CREACION DE DISCRETIZADORES ---------------------------------------------------------------
	
	Discretizador *discret_car = new Discretizador(50,4401,2, bus);		
	
// CREACION DE CONECTORES CONVENCIONALES -----------------------------------------------------
	
//	Conector *car_dis = new Conector(6401); //Entre carro y discretizador	
//	Conector *roz_car = new Conector(6402); // Entre rozamiento y carro	
//  Conector *vol_car = new Conector(6403); // Entre voltaje y carro	

										   
// CREACION DE GRABADORES --------------------------------------------------------------------
	
	Grabador *grabador = new Grabador(10,8401, PROCESO_PLANTA, false, 500); // 500 kbytes de grabacion max

	
// CREACION DE LOS CONECTORES DE RECOLECCION DE DATOS (para grabar)---------------------------	
											
	Conector_rec *car_dis = new Conector_rec(7401, grabador); //Entre Carro y discretizador	
	Conector_rec *roz_car = new Conector_rec(7402, grabador); // Entre rozamiento y carro	
	Conector_rec *vol_car = new Conector_rec(7403, grabador); // Entre voltaje y carro		
	
// CONECTAMOS TODOS LOS COMPONENTES	----------------------------------------------------------	
		
	voltaje -> conectar(vol_car,1); // voltaje-carro
	carro -> conectar(vol_car,6); // Puerto 6 (OJO: Ahora conectamos el carro, y es puerto 6 
	
	rozamiento -> conectar(roz_car,1); // rozamiento-carro
	carro -> conectar(roz_car,5);	
	
	carro -> conectar(car_dis, 4); // Motor-discretizador	
	discret_car -> asociar(car_dis); // El discretizador ha de capturar los datos de esid, del motor (asociar)
									// Obtiene datos de E/S con este conector (aplicar en todos los discretizadores)
	
	discret_car -> conectar(car_dis, 1); // Tras asociar, conectar en 1 
	
// CREAMOS SONDAS PARA GRABAR DATOS A ARCHIVO EXTERNO ----------------------------------------	
	
	car_dis -> crea_sonda(false, 0, FLOAT, "Velocidad");
	car_dis -> crea_sonda(false, 0+sizeof(float), FLOAT, "Posición");
	car_dis -> crea_sonda(false, 0+2*sizeof(float), FLOAT, "Intensidad");
	car_dis -> crea_sonda(false, 0+3*sizeof(float), BOOL, "detector_derecha");	
    car_dis -> crea_sonda(false, 0+3*sizeof(float)+sizeof(bool), BOOL, "detector_izquierda");
	car_dis -> crea_sonda(false, 0+3*sizeof(float)+2*sizeof(bool), BOOL, "Alarma");
	roz_car -> crea_sonda(false, 0, FLOAT, "Rozamiento");	
	vol_car -> crea_sonda(false, 0, FLOAT, "Voltaje");

// CREAMOS EL MENU INTERACTIVO ---------------------------------------------------------------
	
	Interactivo menu;
	
// REGISTRAMOS TODOS LOS DISPOSITIVOS EN EL MENU INTERACTIVO
	
	menu.registro(voltaje, "Voltaje_entrada");	
	menu.registro(rozamiento, "Rozamiento");
	menu.registro(carro, "Carro");
	menu.registro(vol_car, "conector vol_car");
	menu.registro(roz_car, "conector roz_car");
	menu.registro(car_dis, "conector car_dis");
	menu.registro(discret_car, "discretizador_car");
	menu.registro(bus, "Bus");
	menu.registro(grabador, "Grabador");
	
// EJECUTAMOS MENU INTERACTIVO ---------------------------------------------------------------

	menu.ejecutar(true); // ejecutamos y liberamos todos los objetos llamando a sus destructores (true)	
	
	std::cout << "\nSalida del programa PLANTA\n";  // Mensaje final
		
	return 0;
	
}