
// PLANTA_DEPOSITO_SOLO.CPP: Implementación del deposito. Ensayo y prueba. 

// Se utilizará puerto 1 para el caudal de entrada del depósito (caudal positivo)

// Esta implementación utiliza el controlador contro_deposito.cpp que se ha de iniciar como otro proceso

// Se colocan sondas de medición y se ensaya el dispositivo, teniendo en cuenta limitaciones del modelo 
// expuestas en la memoria de trabajo 

//#include "base.h"

#include "sistemas/aleatorio.h"
#include "sistemas/deposito.h"
#include "sistemas/constante.h"
#include "base.h"

#include <iostream>

int main()
{	

// CREACION DE WATCHDOGS ---------------------------------------------------------------------

// CREACION DE CONTROLADORES -----------------------------------------------------------------

	
// CREACION DE DISPOSITIVOS DE FISICOS DE PLANTA, GENERADORES Y FUNCIONES MATEMATICAS --------

	Constante *coef_des = new Constante(1,2601,0.8,0,1); // Coeficiente descarga típico: 0.8	
	
	Deposito *deposito = new Deposito(200,2603,0.01,2,0.1,0,0,0.8); // Modelo físico del depósito 
							// Abertura en el sumidero de 0.01 m^2 (cuadrado de 10 cm x 10 cm)
 

// CREACION DE BUSES -------------------------------------------------------------------------
	
	Bus *bus = new Bus(3701, PROCESO_PLANTA);	

// CREACION DE DISCRETIZADORES ---------------------------------------------------------------
	
	Discretizador *discret_dep = new Discretizador(50,4701,2, bus);		
	
// CREACION DE CONECTORES CONVENCIONALES -----------------------------------------------------
										   
// CREACION DE GRABADORES --------------------------------------------------------------------
	
	Grabador *grabador = new Grabador(10,8701, PROCESO_PLANTA, false, 0); // GRABACION INDEFINIDA
					// AVISO: Si el proceso en ejecución se desatiende, podría ocupar toda la memoria física y virtual
	
// CREACION DE LOS CONECTORES DE RECOLECCION DE DATOS (para grabar)---------------------------	
											
	Conector_rec *dep_dis = new Conector_rec(7701, grabador); // Entre deposito y discretizador
	Conector_rec *coef_dep = new Conector_rec(7702, grabador); // Entre coef_descarga y deposito	
	
	
// CONECTAMOS TODOS LOS COMPONENTES	----------------------------------------------------------		
	
	coef_des -> conectar(coef_dep,1); // coef_descarga a deposito
	deposito -> conectar(coef_dep,2);	
	
	deposito -> conectar(dep_dis, 1); // Motor-discretizador	
	discret_dep -> asociar(dep_dis); // El discretizador ha de capturar los datos de esid, del depósito (asociar)
									// Obtiene datos de E/S con este conector (aplicar en todos los discretizadores)
	
	discret_dep -> conectar(dep_dis, 1); // Tras asociar, conectar en 1 
	
// CREAMOS SONDAS PARA GRABAR DATOS A ARCHIVO EXTERNO ----------------------------------------	
	
	dep_dis -> crea_sonda(false, 0, FLOAT, "Altura");
	dep_dis -> crea_sonda(false, 0+sizeof(float), FLOAT, "Caudal salida");	
	dep_dis -> crea_sonda(false, 0+2*sizeof(float), BOOL, "Alarma");
	dep_dis -> crea_sonda(true, 0, FLOAT, "Caudal entrada"); // Procede del controlador (sentido contrario -> true)	
	coef_dep -> crea_sonda(false, 0, FLOAT, "ks descarga");

// CREAMOS EL MENU INTERACTIVO ---------------------------------------------------------------
	
	Interactivo menu;
	
// REGISTRAMOS TODOS LOS DISPOSITIVOS EN EL MENU INTERACTIVO	
		
	menu.registro(coef_des, "Coef_descarga");
	menu.registro(deposito, "Depósito");
	menu.registro(coef_dep, "Conector coef_descarga");
	menu.registro(dep_dis, "conector dep_dis");
	menu.registro(discret_dep, "discretizador_dep");
	menu.registro(bus, "Bus");
	menu.registro(grabador, "Grabador");
	
// EJECUTAMOS MENU INTERACTIVO ---------------------------------------------------------------

	menu.ejecutar(true); // ejecutamos y liberamos todos los objetos llamando a sus destructores (true)	
	
	std::cout << "\nSalida del programa PLANTA\n";  // Mensaje final
		
	return 0;
	
}