
// PLANTA_DEPOSITO_SOLO.CPP: Implementación del deposito. Ensayo y prueba. 

// Se utilizará puerto 3 para el caudal de entrada del depósito (caudal positivo)

// Esta implementación no necesita controlador.

// Se colocan sondas de medición y se ensaya el dispositivo, teniendo en cuenta limitaciones del modelo 
// expuestas en la memoria de trabajo 

//#include "base.h"

#include "sistemas/aleatorio.h"
#include "sistemas/deposito.h"
#include "sistemas/constante.h"
#include "sistemas/combinacion_lineal.h"
#include "base.h"

#include <iostream>

int main()
{	

// CREACION DE WATCHDOGS ---------------------------------------------------------------------

// CREACION DE CONTROLADORES -----------------------------------------------------------------

	
// CREACION DE DISPOSITIVOS DE FISICOS DE PLANTA, GENERADORES Y FUNCIONES MATEMATICAS --------

	Aleatorio *coef_des = new Aleatorio(20,2601,419,577,129,899,1002,0.79,0.81); // Coeficiente descarga típico: 0.8
	
	Constante *caudal_entrada = new Constante(1,2602,0.044,100,0); // Inicialmente 0.044 (En m^3 / s) -> 44 L /s
	Aleatorio *fluctuacion_caudal = new Aleatorio(5,2603,193,577,599,774,1002,0.001,-0.0001); // +- 1 L / s 
				// Se recomienda que los generadores pseudaleatorios tengan diferentes semillas; y
				// si tienen las mismas, que difieran en muestreo; de esta forma se comportarán como fuentes distintas.
	
	Combinacion *suma = new Combinacion(5,2604,1,1); // Sumador
	
	Deposito *deposito = new Deposito(200,2603,0.01,2,0.1,0.1,0,0.8); // Modelo físico del depósito 
							// Abertura en el sumidero de 0.01 m^2 (cuadrado de 10 cm x 10 cm)
 

// CREACION DE BUSES -------------------------------------------------------------------------
	
	Bus *bus = new Bus(3601, PROCESO_PLANTA);	

// CREACION DE DISCRETIZADORES ---------------------------------------------------------------
	
	Discretizador *discret_dep = new Discretizador(50,4601,2, bus);		
	
// CREACION DE CONECTORES CONVENCIONALES -----------------------------------------------------
										   
// CREACION DE GRABADORES --------------------------------------------------------------------
	
	Grabador *grabador1 = new Grabador(10,8601, PROCESO_PLANTA, false, 0); // GRABACION INDEFINIDA
				// AVISO: Si el proceso en ejecución se desatiende, podría ocupar toda la memoria física y virtual
				
	Grabador *grabador2 = new Grabador(10,8602, PROCESO_PLANTA, false, 0); // GRABACION INDEFINIDA
				// AVISO: Si el proceso en ejecución se desatiende, podría ocupar toda la memoria física y virtual
	
// CREACION DE LOS CONECTORES DE RECOLECCION DE DATOS (para grabar)---------------------------	
	
	Conector_rec *dep_dis = new Conector_rec(7601, grabador1); // Entre deposito y discretizador
	Conector_rec *cau_sum = new Conector_rec(7602, grabador2); //Entre caudal y suma	
	Conector_rec *flu_sum = new Conector_rec(7603, grabador2); //Entre fluctuación y suma	
	Conector_rec *sum_dep = new Conector_rec(7604, grabador2); //Entre suma y deposito	
	Conector_rec *coef_dep = new Conector_rec(7605, grabador2); // Entre coef_descarga y deposito	
	
	
// CONECTAMOS TODOS LOS COMPONENTES	----------------------------------------------------------	
		
	suma -> conectar(sum_dep,3); // Puerto 3 del sumador
	deposito -> conectar(sum_dep,3); // Puerto 3 (deposito autónomo, sin controlador)
	
	caudal_entrada -> conectar(cau_sum,1); // Puerto 1 de la constante
	suma -> conectar(cau_sum,1); // Puerto 1 del sumador
	
	fluctuacion_caudal -> conectar(flu_sum,1); // Puerto 1 del generador de números aleatorios
	suma -> conectar(flu_sum,2); // Puerto 2 del sumador
	
	
	coef_des -> conectar(coef_dep,1); // coef_descarga a deposito
	deposito -> conectar(coef_dep,2);	
	
	deposito -> conectar(dep_dis, 1); // Motor-discretizador	
	discret_dep -> asociar(dep_dis); // El discretizador ha de capturar los datos de esid, del depósito (asociar)
									// Obtiene datos de E/S con este conector (aplicar en todos los discretizadores)
	
	discret_dep -> conectar(dep_dis, 1); // Tras asociar, conectar en 1 
	
// CREAMOS SONDAS PARA GRABAR DATOS A ARCHIVO EXTERNO ----------------------------------------	
	
	dep_dis -> crea_sonda(false, 0, FLOAT, "Altura");
	dep_dis -> crea_sonda(false, 0+sizeof(float), FLOAT, "Caudal salida");	// Grupo Grabador 1
	dep_dis -> crea_sonda(false, 0+2*sizeof(float), BOOL, "Alarma");
	
	sum_dep -> crea_sonda(false, 0, FLOAT, "Caudal total entrada");	
	cau_sum -> crea_sonda(false, 0, FLOAT, "Caudal fuente");                // Grupo Grabador 2
	flu_sum -> crea_sonda(false, 0, FLOAT, "Caudal fluctuación");
	coef_dep -> crea_sonda(false, 0, FLOAT, "ks descarga");

// CREAMOS EL MENU INTERACTIVO ---------------------------------------------------------------
	
	Interactivo menu;
	
// REGISTRAMOS TODOS LOS DISPOSITIVOS EN EL MENU INTERACTIVO
	
	menu.registro(caudal_entrada, "Caudal entrada");	
	menu.registro(fluctuacion_caudal, "Fluctuacion");
	menu.registro(suma, "Sumador");
	menu.registro(coef_des, "Coef_descarga");
	menu.registro(deposito, "Depósito");
	
	menu.registro(sum_dep, "Conector suma-dep");
	menu.registro(cau_sum, "Conector caudal-suma");
	menu.registro(flu_sum, "Conector fluctuacion-suma");
	menu.registro(coef_dep, "Conector coef_descarga");
	menu.registro(dep_dis, "Conector dep_dis");
	menu.registro(discret_dep, "discretizador_dep");
	menu.registro(bus, "Bus");
	menu.registro(grabador1, "Grabador1");
	menu.registro(grabador2, "Grabador2");
	
// EJECUTAMOS MENU INTERACTIVO ---------------------------------------------------------------

	menu.ejecutar(true); // ejecutamos y liberamos todos los objetos llamando a sus destructores (true)	
	
	std::cout << "\nSalida del programa PLANTA\n";  // Mensaje final
		
	return 0;
	
}