// PLANTA_MOTAMO - Planta de motorCC y amortiguador combinados. Ejemplo de escalado. Dispositivos de
// diferente naturaleza conviviendo en el mismo proceso


//#include "base.h"
#include "sistemas/aleatorio.h"
#include "sistemas/funciones_periodicas.h"
#include "sistemas/motorcc.h"
#include "sistemas/amortiguador.h"
#include "sistemas/constante.h"
#include "base.h"

#include <iostream>

int main()
{	

// CREACION DE CONTROLADORES -----------------------------------------------------------------

	
// CREACION DE DISPOSITIVOS DE FISICOS DE PLANTA, GENERADORES Y FUNCIONES MATEMATICAS --------

	Periodica *par_resistente = new Periodica(10,2102,F_CUADRADA,200,200,0,400);
	MotorCC *motor = new MotorCC(500,2103,10,0.1,0.3,220,30,1.9,0,0,0,0,0,20); 
	
	Aleatorio *perturbacion_actuador = new Aleatorio(30,2201,419,577,129,899,1002,-18,18);	
	Periodica *calzada = new Periodica(100,2202,F_TRIANGULAR,300,300,-0.4,0.4);
	Amortiguador *amortiguador = new Amortiguador(500,2203,250,20,15000,150000,3150,0.,0.,0.,0.,1,10000,10000); 

// CREACION DE BUSES -------------------------------------------------------------------------
	
	Bus *bus = new Bus(3301, PROCESO_PLANTA);	

// CREACION DE DISCRETIZADORES ---------------------------------------------------------------
	
	Discretizador *discret_mot = new Discretizador(50,4301,2, bus);		
	Discretizador *discret_amo = new Discretizador(50,4302,2, bus);	

// CREACION DE WATCHDOGS ---------------------------------------------------------------------
	
	
// CREACION DE CONECTORES CONVENCIONALES -----------------------------------------------------	

	Conector *mot_dis = new Conector(6101); //Entre motor y discretizador	
	Conector *par_mot = new Conector(6102); // Entre par y motor		
	
											
	Conector *amo_dis = new Conector(6201); //Entre amortiguador y discretizador	
	Conector *per_amo = new Conector(6202); //Entre "ruido" (perturbaciones) y actuador del amortiguador
												   // (imperfecciones del actuador)
	Conector *cal_amo = new Conector(6203); 
										// Entre calzada (generador función periodica) y rueda del amortiguador
										   
// CREACION DE GRABADORES --------------------------------------------------------------------
	
//	Grabador *grabador1 = new Grabador(10,8301, PROCESO_PLANTA, false, 500); // 500 kbytes de grabacion max
//	Grabador *grabador2 = new Grabador(10,8302, PROCESO_PLANTA, false, 500); 

	
// CREACION DE LOS CONECTORES DE RECOLECCION DE DATOS (para grabar)---------------------------
	
//	Conector_rec *mot_dis = new Conector_rec(7101, grabador1); //Entre motor y discretizador	
//	Conector_rec *par_mot = new Conector_rec(7102, grabador1); // Entre par y motor		
	
											
//	Conector_rec *amo_dis = new Conector_rec(7201, grabador2); //Entre amortiguador y discretizador	
//	Conector_rec *per_amo = new Conector_rec(7202, grabador2); //Entre "ruido" (perturbaciones) y actuador del amortiguador
												   // (imperfecciones del actuador)
//	Conector_rec *cal_amo = new Conector_rec(7203, grabador2); 
										// Entre calzada (generador función periodica) y rueda del amortiguador
	
// CONECTAMOS TODOS LOS COMPONENTES	----------------------------------------------------------	
		
	par_resistente -> conectar(par_mot,1); // par-motor
	motor -> conectar(par_mot,2);
	
	
	motor -> conectar(mot_dis, 1); // Motor-discretizador	
	discret_mot -> asociar(mot_dis); // El discretizador ha de capturar los datos de esid, del motor (asociar)
									// Obtiene datos de E/S con este conector (aplicar en todos los discretizadores)
	
	discret_mot -> conectar(mot_dis, 1); 	
	
	perturbacion_actuador -> conectar(per_amo,1); // amortiguador-perturbacion
	amortiguador -> conectar(per_amo,3);
	
	calzada -> conectar(cal_amo,1); // Amortiguador-calzada
	amortiguador -> conectar(cal_amo,2);
	
	
	amortiguador -> conectar(amo_dis, 1); // Amortiguador-Discretizador		
	discret_amo -> asociar(amo_dis); 
									// Obtiene datos de E/S con este conector (aplicar en todos los discretizadores)
	
	discret_amo -> conectar(amo_dis, 1); 
	
// CREAMOS SONDAS PARA GRABAR DATOS A ARCHIVO EXTERNO ----------------------------------------	
	
/*	mot_dis -> crea_sonda(false, 0, FLOAT, "Vel. angular");
	mot_dis -> crea_sonda(false, 0+sizeof(float), FLOAT, "Pos. angular");
	mot_dis -> crea_sonda(false, 0+2*sizeof(float), FLOAT, "Intensidad");
	mot_dis -> crea_sonda(false, 0+3*sizeof(float), BOOL, "Alarma");	
	
	par_mot -> crea_sonda(false, 0, FLOAT, "Par_externo");	
	
	amo_dis -> crea_sonda(false, 0, FLOAT, "Pos_chasis");
	amo_dis -> crea_sonda(false, 0+sizeof(float), FLOAT, "Pos_rueda");
	amo_dis -> crea_sonda(false, 0+sizeof(float)+sizeof(float), BOOL, "Alarma");
	amo_dis -> crea_sonda(true, 0, FLOAT, "Actuador");
	cal_amo -> crea_sonda(false, 0, FLOAT, "Calzada");
	per_amo -> crea_sonda(false, 0, FLOAT, "Perturbacion");*/

// CREAMOS EL MENU INTERACTIVO ---------------------------------------------------------------
	
	Interactivo menu;
	
// REGISTRAMOS TODOS LOS DISPOSITIVOS EN EL MENU INTERACTIVO
	
	menu.registro(par_resistente, "Par_resistente");
	menu.registro(motor, "Motor");
	menu.registro(par_mot, "conector par_mot");
	menu.registro(mot_dis, "conector mot_dis");
	menu.registro(discret_mot, "Discretizador motor");		
	menu.registro(perturbacion_actuador, "perturbacion");	
	menu.registro(calzada, "calzada");
	menu.registro(amortiguador, "amortiguador");
	menu.registro(cal_amo, "conector calzada-amort");
	menu.registro(amo_dis, "conector amort-discret");
	menu.registro(per_amo, "conector per-amort");
	menu.registro(discret_amo, "Discretizador amortiguador");
	menu.registro(bus, "Bus");
//	menu.registro(grabador1, "Grabador MOTOR");
//	menu.registro(grabador2, "Grabador AMORT");

	
// EJECUTAMOS MENU INTERACTIVO ---------------------------------------------------------------

	menu.ejecutar(true); // ejecutamos y liberamos todos los objetos llamando a sus destructores (true)	
	
	std::cout << "\nSalida del programa PLANTA\n";  // Mensaje final
		
	return 0;
	
}