// Aleatorio.cpp
//
// Implementación del dispositivo de números aleatorios

#include "aleatorio.h"
#include <iostream> 

Aleatorio::Aleatorio(unsigned int t, int id, unsigned int k1, unsigned int k2, unsigned int k3, unsigned int k4, 
												unsigned int modulo, float sup, float inf) : Dinamico(t,id) { 
												//constructor
	Aleatorio::configura(k1, k2, k3, k4, modulo, sup, inf); // llamada al configurador
	
												
}

void Aleatorio::configura(unsigned int k1, unsigned int k2, unsigned int k3, unsigned int k4, 
												unsigned int modulo, float sup, float inf) { //configurador
												
													
	std::vector<unsigned int> inicial(1,4); // Vector de un elemento, de valor entero 4 (4 = longitud de cola) 
	
	Dinamico<unsigned int>::configura(inicial); // Llamada al configurador base
	
	Dinamico::estado[0][0] = k4; // Ponemos las cuatro semillas en la cola
	Dinamico::estado[0][1] = k3;
	Dinamico::estado[0][2] = k2;
	Dinamico::estado[0][3] = k1;
	
	Aleatorio::modulo = modulo; // Valores para la generación de los números aleatorios
	if (sup >= inf) { 
		Aleatorio::sup = sup;
		Aleatorio::inf = inf;
	} else {  // invertir superior e inferior en caso de que inf > sup
		Aleatorio::sup = inf;
		Aleatorio::inf = sup;
	}
	Aleatorio::rango = Aleatorio::sup - Aleatorio::inf; // Rango de números
	
	Aleatorio::sup_m = Aleatorio::sup; // Hacemos copia en modificables 
	Aleatorio::inf_m = Aleatorio::inf;
	Aleatorio::rango_m = Aleatorio::rango;
	
	// Habilitar el puerto para ser conectado
	
	//AVISO: Las salidas desde el dispositivo son entradas desde el conector
	//       Una entrada se considera salida y viceversa, al crear las conexiones.
	//		 Esto es aplicable en todas las conexiones y hay que tenerlo en cuenta al crear el dispositivo
	
	Conectable::nuevo_puerto(1,2,(BYTE *)&Aleatorio::es_2_1.salida,(BYTE *)nullptr,
							sizeof(Aleatorio::es_2_1.salida), 0); // Habilita sólo el puerto 1. 
																  //(dispositivo de solo salida)

												
}

int Aleatorio::actualiza() { // Rutina de actualización por cada iteracción de muestreo
	
	int resultado = 0;
	unsigned int aditivo;

	if(Aleatorio::modificado) { // ¿Hubo modificación en propiedades?
		Aleatorio::sup = Aleatorio::sup_m; // Actualizamos valores
		Aleatorio::inf = Aleatorio::inf_m; 
		Aleatorio::rango = Aleatorio::rango_m; 
		
		Aleatorio::modificado = false;
	}
		
	aditivo = Dinamico::estado[0].front() + Dinamico::estado[0].back(); // Algoritmo de generación de números
	Dinamico::siguiente[0] = aditivo % Aleatorio::modulo;				// aleatorios. Nueva semilla en "siguiente"
																		// Método congruencial aditivo
	
	Aleatorio::es_2_1.salida.s1 = ((float)(Dinamico::siguiente[0] / 
								(float)(modulo - 1))) * Aleatorio::rango + Aleatorio::inf;	
																	// Ya tenemos nuevo número aleatorio
																	// Lo ajustamos para una distribución
																	// uniforme entre sup e inf.
																	// No hace falta más procesos y lo enviamos al puerto
																	 
												 
																	 
//	std::cout << Aleatorio::es_2_1.salida.aleatorios << "\n"; ///////////// DEPURACION ///////////
			
	resultado += Dinamico<unsigned int>::actualiza(); // Actualizar clase base, incorporamos nueva semilla
													  // en la cola 
	return resultado;
	
}

void Aleatorio::rutina() { // Rutina de thread 
	
	while (Dispositivo::obtiene_ejecucion()) {
		
		Aleatorio::actualiza();
	
	}
	
}

void Aleatorio::propiedades() { //Propiedades en pantalla del generador aleatorio. 
	
	unsigned int opcion; 	
	float sup, inf; // valores de entrada de teclado
	bool editable = false; // ¿editable?

	do {
		std::cout << "\n\n-------- PROPIEDADES DEL GENERADOR ALEATORIO ------------------------\n\n";	
		
		
		std::cout << "\t\tInferior:\t" << Aleatorio::inf_m;
		std::cout << "\n\t\tSuperior:\t" << Aleatorio::sup_m;
		std::cout << "\n\t\tRango:\t" << Aleatorio::rango_m;
		
		if(!Aleatorio::modificado && Dispositivo::obtiene_ejecucion()) { // Mostrar opciones si son modificables 
			std::cout << "\n\n\t1) Modificar valor inferior";
			std::cout << "\n\t2) Modificar valor superior";
			editable = true;
		} else {
			std::cout << "\n\nLos valores no son modificables en este momento, hasta que los procese el dispositivo."
					  << "\nSi el dispositivo está parado/pausado hay que iniciarlo para hacer efectivos los cambios" 
					  << "\nPulse cualquier valor que no sea 0 para actualizar esta pantalla o 0 para salir.";
			editable = false;
		}
		std::cout << "\n\n\t0) Salir";
		
		std::cout << "\n\n---------------------------------------------------------------------\n";
		
		std::cout << "\nSeleccione opción -> "; 
		std::cin >> opcion; //espera orden de teclado			 
		
		if(editable) { // Solo se pueden modificar si son editables
			switch(opcion) {
				case 1:
					std::cout << "\n\nIntroduzca nuevo valor inferior -> ";
					std::cin >> inf; //espera orden de teclado
					if(inf <= Aleatorio::sup_m) { // Comprobación
						Aleatorio::inf_m = inf; 	
						Aleatorio::rango_m = Aleatorio::sup_m - inf;
						Aleatorio::modificado = true; // ¿Se modificaron valores? si
					} else std::cout << "\nEl valor inferior no puede ser mayor que el superior actual\n";
					break;
				case 2:
					std::cout << "\n\nIntroduzca nuevo valor superior -> ";
					std::cin >> sup; //espera orden de teclado				
					if(Aleatorio::inf_m <= sup) { // Comprobación
						Aleatorio::sup_m = sup; 	
						Aleatorio::rango_m = sup - Aleatorio::inf_m;
						Aleatorio::modificado = true; // ¿Se modificaron valores? si
					} else std::cout << "\nEl valor superior no puede ser menor que el inferior actual\n";				
					break;			
				default:
					break;		
			};
		} 

	} while (opcion != 0);	
		
	return;
}	

void Aleatorio::propiedades_base() { //Propiedades de clase base y superclases
	
	Dinamico<unsigned int>::propiedades();
	
	return;
}
											
