// Amortiguador.cpp

#include <iostream> 
#include "amortiguador.h" // Prototipo de la clase "Amortiguador", derivada de "continuo"

Amortiguador::Amortiguador(unsigned int t, int id, float m1, float m2, float k1, float k2, 
			float amort, float chas_init, float rued_init, float act_init, float cal_init, 
					float dmax, float act_max, float act_sat) : Continuo<float>(t,id) { // constructor
			
						
	Amortiguador::configura(m1, m2, k1, k2, amort,chas_init, rued_init, act_init, cal_init, dmax, act_max, act_sat); 
																				// se configura dispositivo
	
}

void Amortiguador::configura(float m1, float m2, float k1, float k2, float amort,
									float chas_init, float rued_init, float act_init, float cal_init, 
															float dmax, float act_max, float act_sat) 
														{ //configurador
	int a = 0., b = 0., c = 0., d = 0., e = 0.;
	
	std::vector< std::vector<float> > matriz; // Matriz asociada.
	std::vector<float> ind; // Terminos independientes
						// Ambos objetos serán traspasados al configurador de la clase padre Continuo. 
						// antes, hay que darles los valores del sistema a simular (matriz asociada, término ind.)
	
	if (m1 > 0. ) {
	a = k1/m1;
	b = amort/m1;	
	
	} else { a = 0; b = 0;}
	
	if (m2 > 0. ){
	e = k2/m2;
	c = k1/m2;
	d = amort/m2;
	} else { e = 0; c = 0; d = 0;}
	
	 // Serán utilizados más adelante mientras se actualizan los valores del actuador y calzada
	Amortiguador::m1 = m1 > 0. ? m1 : 1.; // No se pueden dar valores ilegales. 
	Amortiguador::m2 = m2 > 0. ? m2 : 1.;
	
	Amortiguador::k2 = k2; 
	Amortiguador::dmax = dmax > 0. ? dmax : 1000.;
	Amortiguador::act_max = act_max > 0 ? act_max : 1000000.;
	Amortiguador::act_sat = act_sat > 0 ? act_sat : -act_sat;
	
	matriz.resize(4); matriz[0].resize(4); matriz[1].resize(4); matriz[2].resize(4); matriz[3].resize(4);
	ind.resize(4);
	
	// Rellenamos valores en matriz y condiciones iniciales en los términos independientes
	
	matriz[0][0]=0 ; matriz[0][1]=0   ; matriz[0][2]=1 ; matriz[0][3]=0;
	matriz[1][0]=0 ; matriz[1][1]=0   ; matriz[1][2]=0 ; matriz[1][3]=1;
	matriz[2][0]=-a; matriz[2][1]=a   ; matriz[2][2]=-b; matriz[2][3]=b;
	matriz[3][0]=c ; matriz[3][1]=-c-e; matriz[3][2]=d ; matriz[3][3]=-d;
	
	ind[0]=0; ind[1]=0; ind[2]=(act_init/m1)-GRAV; ind[3]=((k2*cal_init-act_init)/m2)-GRAV; // aleatorio_init = 0	
	
														 // el término independiente indep[2] se irá actualizando
														 // junto con el término indep[3] según calzada y actuador
	
										     // Conformada la matriz y el vector
	Continuo<float>::configura(matriz, ind); // los trasferimos al configurador base para inicializar el dispositivo
											 // continuo
	
	// Establecemos las condiciones iniciales de las variables de estado
	
	Dinamico<float>::estado[0][0] = chas_init;
	Dinamico<float>::estado[1][0] = rued_init;
	
	// Dinámico al iniciar colas, pone todos los valores a cero, así que no es necesario colocarlos aquí
	// puesto que la velocidad inicial de chasis y rueda la dejaremos a cero. 
	
	// Los valores iniciales, también en los puertos
	
	Amortiguador::es_1_1.salida.s1 = chas_init; // Los valores iniciales, también en los puertos
	Amortiguador::es_1_1.salida.s2 = rued_init;
	Amortiguador::es_1_1.salida.alarma = false; // Sin alarma 
	
	Amortiguador::es_1_1.entrada.e1 = act_init; // Actualicemos actuador inicial
	
	Amortiguador::es_n2_2.entrada.e1 = cal_init; // Actualizamos calzada inicial
	Amortiguador::es_n2_3.entrada.e1 = 0; // Perturbación aleatoria, a cero inicialmente
	

	// Creamos los puertos de comunicaciones. 
	
	//AVISO: Las salidas desde el dispositivo son entradas desde el conector
	//       Una entrada se considera salida y viceversa, al crear las conexiones.
	//		 Esto es aplicable en todas las conexiones y hay que tenerlo en cuenta al crear el dispositivo
	
	Conectable::nuevo_puerto(1,1,(BYTE *)&Amortiguador::es_1_1.salida,(BYTE *)&Amortiguador::es_1_1.entrada,
							sizeof(Amortiguador::es_1_1.salida), sizeof(Amortiguador::es_1_1.entrada));
							
	Conectable::nuevo_puerto(2,-2,(BYTE *)nullptr,(BYTE *)&Amortiguador::es_n2_2.entrada,
								0,sizeof(Amortiguador::es_n2_2.entrada));
							
	Conectable::nuevo_puerto(3,-2,(BYTE *)nullptr,(BYTE *)&Amortiguador::es_n2_3.entrada,
								0,sizeof(Amortiguador::es_n2_3.entrada));
	// Fin del configurador
}

int Amortiguador::actualiza() { // Rutina de actualización por cada iteración de muestreo
	
	int resultado = 0;
	
	float act; // Fuerza del actuador (será la suma de las entradas procedentes del puerto 1 y el puerto 3.
	float m1, m2, k2; 
	
	float dif, act_abs; // Diferencia de posiciones y valor absoluto del actuador (a efectos de alarma)

	resultado += Continuo<float>::actualiza(); // Actualizar clase base, incluye E/S por conectable	

	// Actualizamos los términos independientes que tengan variables de contorno según lo que nos llegó por E/S
	
	// primero las variables locales de este método (para ver más claras las operaciones en el término independiente)	
	m1 = Amortiguador::m1; m2 = Amortiguador::m2; k2 = Amortiguador::k2;
	
	act = Amortiguador::es_1_1.entrada.e1 + Amortiguador::es_n2_3.entrada.e1; // al actuador se le añade el valor 
																			  // del puerto 3
	if (act > Amortiguador::act_sat) act = Amortiguador::act_sat; // Valor saturación act_sat
	if (act < -Amortiguador::act_sat) act = Amortiguador::act_sat; // Límites de operación del elemento físico
	
	Continuo<float>::indep[2]= act/m1 - GRAV;  		
	Continuo<float>::indep[3]= ((k2 * Amortiguador::es_n2_2.entrada.e1 - act)/m2) - GRAV; 
									
								
	// Método numérico de integración Runke-Kutta de orden 4
	
	Continuo<float>::rk4(); // Aplicamos RK4 en el sistema del amortiguador para calcular siguiente estado
							// Nota: para la primera iteración tenemos que haber configurado bien la matriz
							// del sistema, el vector de términos independientes, las condiciones iniciales	
	
	// Finalmente, actualizamos puertos según los estados que nos proporciona Continuo		
						
	Amortiguador::es_1_1.salida.s1 = Dinamico<float>::estado[0][0]; // Posición chasis
	Amortiguador::es_1_1.salida.s2 = Dinamico<float>::estado[1][0]; // Posición rueda					
	
	// Aquí definimos si sobrepasamos alguna tolerancia, algún requisito, y en caso de ser así, se activa alarma.
	// Esa alarma se transmite al controlador correspondiente
	
	dif = Dinamico<float>::estado[0][0] > Dinamico<float>::estado[1][0] ? Dinamico<float>::estado[0][0] -
		Dinamico<float>::estado[1][0] : Dinamico<float>::estado[1][0] - Dinamico<float>::estado[0][0];
	
	act_abs = act > 0 ? act : -act;
	
	if (!Dispositivo::obtiene_alarma()) {
		if (dif > Amortiguador::dmax || act_abs > act_max) { // ¿Se sobrepasan tolerancias?
			Dispositivo::activa_alarma(); // Alarma activada
			Amortiguador::es_1_1.salida.alarma = true;  // Queda permanente en el puerto
		}	
	}	
	
	return resultado;
	
}

void Amortiguador::rutina() { // Rutina de thread 
	
	while (Dispositivo::obtiene_ejecucion()) {
		
		Amortiguador::actualiza();
		
		// Actualiza E/S del dispositivo
		
	}
	return; 
}