// Amortiguador.h 
//
// Amortiguador chasis-rueda-suelo en un vehículo. Consultar memoria para más información.
// Es un sistema reducible a ecuación diferencial lineal de orden cuatro; por tanto, con cuatro variables internas.
// y matriz de 4x4


#include "../base.h" // Base de dispositivos
#include "estructuras_datos.h"
#include <iostream>

#ifndef GRAV
#define GRAV 9.8 // Constante gravitatoria 9.8 m/s^2
#endif

class Amortiguador : public Continuo<float> {

	float m1, m2; // Masas del amortiguador (chasis y rueda) que influyen según el actuador en dos términos independientes
	float k2; // coeficiente elástico de la rueda
	float dmax, act_max; // distancia máxima rueda-chasis; fuerza máxima del actuador (si se sobrepasa se activa alarma)
	float act_sat; // Máxima fuerza (positiva o negativa) que puede admitir el actuador
	
	
protected: 
	void configura(float m1, float m2, float k1, float k2, 
						float amort, float chas_init, float rued_init, float act_init, 
										float cal_init, float dmax, float act_max, float act_sat); 
																//configuracion de amortiguador

	void rutina();
	int actualiza(); // Actualización
	
	// ////////// Entrada-salida del dispositivo del amortiguador
	
		
	struct ES_1 es_1_1; // Estructura de datos esid=1 al puerto 1 (entrada/salida del amortiguador al controlador)
	
	struct ES_N2 es_n2_2; // Estructura de datos esid=-2 al puerto 2 (entrada de contorno de calzada) 
	
	struct ES_N2 es_n2_3; // Estructura de datos esid=-2 al puerto 3 (entrada de perturbaciones del sistema hidraulico)
	
public:
	Amortiguador(unsigned int t, int id, float m1, float m2, float k1, float k2, 
						float amort, float chas_init, float rued_init, float act_init, 
										float cal_init, float dmax, float act_max, float act_sat); 
														// constructor del amortiguador
	
};