// Carro_motorizado.cpp

// implementación del carro motorizado 

#include <iostream> 
#include "carro_motorizado.h" // Prototipo de la clase carro_motorizado

Carro::Carro(unsigned int t, int id, float c, float m, float posdd, float postd, float posdi, 
			float posti, float posini) : MotorCC(t,id, PARAMETRIZACIONCC_CARRO1) { 
																					// constructor del carro
			
						
	Carro::configura(c, m, posdd, postd, posdi, posti, posini); // se configura el carro
	
}

void Carro::configura(float c, float m, float posdd, float postd, float posdi, float posti, float posini) { 
														//configurador
	
	
	Carro::posini = posini; // Posición inicial, será utilizado como punto cero de referencia
	
	// filtro para evitar valores no autorizados. No se pueden filtrar todas las combinaciones
	// Es responsabilidad del usuario poner valores correctos para que el sistema no pase por estados indeterminados	
	
	Carro::c = c > 0 ? c : -c;
	if(c == 0) Carro::c = 1; // c No puede ser NULO
	Carro::m = m > 0 ? m : -m;
	
	Carro::posdd = posdd;
	Carro::postd = postd;
	Carro::posdi = posdi;
	Carro::posti = posti;
		
/*	Carro::posdd = posdd > posdi ? posdd : posdi; // posdd siempre mayor que posdi 
	Carro::posdi = posdd > posdi ? posdi : posdd;
	
	Carro::postd = postd > posti ? postd : posti; // postd siempre mayor que posti 
	Carro::posti = postd > posti ? posti : postd; */
	
	Carro::es_4_4.salida.s4 = false;	
	Carro::es_4_4.salida.s5 = false;	

	if(Carro::posini >= Carro::posdd) { // Si se alcanzó el detector derecho, se pone el bit a true
		Carro::es_4_4.salida.s4 = true;
		
		if (Carro::posini > Carro::postd) // No se puede superar el tope derecho
			Carro::posini = Carro::postd;		
	} 
	
	if(Carro::posini <= Carro::posdi) { // Si se alcanzó el detector izquierdo, se pone el bit a true
		Carro::es_4_4.salida.s5 = true;		
		
		if (Carro::posini < Carro::posti) // No se puede superar el tope izquierdo
			Carro::posini = Carro::posti;		
	} 
	
	// La posición del carro es la del ángulo del motor por el coeficiente linealizador más la posición inicial
	// En la posición inicial, el ángulo del motor es cero. Por tanto, con sumar la posición inicial tendremos la 
	// posición absoluta del carro, en todo momento. 	
	
	// Establece las condiciones iniciales
	
//	Carro::pos = posini; // Posición y velocidad
//	Carro::vel = 0;
							// Los valores iniciales de posicion y velocidad lineal, también en los puertos
	Carro::es_4_4.salida.s1 = Carro::posini + c * MotorCC::es_3_1.salida.s1;  						
	Carro::es_4_4.salida.s2 = c * MotorCC::es_3_1.salida.s2;	
	Carro::es_4_4.salida.s3 = 0; // Intensidad nula
	
	Carro::o_max = (Carro::postd - Carro::posini) / Carro::c; // Calculamos giros máximos y mínimos (topes)
	Carro::o_min = (Carro::posti - Carro::posini) / Carro::c; 
	
//	std::cout << ">>>>>>>>>>>>>>>>>>>> " << Carro::o_min;
	
	// Las condiciones iniciales de los detectores ya han sido determinadas anteriormente
	
	Carro::es_4_4.salida.alarma = false; // Sin alarma 	
	
	Carro::es_4_4.entrada.e1 = 0; // De entrada, voltaje nulo como el voltaje por defecto (motor parado)
	// Esta consigna inicial de voltaje nulo también se transmite al puerto de la superclase, se indica en el 
	// constructor
	
	Carro::es_n2_5.entrada.e1 = 0; // De entrada, par nulo por rozamiento, al entenderse que está el motor parado.

	// Crea los puertos de comunicaciones. 
	
	//AVISO: Las salidas desde el dispositivo son entradas desde el conector
	//       Una entrada se considera salida y viceversa, al crear las conexiones.
	//		 Esto es aplicable en todas las conexiones y hay que tenerlo en cuenta al crear el dispositivo
	
	Conectable::nuevo_puerto(4,4,(BYTE *)&Carro::es_4_4.salida,(BYTE *)&Carro::es_4_4.entrada,
							sizeof(Carro::es_4_4.salida), sizeof(Carro::es_4_4.entrada)); // Puerto 4 al controlador 
							
	Conectable::nuevo_puerto(5,-2,(BYTE *)nullptr,(BYTE *)&Carro::es_n2_5.entrada, // Puerto 5 de entrada (rozamiento)
								0,sizeof(Carro::es_n2_5.entrada));
	
	Conectable::nuevo_puerto(6,-2,(BYTE *)nullptr,(BYTE *)&Carro::es_4_4.entrada, // Puerto 6 de entrada de voltaje
								0, sizeof(Carro::es_4_4.entrada));
								
	// Nota: El puerto 6 enmascara la entrada del puerto 4, correspondiente con el voltaje de entrada. Si queremos
	// hacer funcionar el carro sin controlador, usaremos el puerto 6, pues prevalece sobre el voltaje del puerto 4 
	
	// Obsérvese también que la asignación de puertos no ha de interferir con los puertos que se asignan en la 
	// clase base MotorCC y que no se usarán, ya que estamos creando una clase derivada que hará un uso diferente
	// del motor y con otros parámetros físicos.	
	
	return;	
	
}

int Carro::actualiza() { // Rutina de actualización por cada iteracción de muestreo
	
	int resultado = 0;

	float pos, vel; 
	
	
	// Aquí no utilizamos la integración numérica, es un trabajo de la superclase MotorCC. En esta cadena de 
	// actualizaciones nos limitamos a realizar las conversiones algebraicas con el angulo y velocidad de giro
	// para transformarlas en posición y velocidad lineal. 
	
	// Según la posición, se activarían o desactivarían los detectores. Hay que hacer una comprobación
	// Según la posición, si ésta llega a uno de los topes hay que transformar la matriz de sistema y anular
	// completamente los coeficientes de la segunda fila, que atañen al estado de la velocidad de giro 
	// También hay que garantizar que el par torsor que se opone sea siempre opuesto al par ofrecido por la
	// corriente del circuito. 	
	
	// ¿Hemos alcanzado topes? Anular velocidad, hacer que giro no sobrepase, y modificar estados
	// Si estando en el tope varió el voltaje para girar en sentido contrario, se vuelve al estado normal	
	
	resultado += MotorCC::actualiza(); // Actualizar clase base, incluye E/S por conectable	
	
	if(MotorCC::es_3_1.salida.s2 >= Carro::o_max && Carro::es_4_4.entrada.e1 >= 0) {
		Dinamico<float>::siguiente[2] = Carro::o_max; // Alcanza tope superior!
		Dinamico<float>::siguiente[1] = 0; // Velocidad angular nula
		MotorCC::es_n2_2.entrada.e1 = MotorCC::ks * MotorCC::es_3_1.salida.s3; 
		// Par de resistencia igual al par motor (ks * i). Al no haber movimiento (acción-reacción en ligadura), se desprecia el rozamiento b
		// Este par compensa el par motor por la corriente al ser una situación estática
	} else {
	
		if (MotorCC::es_3_1.salida.s2 <= Carro::o_min && Carro::es_4_4.entrada.e1 <= 0) {
			Dinamico<float>::siguiente[2] = Carro::o_min; // Alcanza Tope inferior!
			Dinamico<float>::siguiente[1] = 0; // Velocidad angular nula
			MotorCC::es_n2_2.entrada.e1 = MotorCC::ks * MotorCC::es_3_1.salida.s3; 
			// Par de resistencia igual al par motor (ks * i). Al no haber movimiento (acción-reacción en ligadura), se desprecia el rozamiento b
			// Este par compensa el par motor por la corriente al ser una situación estática
			
		} else { // Modo normal de operación, según el rozamiento habrá un caso u otro 
			
			// Par giratorio
			float par_giratorio = MotorCC::ks * MotorCC::es_3_1.salida.s3 - MotorCC::b * MotorCC::es_3_1.salida.s1;			
			
			// Rozamiento en valor absoluto (es un coeficiente positivo)
			float roz_abs = Carro::es_n2_5.entrada.e1 > 0 ? Carro::es_n2_5.entrada.e1 : -Carro::es_n2_5.entrada.e1;	

			float ts = par_giratorio > 0 ? Carro::m * GRAV * roz_abs : -Carro::m * GRAV * roz_abs; // Torsor de rozamiento
			float ts_abs = ts > 0 ? ts : -ts; // Valor absoluto del torsor de rozamiento
									
			// Situación estática. Velocidad angular nula. El par motor ha de superar ts para poder empezar a moverse
			
			if(Dinamico<float>::siguiente[1] == 0.) {
		
				float par_motor_abs = MotorCC::ks * MotorCC::es_3_1.salida.s3 > 0 ? 
					MotorCC::ks * MotorCC::es_3_1.salida.s3 : -MotorCC::ks * MotorCC::es_3_1.salida.s3; // Valor absoluto del par motor
				
				// Hallamos nuevo par contrario ( m * g * coef_rozamiento), que es contrario al movimiento
				if(ts_abs > par_motor_abs) MotorCC::es_n2_2.entrada.e1 = par_giratorio; // Par motor no puede vencer ts, futura iteración giro cero
				else MotorCC::es_n2_2.entrada.e1 = ts; // Se pudo vencer ts y empezará a girar en la siguiente iteración				
			}
			
			else {
			
			// Situación dinámica. Velocidad angular no nula. Se contrarresta. Si no hay par motor, llegará un momento en el que se detiene 
			// el mecanismo en seco, es decir, en la siguiente iteración la velocidad angular será exactamente nula. 
			
			// Podría darse un caso de inestabilidad en el supuesto de frenado, así que se hace conveniente no proporcionar fuerzas de
			// rozamientos muy elevadas con respecto al par motor, sobre todo si se usa un paso de iteración muy bajo.
			
				MotorCC::es_n2_2.entrada.e1 = Dinamico<float>::siguiente[1] > 0 ? ts_abs : -ts_abs; // Oposición al movimiento		
					
			}			

		}
	}
	
	
	vel = Carro::c * Dinamico<float>::siguiente[1];
	pos = Carro::posini + Carro::c * Dinamico<float>::siguiente[2]; // Nuevos valores de posición y velocidad
		
	// Actualización de detectores, según la posición del carro
	
	if(pos >= Carro::posdd) { // Umbral derecho
		Carro::es_4_4.salida.s4 = true;
		Carro::es_4_4.salida.s5 = false;	
	} else {
		if(pos <= Carro::posdi)	{ // Umbral izquierdo
		Carro::es_4_4.salida.s4 = false;
		Carro::es_4_4.salida.s5 = true;		
		} else {
			Carro::es_4_4.salida.s4 = false; // Entre detectores
			Carro::es_4_4.salida.s5 = false;
			
		}
	}
	
	// Finalmente, actualizamos puertos según los estados que nos proporciona Continuo		
	
	MotorCC::es_3_1.entrada.e1 = Carro::es_4_4.entrada.e1; // Voltaje de entrada carro al voltaje de entrada motor	
	
	Carro::es_4_4.salida.s1 = pos; // Posición
//	Carro::es_4_4.salida.s1 = Dinamico<float>::siguiente[1]; // Der. Velocidad angular /////////////////// DEPURACION

	Carro::es_4_4.salida.s2 = vel; // Velocidad
//	Carro::es_4_4.salida.s1 = Dinamico<float>::siguiente[2]; // Der. Posicion angular /////////////////// DEPURACION
	
	
	Carro::es_4_4.salida.s3 = MotorCC::es_3_1.salida.s3; // Intensidad	(proporcionada por la superclase MotorCC)
//	Carro::es_4_4.salida.s3 = Dinamico<float>::siguiente[1]; // Velocidad angular (proporcionada por la superclase MotorCC) ///////////// DEPURACION


	
	// Replicamos alarma. Es por exceso de corriente, pero lo maneja el MotorCC, no el carro. 
	Carro::es_4_4.salida.alarma = MotorCC::es_3_1.salida.alarma;	
	
	return resultado;
	
}

void Carro::rutina() { // Rutina de thread 
	
	while (Dispositivo::obtiene_ejecucion()) {
		
		Carro::actualiza();
		
		// Actualiza E/S del dispositivo siempre que esté en estado de ejecución
		
	}
	return; 
}