// COMBINACION_LINEAL.H
//
// Dispositivo no dinámico que realiza la función matemática de la combinación lineal
// Se le suministra dos entradas float x, y y la salida sería z = A*x + B*y con A y B constantes elegidas por el 
// usuario y modificables en tiempo real mediante la característica de propiedades();
// 

#include "../base.h" // Base de dispositivo
#include "estructuras_datos.h"


class Combinacion : public Conectable { // La clase base es Conectable. No es necesario conocer estados anteriores
										// y por tanto no lo consideraremos como dispositivo dinámico
										
// Entrada-salida del dispositivo

	struct ES_N2 es_n2_1; // Entrada esid=-2 por puerto 1
	struct ES_N2 es_n2_2; // Entrada esid=-2 por puerto 2
	struct ES_2 es_2_3; // Salida esid=2 por puerto 3

	std::atomic<float> a; // Constantes de la aplicación lineal
	std::atomic<float> b; 

protected: 	
	void rutina(); // Ruina estándar de dispositivo con bucle while esperando condición de parada por orden externa
	int actualiza(); // Actualización
	void configura(float a, float b); //configurador
	
public:
	Combinacion(unsigned int t, int id, float a, float b); // Constructor
	// No hay destructores (nada que cerrar al liberar el objeto de memoria)
	void propiedades(); // Propiedades del objeto. Aquí el usuario modifica la constante.
	
};