// CONSTANTE.H
//
// Dispositivo no dinámico generador que entrega una constante. Modificable por el usuario, en tiempo de ejecución.
//
// Es un dispositivo sencillo pero muy potente ya que estas modificaciones en tiempo de ejecución también simulan 
// escalones, o cambios súbitos de estado aprovechables para los sistemas físicos. También se puede usar como consigna
// para los controladores, y otras aplicaciones que se consideren oportunas. 

#include "../base.h" // Base de dispositivo
#include "estructuras_datos.h"

class Constante : public Conectable { // La clase base es Conectable. No es necesario conocer estados anteriores
										// y por tanto no lo consideraremos como dispositivo dinámico
											     

	
// Entrada-salida del dispositivo

	struct ES_2 es_2_1; // Salida esid=2 por puerto 1
	
	std::atomic<float> valor; // Valor inicial proporcionado en el constructor y que posteriormente se puede modificar
	
	float limite_superior; // Valor superior admisible
	float limite_inferior; // Valor inferior admisible
	

protected: 
	void configura(float valor, float superior, float inferior); // Configurador
	void rutina(); // Ruina estándar de dispositivo con bucle while esperando condición de parada por orden externa
	int actualiza(); // Actualización
	
public:
	Constante(unsigned int t, int id, float valor, float superior, float inferior); // Constructor
	// No hay destructores (nada que cerrar al liberar el objeto de memoria)
	void propiedades(); // Propiedades del objeto. Aquí el usuario modifica la constante.
	
};