// CONTROL_AMORTIGUADOR.CPP



#include <iostream> 
#include "controlador_amortiguador.h" // Prototipo de la clase "Controlador_amortiguador"

Control_amortiguador::Control_amortiguador(unsigned int t, int id, Watchdog<float> *watchdog, 
										float p, float i, float d) : Control_PID<float>(t, id, watchdog, p, i, d) {
																	// Control_PID
																// constructor del controlador
	Control_amortiguador::configura(); // Configurador		
	
	
}

void Control_amortiguador::configura() { //configurador	
	
	
	// Establece las condiciones iniciales
	
	Control_amortiguador::es_n1_1.entrada.e1 = 0;  // Condiciones iniciales de entrada, salida, nulas
	Control_amortiguador::es_n1_1.entrada.e2 = 0;
	Control_amortiguador::es_n1_1.entrada.alarma = false; // Sin alarma
	Control_amortiguador::es_n1_1.salida.s1 = 0; 
	
	Control_amortiguador::es_n2_2.entrada.e1 = 0;
	Control_amortiguador::es_2_3.salida.s1 = 0;	
	
	// Crea los puertos de comunicaciones. 
	
	//AVISO: Las salidas desde el dispositivo son entradas desde el conector
	//       Una entrada se considera salida y viceversa, al crear las conexiones.
	//		 Esto es aplicable en todas las conexiones y hay que tenerlo en cuenta al crear el dispositivo
	
	Conectable::nuevo_puerto(1,-1,(BYTE *)&Control_amortiguador::es_n1_1.salida,
							(BYTE *)&Control_amortiguador::es_n1_1.entrada,
							sizeof(Control_amortiguador::es_n1_1.salida), sizeof(Control_amortiguador::es_n1_1.entrada));
							
	Conectable::nuevo_puerto(2,-2,(BYTE *)nullptr,(BYTE *)&Control_amortiguador::es_n2_2.entrada,
								0, sizeof(Control_amortiguador::es_n2_2.entrada));
							
	Conectable::nuevo_puerto(3,2,(BYTE *)&Control_amortiguador::es_2_3.salida,(BYTE *)nullptr,
								sizeof(Control_amortiguador::es_2_3.salida),0);
						
	return;
	
}

int Control_amortiguador::actualiza() { // Rutina de actualización por cada iteracción de muestreo
	
	int resultado = 0;
	
	// Esto es un COMPARADOR (consigna menos señal de la planta)
	Control_PID<float>::x = Control_amortiguador::es_n2_2.entrada.e1 - Control_amortiguador::es_n1_1.entrada.e1;
		
	Control_amortiguador::es_n1_1.salida.s1 = Control_PID<float>::pid(); // Hacemos uso del PID	
	Control_amortiguador::es_2_3.salida.s1 = Control_amortiguador::es_n1_1.salida.s1; // Replicamos salida en puerto 3	
	
//	std::cout << Control_amortiguador::es_n1_1.entrada.e1	<< " "; DEPURACION
	
	if(Control_amortiguador::es_n1_1.entrada.alarma) Dispositivo::activa_alarma(); // Activamos alarma del controlador
																		// Si se activó la del dispositivo de planta																		
		
			
	resultado += Control_PID<float>::actualiza(); // Actualizar clase base	
	
	return resultado;
	
}

void Control_amortiguador::rutina() { // Rutina de thread del control amortiguador 

	Controlador<float>::estado_ejecucion = std::promise<long unsigned int>(); 
									// Se borra promesa anterior igualando a constructor vacío 
	
	Controlador<float>::futuro_estado_ejecucion = Controlador<float>::estado_ejecucion.get_future(); // Adquirimos futuro
	
	Controlador<float>::estado_ejecucion.set_value_at_thread_exit(CODIGO_FIN_HILO);
	
	while (Dispositivo::obtiene_ejecucion()) {
		
		Control_amortiguador::actualiza();
		
		// Actualiza E/S del dispositivo
		
	}
	return; 
}

/* void Control_amortiguador::propiedades() { // propiedades del control de amortiguacion		
		
	
	return; 
}*/