// CONTROLADOR_CARRO_MOTORIZADO.CPP

// Control completo del carro motor, con dos consignas: voltaje (velocidad) y duración de parada


#include <iostream> 
#include "controlador_carro_motorizado.h" // Prototipo de la clase "Controlador_carro_motorizado"

Control_carro::Control_carro(unsigned int t, int id, Watchdog<float> *watchdog, bool movimiento) 
													: Controlador<float>(t, id, watchdog) {
																	
																// constructor del controlador
	Control_carro::configura(movimiento); // Configurador		
	
	
}

void Control_carro::configura(bool movimiento) { //configurador	
					// movimiento: movimiento inicial false -> a la derecha ; true -> a la izquierda
	
	// Establece las condiciones iniciales	
	
	Control_carro::es_n4_1.entrada.alarma = false; // Sin alarma
	Control_carro::es_n4_1.salida.s1 = 0; // Salida a planta (voltaje a motor) inicialmente nula
	
	
	Control_carro::es_n2_2.entrada.e1 = 0; // Entradas inicialmente nulas
	Control_carro::es_n2_3.entrada.e1 = 0;	
	
	Control_carro::movimiento = movimiento; // Se establece movimiento
	
	// Crea los puertos de comunicaciones. 
	
	//AVISO: Las salidas desde el dispositivo son entradas desde el conector
	//       Una entrada se considera salida y viceversa, al crear las conexiones.
	//		 Esto es aplicable en todas las conexiones y hay que tenerlo en cuenta al crear el dispositivo
	
	Conectable::nuevo_puerto(1,-4,(BYTE *)&Control_carro::es_n4_1.salida,
							(BYTE *)&Control_carro::es_n4_1.entrada,
							sizeof(Control_carro::es_n4_1.salida), sizeof(Control_carro::es_n4_1.entrada));
							
	Conectable::nuevo_puerto(2,-2,(BYTE *)nullptr,(BYTE *)&Control_carro::es_n2_2.entrada,
								0, sizeof(Control_carro::es_n2_2.entrada));
							
	Conectable::nuevo_puerto(3,-2,(BYTE *)nullptr,(BYTE *)&Control_carro::es_n2_3.entrada,
								0, sizeof(Control_carro::es_n2_3.entrada));
							
						
	return;
	
}

int Control_carro::actualiza() { // Rutina de actualización por cada iteracción de muestreo
	
	int resultado = 0;
	
	float voltaje;
	bool modo_conteo = Control_carro::contador > 0 ? true : false; // Modo de conteo (si llegó a uno de los detectores)
	
	if(!Control_carro::parada) {
	
		voltaje = Control_carro::es_n2_2.entrada.e1 > 0 ? Control_carro::es_n2_2.entrada.e1
							: -Control_carro::es_n2_2.entrada.e1; // voltaje = consigna V (en valor absoluto)
		
		voltaje = Control_carro::movimiento ? -voltaje : voltaje; 
									// Según el movimiento, el voltaje tiene un sentido u otro
	} else voltaje = 0; 
	
	// Nota: el carro no puede cambiar de sentido durante la marcha. Se pueden habilitar más botones o controles
	// para ello, según la programación del controlador. Se ha implementado, como ejemplo, el de "Parada", que hace
	// el voltaje nulo, sea cual sea la consigna, y que es modificable en propiedades(). 
	
	if(((Control_carro::es_n4_1.entrada.e4 && !Control_carro::movimiento) || (
				Control_carro::es_n4_1.entrada.e5 && Control_carro::movimiento)) && !modo_conteo) 
												// ¿Se activó algún detector según la dirección de movimiento y
												// no había conteo? (El modo conteo deja de contemplar detectores
												// para evitar efectos rebote)

		modo_conteo = true;   	// Entonces modo conteo	

//	std::cout << (long unsigned int)Control_carro::es_n2_3.entrada.e1 << "\n"; // Depuración
					
	if(modo_conteo) { // Hay modo conteo
					  // Contamos hasta el límite de conteo establecido por consigna
		
		voltaje = 0; // Motor parado en la cuenta	
		
		
		if (Control_carro::contador == (long unsigned int)Control_carro::es_n2_3.entrada.e1) {			
			
			Control_carro::contador = 0; // Si acaba la cuenta, se activa fin de parada
			
			Control_carro::movimiento = !Control_carro::movimiento; // Invertimos movimiento; 
			
			voltaje = -1 * voltaje; // Inversión de voltaje.
																	  // derecha -> voltaje positivo
																	  // izquierda -> voltaje negativo
		
		} else Control_carro::contador++;
	
	} 		
	
	Control_carro::es_n4_1.salida.s1 = voltaje; // Salida al puerto del voltaje, hacia el carro
		
	if(Control_carro::es_n4_1.entrada.alarma) Dispositivo::activa_alarma(); // Activamos alarma del controlador
																		// Si se activó la del dispositivo de planta
	
	resultado += Controlador<float>::actualiza(); // Actualizar clase base	
	
	return resultado;
	
}

void Control_carro::rutina() { // Rutina de thread del control del carro
	
	Controlador::estado_ejecucion = std::promise<long unsigned int>(); 
									// Se borra promesa anterior igualando a constructor vacío 
	
	Controlador::futuro_estado_ejecucion = estado_ejecucion.get_future(); // Adquirimos futuro
	
	Controlador::estado_ejecucion.set_value_at_thread_exit(CODIGO_FIN_HILO);
	
	while (Dispositivo::obtiene_ejecucion()) {
		
		Control_carro::actualiza();
		
		// Actualiza E/S del dispositivo
		
	}
	return; 
}

void Control_carro::propiedades() { // propiedades del control del carro	
		
	 // Propiedades. Se puede modificar valor. Se crea un pequeño menú

	unsigned int opcion; 

	do {
		
		std::string parada = Control_carro::parada ? "SI" : "NO";
		std::string movimiento = Control_carro::movimiento ? "IZQUIERDA" : "DERECHA";
		std::cout << "\n\n-------- PROPIEDADES DEL DISPOSITIVO CONTROLADOR CARRO --------------\n\n";
	
		std::cout << "\t\tParado:\t\t" << parada;
		std::cout << "\n\n\t\tSentido:\t" << movimiento;
		std::cout << "\n\t\tConteo carro (si llegó a un detector):\t" << Control_carro::contador;		
		
		if(!Control_carro::parada)
			std::cout << "\n\n\n\t1) Detener el carro";
		else
			std::cout << "\n\n\n\t1) Continuar el movimiento del carro";
		
		std::cout << "\n\n\t0) Salir";
		
		std::cout << "\n\n---------------------------------------------------------------------\n\n\n";
		
		std::cout << "\n\nSeleccione opción -> "; 
		std::cin >> opcion; //espera orden de teclado
		
		if (opcion == 1) {
			
			Control_carro::parada = !Control_carro::parada; // Conmutamos
		}

	} while (opcion != 0);
	
	return;
}
	
