// CONTROLADOR_DEPOSITO.CPP

// El control se efectúa con el objetivo de estabilizar la altura del depósito (aunque puede servir a otros propósitos).

#include <iostream> 
#include "controlador_deposito.h" // Prototipo de la clase "Controlador_deposito"

Control_deposito::Control_deposito(unsigned int t, int id, Watchdog<float> *watchdog, float max_caudal, 
										float p, float i, float d) : Control_PID<float>(t, id, watchdog, p, i, d) {
																	// Control_PID
																// constructor del controlador
	Control_deposito::configura(max_caudal); // Configurador		
	
}

void Control_deposito::configura(float max_caudal) { //configurador	
	
	Control_deposito::max_caudal = max_caudal > 0 ? max_caudal : -max_caudal;
	
	
	// Establece las condiciones iniciales
	
	Control_deposito::es_n1_1.entrada.e1 = 0;  // Condiciones iniciales de entrada, salida, nulas
	Control_deposito::es_n1_1.entrada.e2 = 0; 	
	Control_deposito::es_n1_1.entrada.alarma = false; // Sin alarma
	Control_deposito::es_n1_1.salida.s1 = 0; 	
	
	Control_deposito::es_n2_2.entrada.e1 = 0;
	Control_deposito::es_2_3.salida.s1 = 0;	
	
	// Crea los puertos de comunicaciones. 
	
	//AVISO: Las salidas desde el dispositivo son entradas desde el conector
	//       Una entrada se considera salida y viceversa, al crear las conexiones.
	//		 Esto es aplicable en todas las conexiones y hay que tenerlo en cuenta al crear el dispositivo
	
	Conectable::nuevo_puerto(1,-1,(BYTE *)&Control_deposito::es_n1_1.salida,
							(BYTE *)&Control_deposito::es_n1_1.entrada,
							sizeof(Control_deposito::es_n1_1.salida), sizeof(Control_deposito::es_n1_1.entrada));
							
	Conectable::nuevo_puerto(2,-2,(BYTE *)nullptr,(BYTE *)&Control_deposito::es_n2_2.entrada,
								0, sizeof(Control_deposito::es_n2_2.entrada));
							
	Conectable::nuevo_puerto(3,2,(BYTE *)&Control_deposito::es_2_3.salida,(BYTE *)nullptr,
								sizeof(Control_deposito::es_2_3.salida),0);
						
	return;
	
}

int Control_deposito::actualiza() { // Rutina de actualización por cada iteracción de muestreo
	
	int resultado = 0;
	float caudal;
	
	// Esto es un COMPARADOR (consigna menos señal de la planta)
	Control_PID<float>::x = Control_deposito::es_n2_2.entrada.e1 - Control_deposito::es_n1_1.entrada.e1;
		
	

	caudal = Control_PID<float>::pid(); // Hacemos uso del PID	
	
	// El caudal no puede sobrepasar el valor máximo ni puede ser menor que cero
	if(caudal < 0) caudal = 0; else if(caudal > Control_deposito::max_caudal) caudal = Control_deposito::max_caudal;
	
	Control_deposito::es_n1_1.salida.s1 = caudal; // A la planta
	
	Control_deposito::es_2_3.salida.s1 = caudal; // Replicamos salida en puerto 3 (uso posible en control en cascada)	
		
	if(Control_deposito::es_n1_1.entrada.alarma) Dispositivo::activa_alarma(); // Activamos alarma del controlador
																		// Si se activó la del dispositivo de planta
	
	resultado += Control_PID<float>::actualiza(); // Actualizar clase base	
	
	return resultado;
	
}

void Control_deposito::rutina() { // Rutina de thread del control amortiguador 
	
	Controlador::estado_ejecucion = std::promise<long unsigned int>(); 
									// Se borra promesa anterior igualando a constructor vacío 
	
	Controlador::futuro_estado_ejecucion = estado_ejecucion.get_future(); // Adquirimos futuro
	
	Controlador::estado_ejecucion.set_value_at_thread_exit(CODIGO_FIN_HILO);
	
	while (Dispositivo::obtiene_ejecucion()) {
		
		Control_deposito::actualiza();
		
		// Actualiza E/S del dispositivo
		
	}
	return; 
}

/*void Control_deposito::propiedades() { // propiedades del control del motorCC		
		
	
	return; 
}*/