// CONTROLADOR_DEPOSITO.H
//
// Controlador PID del Depósito.

// Existen dos circunstancias a tener en cuenta: 

// 1) El depósito es un dispositivo no lineal. Por tanto, polos y ceros cambian según la altura del nivel del líquido
// 2) El depósito sólo admite caudal positivo de entrada (no se puede hacer físicamente de otra forma),
//    así que el controlador no puede ofrecer caudal negativo. El control, por ello, se hace más difícil


#include "../base.h" // Base de dispositivos
#include "estructuras_datos.h"
#include <iostream>


class Control_deposito : public Control_PID<float> { // Utilizaremos aritmética con números 'float'

	float max_caudal; // Maximo caudal admisible por la planta (siempre hay límites físicos en los sistemas reales)

protected: 
	void configura(float max_caudal); //configuracion de controlador 

	void rutina(); // Rutina del dispositivo
	int actualiza(); // Actualización
	
	
	// ////////// Entrada-salida del dispositivo del controlador del motor
	
		
	struct ES_N3 es_n1_1; // Estructura de datos esid=-1 al puerto 1 (entrada/salida del controlador al deposito)
	
	struct ES_N2 es_n2_2; // Estructura de datos esid=-2 al puerto 2 (entrada de consigna: altura de agua) 
	
	struct ES_2 es_2_3; // Estructura de datos esid=2 al puerto 3 (salida a otro controlador/cascada)
	
public:
	Control_deposito(unsigned int t, int id, Watchdog<float> *watchdog, float max_caudal, float p, float i, float d); 
																// constructor del controlador
//	void propiedades();
	
};