// CONTROL_MOTOR.CPP

// El control se efectúa con el objetivo de estabilizar la velocidad angular de giro.



#include <iostream> 
#include "controlador_motorcc.h" // Prototipo de la clase "Controlador_motorCC"

Control_motorCC::Control_motorCC(unsigned int t, int id, Watchdog<float> *watchdog, 
				float p, float i, float d, bool apag) : Control_PID<float>(t, id, watchdog, p, i, d) {
																	// Control_PID
																// constructor del controlador
	Control_motorCC::apag = apag; // Bit de apagado en caso de alarma
	Control_motorCC::configura(); // Configurador		
	
	
}

void Control_motorCC::configura() { //configurador	
	
	
	// Establece las condiciones iniciales
	
	Control_motorCC::es_n3_1.entrada.e1 = 0;  // Condiciones iniciales de entrada, salida, nulas
	Control_motorCC::es_n3_1.entrada.e2 = 0; 
	Control_motorCC::es_n3_1.entrada.e3 = 0; 
	Control_motorCC::es_n3_1.entrada.alarma = false; // Sin alarma
	Control_motorCC::es_n3_1.salida.s1 = 0; 
	
	
	Control_motorCC::es_n2_2.entrada.e1 = 0;
	Control_motorCC::es_2_3.salida.s1 = 0;	
	
	// Crea los puertos de comunicaciones. 
	
	//AVISO: Las salidas desde el dispositivo son entradas desde el conector
	//       Una entrada se considera salida y viceversa, al crear las conexiones.
	//		 Esto es aplicable en todas las conexiones y hay que tenerlo en cuenta al crear el dispositivo
	
	Conectable::nuevo_puerto(1,-3,(BYTE *)&Control_motorCC::es_n3_1.salida,
							(BYTE *)&Control_motorCC::es_n3_1.entrada,
							sizeof(Control_motorCC::es_n3_1.salida), sizeof(Control_motorCC::es_n3_1.entrada));
							
	Conectable::nuevo_puerto(2,-2,(BYTE *)nullptr,(BYTE *)&Control_motorCC::es_n2_2.entrada,
								0, sizeof(Control_motorCC::es_n2_2.entrada));
							
	Conectable::nuevo_puerto(3,2,(BYTE *)&Control_motorCC::es_2_3.salida,(BYTE *)nullptr,
								sizeof(Control_motorCC::es_2_3.salida),0);
						
	return;
	
}

int Control_motorCC::actualiza() { // Rutina de actualización por cada iteracción de muestreo
	
	int resultado = 0;
	
	// Esto es un COMPARADOR (consigna menos señal de la planta)
	Control_PID<float>::x = Control_motorCC::es_n2_2.entrada.e1 - Control_motorCC::es_n3_1.entrada.e1;
	
	if(Control_motorCC::es_n3_1.entrada.alarma && !Dispositivo::obtiene_alarma())  //¿Hay alarma desde la planta?
		Dispositivo::activa_alarma(); // Activamos alarma del controlador
									  
	if(Dispositivo::obtiene_alarma() && Control_motorCC::apag) { // Si hay alarma y se manda apagar en caso de alarma
		Control_motorCC::es_n3_1.salida.s1 = 0.; // Salida nula	
		Control_motorCC::es_2_3.salida.s1 = 0.; // Replicamos salida en puerto 3
	} else {	// Todo va normal
			
		Control_motorCC::es_n3_1.salida.s1 = Control_PID<float>::pid(); // Hacemos uso del PID	
		Control_motorCC::es_2_3.salida.s1 = Control_motorCC::es_n3_1.salida.s1; // Replicamos salida en puerto 3	
			
	}
	
	
	resultado += Control_PID<float>::actualiza(); // Actualizar clase base	
	
	return resultado;
	
}

void Control_motorCC::rutina() { // Rutina de thread del control amortiguador 
	
	Controlador::estado_ejecucion = std::promise<long unsigned int>(); 
									// Se borra promesa anterior igualando a constructor vacío 
	
	Controlador::futuro_estado_ejecucion = estado_ejecucion.get_future(); // Adquirimos futuro
	
	Controlador::estado_ejecucion.set_value_at_thread_exit(CODIGO_FIN_HILO);
	
	while (Dispositivo::obtiene_ejecucion()) {
		
		Control_motorCC::actualiza();
		
		// Actualiza E/S del dispositivo
		
	}
	return; 
}

/*void Control_motorCC::propiedades() { // propiedades del control del motorCC		
		
	
	return; 
}*/