// DEPOSITO.H

// implementación del deposito

#include <iostream> 
#include <cmath> // Para hacer uso de sqrt (raices cuadradas)
#include "deposito.h" // Prototipo de la clase Deposito, derivado de continuo

Deposito::Deposito(unsigned int t, int id, float as, float h_max, float h_init, 
							float qe_init, float qs_init, float ks_init) : Continuo<float>(t,id) { // constructor			
						
	Deposito::configura(as, h_max, h_init, qe_init, qs_init, ks_init); // se configura el depósito	
}

void Deposito::configura(float as, float h_max, float h_init, float qe_init, float qs_init, float ks_init) { 
							//configurador
	
	
	std::vector< std::vector<float> > matriz; // Matriz asociada que construiremos y pasaremos a Continuo
	
	std::vector<float> ind; // Terminos independientes	
						
						// Ambos objetos serán traspasados al configurador de la clase padre Continuo. 
						// antes, hay que darles los valores del sistema a simular (matriz asociada, término ind.)	
	
	Deposito::h_max = h_max > 0 ? h_max : -h_max;
	Deposito::h_init = h_init > 0 ? h_init : -h_init;
	Deposito::as = as > 0 ? as : -as;	
	Deposito::ks_init = ks_init > 0 ? ks_init : -ks_init; 	
			// Guardamos estas variables en sus valores ABSOLUTOS (dominio solo positivos)
			
	Deposito::c1 = Deposito::as * sqrt(2 * GRAV); 
				// Coeficiente c1 
	
	float c = Deposito::ks_init	* c1;
				// Coeficiente c (ver memoria del trabajo, modelo del depósito) aquí es c = ks * c1
	
	
	matriz.resize(1); matriz[0].resize(1); // Dimensionamos la matriz: 1x1 (puede parecer recargado pero así utilizamos
										   // la clase continuo, mediante RK-4. 
	ind.resize(1); // Dimensionamos los términos independientes
	
	// Rellenamos valores en matriz y términos independientes. Un solo valor (escalar).
	// Pero ese valor lo hemos de ir modificando en función de la variable de estado, ya que es un sistema no lineal. 
	
	float d = (1 + sqrt(h_init)) * (1 + sqrt(h_init)); // auxiliar (1+sqrt(h))^2 
	
	matriz[0][0]= -c / (sqrt(h_init)*d);
	
	ind[0]= qe_init / d; 							 
	
	
	Continuo<float>::configura(matriz, ind); // Llamada al configurador base para inicializar el dispositivo
	
	// OJO. IMPORTANTE: PARA EVITAR EXCEPCIONES COMPROBAR DIMENSIONES DE MATRIZ Y TERMINO INDEPENDIENTE
	// HAN DE COINCIDIR.
	
	// En caso de que no coincidieran, no se crearán las memorias para los estados y a partir de aquí es probable
	// que se escriba en zonas no autorizadas, causando la excepción segmentation fault. 
	
	
	// Establece las condiciones iniciales en la variable de estado (altura h)
	
	Dinamico<float>::estado[0][0] = Deposito::h_init;
	
	
	Deposito::es_1_1.salida.s1 = Deposito::h_init; // Los valores iniciales, también en los puertos
	Deposito::es_1_1.salida.s2 = qs_init > 0 ? qs_init : -qs_init;
	
	
	Deposito::es_1_1.salida.alarma = false; // Sin alarma 	
	
	Deposito::es_n2_2.entrada.e1 = Deposito::ks_init; 
											// El coeficiente de descarga variable, especificado en el constructor
											// (positivo)
	
	Deposito::es_1_1.entrada.e1 = qe_init > 0 ? qe_init : -qe_init; // Caudal de entrada inicial (positivo, siempre)

	// Crea los puertos de comunicaciones. 
	
	//AVISO: Las salidas desde el dispositivo son entradas desde el conector
	//       Una entrada se considera salida y viceversa, al crear las conexiones.
	//		 Esto es aplicable en todas las conexiones y hay que tenerlo en cuenta al crear el dispositivo
	
	Conectable::nuevo_puerto(1,1,(BYTE *)&Deposito::es_1_1.salida,(BYTE *)&Deposito::es_1_1.entrada,
							sizeof(Deposito::es_1_1.salida), sizeof(Deposito::es_1_1.entrada)); // Puerto 1 al controlador 
							
	Conectable::nuevo_puerto(2,-2,(BYTE *)nullptr,(BYTE *)&Deposito::es_n2_2.entrada, // Puerto 2 de coef. descarga
								0,sizeof(Deposito::es_n2_2.entrada));
	
	Conectable::nuevo_puerto(3,-2,(BYTE *)nullptr,(BYTE *)&Deposito::es_1_1.entrada, // Puerto 3 de entrada de caudal
								0, sizeof(Deposito::es_1_1.entrada));	
	
	
	return;	
	
}

int Deposito::actualiza() { // Rutina de actualización por cada iteracción de muestreo
	
	int resultado = 0;	
	
	// Método numérico de integración Runke-Kutta de orden 4
	Continuo<float>::rk4(); // Aplicamos RK4 en el sistema del motorCC para calcular siguiente estado
							// Nota: para la primera iteración tenemos que haber configurado bien la matriz
							// del sistema, el vector de términos independientes, las condiciones iniciales	

	// Actualizamos los términos independientes que tengan variables de contorno, según lo que nos llegó por E/S
	
	resultado += Continuo<float>::actualiza(); // Actualizar clase base, incluye E/S por conectable	
   
	// Recuperamos variables que utilizaremos
	float h = Deposito::es_1_1.salida.s1; 
	float qe = Deposito::es_1_1.entrada.e1 > 0 ? Deposito::es_1_1.entrada.e1 : -Deposito::es_1_1.entrada.e1;
																	// Nota: El controlador no puede proporcionar
																	//	caudal negativo
																	
	float ks = Deposito::es_n2_2.entrada.e1 > 0 ? Deposito::es_n2_2.entrada.e1 : -Deposito::es_n2_2.entrada.e1;  
	
	float d = (1 + sqrt(h)) * (1 + sqrt(h)); // auxiliar (1+sqrt(h))^2 
	
	Continuo<float>::indep[0]=qe/d; // Nuevo término independiente

	// Al ser método no lineal, también modificamos coeficiente en la matriz	 
	
	Continuo<float>::asoc[0][0]= -(Deposito::c1 * ks)/(d*sqrt(h)); // Nuevo coeficiente	
							
		// Finalmente, comprobamos tolerancia y actualizamos puertos según los estados que nos proporciona Continuo		
						
	h = Dinamico<float>::estado[0][0]; // Altura instantánea
	
	// Aquí definimos si sobrepasamos alguna tolerancia, algún requisito, y en caso de ser así, se activa alarma.
	// Esa alarma se transmite al controlador correspondiente
		
	if (!Dispositivo::obtiene_alarma()) {
		if (h > h_max) { // ¿Se sobrepasa la altura máxima del depósito
			Dispositivo::activa_alarma(); // Alarma activada
			Deposito::es_1_1.salida.alarma = true;  // Queda permanente en el puerto			
		}	
	}	
	
	if (h > h_max) h = h_max; // Físicamente, se desborda el líquido. Imposible subir más de la altura de desbordamiento 
	 
	Deposito::es_1_1.salida.s1 = h; 
	Deposito::es_1_1.salida.s2 = Deposito::c1 * ks * sqrt(h);	
	
	/*Deposito::es_1_1.salida.s1 = h; 
	Deposito::es_1_1.salida.s2 = Deposito::c1 * ks * sqrt(h);*/	
	
	return resultado;	
	
	// Nota: Hay que hacer muchos cálculos con raíces y el cómputo es mayor que otros dispositivos de planta
	// Hay que tenerlo en cuenta para las simulaciones con mucha frecuencia de muestreo
	
}

void Deposito::rutina() { // Rutina de thread 
	
	while (Dispositivo::obtiene_ejecucion()) {
		
		Deposito::actualiza();
		
		// Actualiza E/S del dispositivo siempre que esté en estado de ejecución
		
	}
	return; 
}