// ESTRUCTURAS_DATOS.H
//
// Estructuras de datos que manejan los dispositivos. Se pueden añadir más estructuras, a conveniencia.

// Nomenclatura: ES_x El número x es el esid. Se antepone N si es negativo o complementario, es decir -x

// esid = 1 y esid = -1 se usan para comunicar amortiguador con su controlador; o el motorCC con su controlador
 

#ifndef ESTRUCTURAS_DATOS_H // Evitamos redefiniciones que podrían provocar fallos de compilación
#define ESTRUCTURAS_DATOS_H
 
	struct ES_1 { // Estructura para esid=1. 
	
		struct ENTRADA_ES_1 { // El dispositivo ha de especificar esid=1
			float e1; 	// Entrada float	
		} entrada;
	
		struct SALIDA_ES_1 {	
			float s1; // Salidas float
			float s2;	
			bool alarma; // Notificación de alarma
		} salida;
	
	};	
	
	struct ES_N1 { // Estructura para esid=-1. (Complementario al anterior; entradas y salidas intercambiadas)
			
		struct ENTRADA_ES_N1 {	
			float e1; 
			float e2;	
			bool alarma; // Notificación de alarma
		} entrada;
		
		struct SALIDA_ES_N1 { // El dispositivo ha de especificar esid=1
			float s1;			
		} salida;
	
	};
	
	struct ES_2 { // Estructura para esid=2. 
		struct SALIDA_ES_2 {
			float s1; // Salida float	
		} salida;	// No hay entrada. Para dispositivos de solo salida 
	};
	
	struct ES_N2 { // Estructura de esid=-2 (complementario al anterior). 
		struct ENTRADA_ES_N2 {
			float e1; // entrada float		
		} entrada;	
	};	
	
	
	struct ES_3 { // Estructura para esid=3. 
	
		struct ENTRADA_ES_3 { 
			float e1; 	// Entrada float	
		} entrada;
	
		struct SALIDA_ES_3 {	
			float s1; // Salidas float
			float s2;	
			float s3;
			bool alarma; // Notificación de alarma
		} salida;
	
	};	
	
	struct ES_N3 { // Estructura para esid=-3. (Complementario al anterior; entradas y salidas intercambiadas)
			
		struct ENTRADA_ES_N3 {	
			float e1; 
			float e2;	
			float e3;
			bool alarma; // Notificación de alarma
		} entrada;
		
		struct SALIDA_ES_N3 { // El dispositivo ha de especificar esid=1
			float s1;			
		} salida;
	
	};
	
	struct ENTRADA_ES_3_HW { // Estructura esid = 3 para los discretizadores hardware HW1 y HW2. Igual que ES_3
			float e1;
	};
	
	struct SALIDA_ES_3_HW {	
			float s1; // Salidas float
			float s2;	
			float s3;
			bool alarma; // Notificación de alarma
	};
	

	struct ES_4 { // Estructura para esid=4. 

		struct ENTRADA_ES_4 { 
			float e1; 	// Entrada float	
		} entrada;

		struct SALIDA_ES_4 {	
			float s1; // Salidas float
			float s2;	
			float s3;
			bool s4; // Salidas bool
			bool s5;
			bool alarma; // Notificación de alarma
		} salida;
	
	};	
	
	struct ES_N4 { // Estructura para esid=-4. (Complementario al anterior; entradas y salidas intercambiadas)
			
		struct ENTRADA_ES_N4 {	
			float e1; // Entradas float
			float e2;	
			float e3;
			bool e4; // Entradas bool
			bool e5;
			bool alarma; // Notificación de alarma
		} entrada;
		
		struct SALIDA_ES_N4 { // El dispositivo ha de especificar esid=1
			float s1;			
		} salida;
	
	};
	
	
	
#endif // ESTRUCTURAS_DATOS_H