// funciones_periodicas.cpp
//
// Implementación del dispositivo de funciones periódicas

#include "funciones_periodicas.h"

#include <iostream> 

Periodica::Periodica(unsigned int t, int id, tipo_funcion modo, unsigned int cuenta_asc, unsigned int cuenta_desc,
						float superior, float inferior) : Conectable(t,id) { // Constructor : Conectable(t,id) 
												//constructor
												
	Periodica::configura(modo, cuenta_asc, cuenta_desc, superior, inferior); // llamada al configurador	
												
}

void Periodica::configura(tipo_funcion modo, unsigned int cuenta_asc, unsigned int cuenta_desc,
						float superior, float inferior) { //configurador
												
	if(cuenta_asc == 0 || cuenta_desc == 0 || superior == inferior) return; // Valores no permitidos. Salir. 

	Periodica::superior = superior > inferior ? superior : inferior; // Si inferior > superior, invertir orden
	Periodica::inferior = superior > inferior ? inferior : superior;
	
	Periodica::cuenta_ascendente = cuenta_asc; // establecemos los límites de conteo
	Periodica::cuenta_descendente = cuenta_desc;
	
	Periodica::paso_ascendente = (Periodica::superior - Periodica::inferior) / (float)cuenta_asc; // Determina pasos
	Periodica::paso_descendente = (Periodica::superior - Periodica::inferior) / (float)cuenta_desc;
	
	Periodica::superior_m = Periodica::superior; // Copiamos a modificables
	Periodica::inferior_m = Periodica::inferior;
	Periodica::cuenta_ascendente_m = Periodica::cuenta_ascendente;
	Periodica::cuenta_descendente_m = Periodica::cuenta_descendente;
	
	
	
	Periodica::modo = modo; // Modo de operación. En caso de ser modo no permitido, se toma triangular por defecto
	Periodica::modo_m = Periodica::modo; // Réplica en modificable
	
	Periodica::es_2_1.salida.s1 = Periodica::inferior; // Condiciones iniciales en el puerto de salida
	
	
	// Habilitar el puerto para ser conectado
	
	//AVISO: Las salidas desde el dispositivo son entradas desde el conector
	//       Una entrada se considera salida y viceversa, al crear las conexiones.
	//		 Esto es aplicable en todas las conexiones y hay que tenerlo en cuenta al crear el dispositivo
	
		Conectable::nuevo_puerto(1,2,(BYTE *)&Periodica::es_2_1.salida,(BYTE *)nullptr,
							sizeof(Periodica::es_2_1.salida), 0);

												
}

int Periodica::actualiza() { // Rutina de actualización por cada iteracción de muestreo
	
	int resultado = 0;
	
	if(Periodica::modificado) { // ¿Hubo modificación en propiedades()? 
		Periodica::superior = Periodica::superior_m; // Modificamos los límites
		Periodica::inferior = Periodica::inferior_m;
													 // Respetando la fase ascendente o descendente del conteo
													 // modificamos el conteo en sí, a partir de los nuevos valores
		
		Periodica::paso_ascendente = (Periodica::superior_m - Periodica::inferior_m) / // Determinamos nuevos pasos
							(float)Periodica::cuenta_ascendente_m; // Determina pasos
		Periodica::paso_descendente = (Periodica::superior_m - Periodica::inferior_m) / 
							(float)Periodica::cuenta_descendente_m;
		
		if(Periodica::contador_descendente == 0) {// Se intenta que la transición en el cambio sea "suave" 
												 // si se modificaron las cuentas ascendentes/descendentes
			float rel_asc = (float)(Periodica::contador_ascendente / Periodica::cuenta_ascendente);
			Periodica::contador_ascendente = (unsigned int)((float)(Periodica::cuenta_ascendente_m) * rel_asc); 
		}
		
		if(Periodica::contador_ascendente == 0) { 
												 
			float rel_des = (float)(Periodica::contador_descendente / Periodica::cuenta_descendente);
			Periodica::contador_descendente = (unsigned int)((float)(Periodica::cuenta_descendente_m) * rel_des);
		
		}
		
		Periodica::cuenta_ascendente = Periodica::cuenta_ascendente_m; // Modificamos cuentas asc/desc
		Periodica::cuenta_descendente = Periodica::cuenta_descendente_m;
		
		Periodica::modo = Periodica::modo_m; // Modificamos tipo de funcion
		
		Periodica::modificado = false; // Ya se modificó
	}
		
	if(Periodica::tipo_cuenta) { // ¿Es tendencia ascendente?	
		switch(Periodica::modo) { // Diferentes modos de operación (triangular, cuadrada y cuadrada inversa) 
			case F_CUADRADA:
				Periodica::es_2_1.salida.s1 = inferior; 
				break;
			case F_CUADRADA_INVERTIDA:
				Periodica::es_2_1.salida.s1 = superior; 
				break;
			case F_TRIANGULAR:
			default:	
				Periodica::es_2_1.salida.s1 = inferior + (float)(Periodica::contador_ascendente) 
												* Periodica::paso_ascendente;
				
		
		}
		Periodica::contador_ascendente++;
		
		if (Periodica::contador_ascendente >= Periodica::cuenta_ascendente) { // ¿llegó al límite ascendente?
			Periodica::tipo_cuenta = false;		// En la siguiente iteración será descendente
			Periodica::contador_ascendente = 0; // Reseteo del contador ascendente
		}
		
	} else { // En caso de tendencia descendente
		switch(Periodica::modo) { // Diferentes modos de operación (triangular, cuadrada y cuadrada inversa) 
			case F_CUADRADA:
				Periodica::es_2_1.salida.s1 = superior; 
				break;
			case F_CUADRADA_INVERTIDA:
				Periodica::es_2_1.salida.s1 = inferior; 
				break;
			case F_TRIANGULAR:
			default:	
				Periodica::es_2_1.salida.s1 = superior - (float)(Periodica::contador_descendente) 
												* Periodica::paso_descendente; 
				
		}
		Periodica::contador_descendente++;
		
		if (Periodica::contador_descendente >= Periodica::cuenta_descendente) { // ¿llegó el límite ascendente?
			Periodica::tipo_cuenta = true;		// En la siguiente iteración será ascendente
			Periodica::contador_descendente = 0; // Reseteo del contador descendente. Repite ciclo inicial.
		}
	
	}
	
//	std::cout << "periodica: " << Periodica::es_2_1.salida.periodico << "\n"; /////////// DEPURACION /////////////
			
	resultado += Conectable::actualiza(); // Actualizar clase base
													   
	return resultado;
	
}

void Periodica::rutina() { // Rutina de thread 
	
	while (Dispositivo::obtiene_ejecucion()) {
		
		Periodica::actualiza();
	}
	
}

void Periodica::propiedades() { //Propiedades en pantalla
	
	unsigned int opcion; 	
	unsigned int cuenta_ascendente, cuenta_descendente; // valores de entrada de teclado
	float superior, inferior;
	tipo_funcion modo = F_TRIANGULAR;
	int modo_int = 0;
	bool editable = false; // ¿Se puede editar?

	do {
		std::cout << "\n\n-------- PROPIEDADES DEL GENERADOR DE FUNCIONES ---------------------\n\n";
		std::cout << "\t\tTipo de función: ";
		switch(Periodica::modo_m) {
			case F_TRIANGULAR:
				std::cout << "Triangular";
				break;
			case F_CUADRADA:
				std::cout << "Cuadrada";
				break;
			case F_CUADRADA_INVERTIDA:
				std::cout << "Cuadrada inversa";
				break;
			default:
			break;
		};
		std::cout << "\n\t\tCuenta ascendente:\t" << Periodica::cuenta_ascendente_m;
		std::cout << "\n\t\tCuenta descendente:\t" << Periodica::cuenta_descendente_m;
		std::cout << "\n\t\tValor superior:\t" << Periodica::superior_m;
		std::cout << "\n\t\tValor inferior:\t" << Periodica::inferior_m;
		
		if(!Periodica::modificado && Dispositivo::obtiene_ejecucion()) { // Mostrar opciones si son modificables 
			std::cout << "\n\n\t1) Modificar forma de onda";
			std::cout << "\n\t2) Modificar cuenta ascendente";
			std::cout << "\n\t3) Modificar cuenta descendente";
			std::cout << "\n\t4) Modificar valor superior";
			std::cout << "\n\t5) Modificar valor inferior";
			editable = true;
		} else {
			std::cout << "\n\nLos valores no son modificables en este momento, hasta que los procese el dispositivo."
					  << "\nSi el dispositivo está parado/pausado hay que iniciarlo para hacer efectivos los cambios" 
					  << "\nPulse cualquier valor que no sea 0 para actualizar esta pantalla o 0 para salir.";
			editable = false;
		}

		std::cout << "\n\n\t0) Salir";
		
		std::cout << "\n\n---------------------------------------------------------------------\n";
		
		std::cout << "\nSeleccione opción -> "; 
		std::cin >> opcion; //espera orden de teclado			 
		
		if(editable) { // ¿Se puede modificar? 
			switch(opcion) {
				case 1:
					std::cout << "\n\nIntroduzca nuevo tipo de funcion (" << F_TRIANGULAR << "=Triangular, " <<
					F_CUADRADA << "=Cuadrada, " << F_CUADRADA_INVERTIDA << "=Cuadrada invertida) -> ";				
					std::cin >> modo_int; //espera orden de teclado
					modo = (tipo_funcion)modo_int;
					switch(modo) {
						case F_TRIANGULAR:
						case F_CUADRADA:
						case F_CUADRADA_INVERTIDA:
							Periodica::modo_m = modo;
							Periodica::modificado = true; // ¿Se modificaron valores? si
							break;
						default:
						std::cout << "\nTipo de funcion no permitido\n";
						break;
					};
					break;
				case 2:
					std::cout << "\n\nIntroduzca nuevo conteo ascendente -> ";
					std::cin >> cuenta_ascendente; //espera orden de teclado
					if (cuenta_ascendente >= 0) {
						Periodica::cuenta_ascendente_m = cuenta_ascendente;
						Periodica::modificado = true; // ¿Se modificaron valores? si
					} else std::cout << "\nValor de conteo no permitido\n";
					break;
				case 3:
					std::cout << "\n\nIntroduzca nuevo conteo descendente -> ";
					std::cin >> cuenta_descendente; //espera orden de teclado
					if (cuenta_descendente >= 0) {
						Periodica::cuenta_descendente_m = cuenta_descendente;
						Periodica::modificado = true; // ¿Se modificaron valores? si
					} else std::cout << "\nValor de conteo no permitido\n";				
					break;
				case 4:
					std::cout << "\n\nIntroduzca nuevo valor superior -> ";
					std::cin >> superior; //espera orden de teclado
					if (superior >= Periodica::inferior) {
						Periodica::superior_m = superior;
						Periodica::modificado = true; // ¿Se modificaron valores? si
					} else std::cout << "\nEl valor superior no puede ser menor al valor inferior\n";				
					break;
				case 5:
					std::cout << "\n\nIntroduzca nuevo valor inferior -> ";
					std::cin >> inferior; //espera orden de teclado
					if (inferior <= Periodica::superior) {
						Periodica::inferior_m = inferior;
						Periodica::modificado = true; // ¿Se modificaron valores? si
					} else std::cout << "\nEl valor inferior no puede ser mayor al valor superior\n";				
					break;
				default:
					break;		
			};	
		
		} 

	} while (opcion != 0);	
		
	return;
	
}

void Periodica::propiedades_base() { //Propiedades de clase base y superclases
	
	Conectable::propiedades_base();
	
	return;
}
											
