// funciones_periodicas.h
//
// Dispositivo no dinámico generador de funciones cíclicas o periódicas
//
// Estas funciones se generan fácilmente mediante conteo aunque también es posible recurrir a librerías 
// matemáticas para implementar funciones trigonométricas (movimientos armónicos simples) o de otra índole
// 
// En este caso generaremos funciones cuadradas y triangulares, simétricas y asimétricas, según los parámetros
// proporcionados

#include "../base.h" // Base de dispositivos
#include "estructuras_datos.h"

enum tipo_funcion {F_TRIANGULAR, F_CUADRADA, F_CUADRADA_INVERTIDA}; // tipo de funciones generadas


class Periodica : public Conectable { // La clase base es Conectable. No es necesario conocer estados anteriores
										// y por tanto no lo consideraremos como dispositivo dinámico
											     

	
// Entrada-salida del dispositivo ciclico
	
	struct ES_2 es_2_1; // Salida esid=2 por puerto 1

	unsigned int cuenta_ascendente; // Cuenta ascendente de la función periódica
	unsigned int cuenta_descendente; // Cuenta descendente de la función periódica
	float superior; // Valor más elevado que alcanzará la función
	float inferior; // Valor inferior que alcanzará la función
	
	std::atomic<bool> modificado {false}; // Si se modifica desde propiedades, se actualizan variables
	
	unsigned int cuenta_ascendente_m; // Variables modificadas en propiedades
	unsigned int cuenta_descendente_m; 
	float superior_m; 
	float inferior_m; 
	
	float paso_ascendente, paso_descendente; // Pasos de la función (ascendente/descendente)
	
	unsigned int contador_ascendente = 0; // Contador ascendente (varía y alcanza el límite de cuenta ascendente)
	unsigned int contador_descendente = 0; // Contador descendente (varía y alcanza el límite de cuenta descendente)
	bool tipo_cuenta = true; // true = ascendente ; false = descendente
	
	tipo_funcion modo; // Modo de operación : Señal triangular o cuadrada
	tipo_funcion modo_m; // Tipo de funcion modificado
	

protected: 
	void configura(tipo_funcion modo, unsigned int cuenta_asc, unsigned int cuenta_desc,
						float superior, float inferior); // Configurador
	void rutina();
	int actualiza(); // Actualización
	
public:
	Periodica(unsigned int t, int id, tipo_funcion modo, unsigned int cuenta_asc, unsigned int cuenta_desc,
						float superior, float inferior); // Constructor
						
	void propiedades(); // Propiedades de Funciones periodicas
	void propiedades_base(); // Propiedades base 
	
};

