// Motorcc.cpp

// implementación del motor en corriente continua

#include <iostream> 
#include "motorcc.h" // Prototipo de la clase motorcc, derivado de continuo

MotorCC::MotorCC(unsigned int t, int id, float rr, float lr, float ke, float ks, 
					float il, float b, float v_init, float i_init, float w_init, 
						float o_init, float ts_init, float i_max) : Continuo<float>(t,id) { // constructor
			
						
	MotorCC::configura(rr,lr,ke,ks,il,b, v_init,i_init,w_init,o_init, ts_init, i_max); // se configura el motorCC
	
}

void MotorCC::configura(float rr, float lr, float ke, float ks, float il, 
					float b, float v_init, float i_init, float w_init, float o_init, float ts_init, float i_max) { 
							//configurador
	
	int a = 0., b1 = 0., c = 0., d = 0., e = 0., f = 0.;
	
	std::vector< std::vector<float> > matriz; // Matriz asociada que construiremos y pasaremos a Continuo
	
	std::vector<float> ind; // Terminos independientes
						
						// Ambos objetos serán traspasados al configurador de la clase padre Continuo. 
						// antes, hay que darles los valores del sistema a simular (matriz asociada, término ind.)
	
	if (lr != 0. ) { // Evitemos divisiones por cero
	a = -rr/lr;
	b1 = -ke/lr;	
	c = 1/lr; // Nota: Luego se multiplica por la variable de contorno
	}
	
	if (il != 0. ){
	d = ks/il;
	e = -b/il;
	f = -1/il; // Nota: Luego se multiplica por la variable de contorno
	}
	
	MotorCC::lr = lr;
	MotorCC::il = il;
	MotorCC::i_max = i_max;
	
	MotorCC::ks = ks; // Guardamos estas variables en protected (para clases derivadas), ver Carro.cpp.
	MotorCC::b = b;
	
	matriz.resize(3); matriz[0].resize(3); matriz[1].resize(3); matriz[2].resize(3); // Dimensionamos la matriz
	ind.resize(3); // Dimensionamos los términos independientes
	
	// Rellenamos valores en matriz y términos independientes
	
	matriz[0][0]=a ; matriz[0][1]=b1   ; matriz[0][2]=0.; 
	matriz[1][0]=d ; matriz[1][1]=e   ; matriz[1][2]=0.;
	matriz[2][0]=0.; matriz[2][1]=1   ; matriz[2][2]=0.;	
	
	ind[0]=c * v_init ; ind[1]=f * ts_init; ind[2]=0.;  
	
	
									// Los términos independientes [0] y [1] se han de ir actualizando porque dependen
									// de las variables de contorno.
														 
	
	
	Continuo<float>::configura(matriz, ind); // Llamada al configurador base para inicializar el dispositivo
	
	// OJO. IMPORTANTE: PARA EVITAR EXCEPCIONES COMPROBAR DIMENSIONES DE MATRIZ Y TERMINO INDEPENDIENTE
	// HAN DE COINCIDIR.
	
	// En caso de que no coincidieran, no se crearán las memorias para los estados y a partir de aquí es probable
	// que se escriba en zonas no autorizadas, causando la excepción segmentation fault. 
	
	
	// Establece las condiciones iniciales
	
	Dinamico<float>::estado[0][0] = i_init;
	Dinamico<float>::estado[1][0] = w_init;
	Dinamico<float>::estado[2][0] = o_init;
	
	MotorCC::es_3_1.salida.s1 = w_init; // Los valores iniciales, también en los puertos
	MotorCC::es_3_1.salida.s2 = o_init;
	MotorCC::es_3_1.salida.s3 = i_init;
	
	MotorCC::es_3_1.salida.alarma = false; // Sin alarma 	
	
	MotorCC::es_n2_2.entrada.e1 = ts_init; // El par inicial indicado en el argumento del constructor
	
	MotorCC::es_3_1.entrada.e1 = v_init; // iniciamos voltaje de entrada

	// Crea los puertos de comunicaciones. 
	
	//AVISO: Las salidas desde el dispositivo son entradas desde el conector
	//       Una entrada se considera salida y viceversa, al crear las conexiones.
	//		 Esto es aplicable en todas las conexiones y hay que tenerlo en cuenta al crear el dispositivo
	
	Conectable::nuevo_puerto(1,3,(BYTE *)&MotorCC::es_3_1.salida,(BYTE *)&MotorCC::es_3_1.entrada,
							sizeof(MotorCC::es_3_1.salida), sizeof(MotorCC::es_3_1.entrada)); // Puerto 1 al controlador 
							
	Conectable::nuevo_puerto(2,-2,(BYTE *)nullptr,(BYTE *)&MotorCC::es_n2_2.entrada, // Puerto 2 de entrada de par
								0,sizeof(MotorCC::es_n2_2.entrada));
	
	Conectable::nuevo_puerto(3,-2,(BYTE *)nullptr,(BYTE *)&MotorCC::es_3_1.entrada, // Puerto 3 de entrada de voltaje
								0, sizeof(MotorCC::es_3_1.entrada));
								
	// Nota: El puerto 3 enmascara la entrada del puerto 1, correspondiente con el voltaje de entrada. Si queremos
	// hacer funcionar el motor sin controlador, usaremos el puerto 3, pues prevalece sobre el voltaje del puerto 1 
	// cuando ambos están conectados a la vez. Sin embargo, a veces los puertos no reciben porque el emisor
	// no quiso o tiene una frecuencia de muestreo baja, por tanto, la mezcla de las dos señales puede dar lugar
	// a un comportamiento errático que provocaría una mala función del dispositivo
	
	// En general, si dos puertos escriben la misma variable, prevalece el último inscrito (el 3 aquí). Si quisiéramos
	// usar el motor conectado a un controlador, entonces nos basta con tener el puerto 3 desconectado y éste no podrá
	// enmascarar el puerto 1 que es el principal de conexión. 
	
	// Es una forma de que el dispositivo sea más versátil. Así lo podemos ensayar sin controlador y efectuando la 
	// grabación en el puerto 1.  
	
	
	return;	
	
}

int MotorCC::actualiza() { // Rutina de actualización por cada iteracción de muestreo
	
	int resultado = 0;
	
//	float voltaje; // Voltaje de entrada (desde el puerto 1)
	float lr, il; // Constantes
	float ts; // Par resistente (desde el puerto 2)
	float v; // Voltaje (desde el puerto 1)
		
	float intensidad, i_max; // Corriente y corriente maxima

	resultado += Continuo<float>::actualiza(); // Actualizar clase base, incluye E/S por conectable	

	// Actualizamos los términos independientes que tengan variables de contorno según lo que nos llegó por E/S
	
	// primero las variables locales de este método (para ver más claras las operaciones en el término independiente)	
	lr = MotorCC::lr; il = MotorCC::il;
	
	ts = MotorCC::es_n2_2.entrada.e1; // recuperamos par resistente 
	v = MotorCC::es_3_1.entrada.e1; // recuperamos voltaje 	

//	std::cout << v << " "; ////////////////// Depuracion
	
	Continuo<float>::indep[0]= v/lr;  	// Actualizamos términos independientes	
	Continuo<float>::indep[1]= -ts/il; 
									
								
	// Método numérico de integración Runke-Kutta de orden 4
	
	Continuo<float>::rk4(); // Aplicamos RK4 en el sistema del motorCC para calcular siguiente estado
							// Nota: para la primera iteración tenemos que haber configurado bien la matriz
							// del sistema, el vector de términos independientes, las condiciones iniciales	
	
	// Finalmente, actualizamos puertos según los estados que nos proporciona Continuo		
						
	
	MotorCC::es_3_1.salida.s1 = Dinamico<float>::estado[1][0]; // Velocidad angular
	MotorCC::es_3_1.salida.s2 = Dinamico<float>::estado[2][0]; // Posición angular		
	MotorCC::es_3_1.salida.s3 = Dinamico<float>::estado[0][0]; // corriente					
	
	
	// Aquí definimos si sobrepasamos alguna tolerancia, algún requisito, y en caso de ser así, se activa alarma.
	// Esa alarma se transmite al controlador correspondiente
	
	// Hallamos intensidad (VALOR ABSOLUTO) 
	intensidad = Dinamico<float>::estado[0][0] > 0 ? Dinamico<float>::estado[0][0] : - Dinamico<float>::estado[0][0];
	
	i_max = MotorCC::i_max; // obtenemos corriente máxima
		
	if (!Dispositivo::obtiene_alarma()) {
		if (intensidad > i_max) { // ¿Se sobrepasa la intensidad máxima?
			Dispositivo::activa_alarma(); // Alarma activada
			MotorCC::es_3_1.salida.alarma = true;  // Queda permanente en el puerto
		}	
	}	
	
	// El grado de realismo puede ser aún mayor si, al sobrepasar la intensidad durante cierto tiempo, y por
	// no poder evacuar el calor, la bobina se rompe. Entonces el modelo matemático cambia notablemente. 
	// La corriente, sea cual sea el voltaje, se hace nula. Se deja de suministrar par motor, y el motor
	// termina a merced del par resistente, pasando a actuar este como par motor; y el motor, se vuelve elemento pasivo
	// En la matriz del sistema, la fila de la variable de estado de corriente se anula totalmente. Igualmente, el
	// término independiente de la fila también se anula. Pasamos a tener dos variables de estado 
	// y prescindimos del voltaje como variable de contorno. 
	
	
	return resultado;
	
}

void MotorCC::rutina() { // Rutina de thread 
	
	while (Dispositivo::obtiene_ejecucion()) {
		
		MotorCC::actualiza();
		
		// Actualiza E/S del dispositivo siempre que esté en estado de ejecución
		
	}
	return; 
}